{
  easySDL is the main component of the easySDL Framework and allows to create easily a sdl window and handle
  events, syncronisation. It is not supposed to replace SDL itself, it is just a framework to create easily
  a application right out of the box. Other components of easySDL are avaible:
  @unorderedList(
    @item(easySDLTextures     for texture loading)
  )


  @author(Florian Sievert (Phobeus@DelphiGL.com))
  @author(Florian Koeberle )
}
unit easySDL;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the dglsdk for linux
  Visit us @ http://www.delphigl.com
  Maintained by Florian Sievert (Phobeus)
  Phobeus@DelphiGL.com
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2005 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications,
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original
  software. If you use this software in a product, an acknowledgment in the product documentation would be
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.


  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2005-12-31 Constructor for icon added (FS)
  2005-12-28 Input Handling improed (FS)
  2005-09-20 FPS added (JS)
  2005-08-24 Created (FS & FK)


  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors.

*******************************************************************************)

interface

uses
  SysUtils,
  dglOpenGL,
  SDL,
  SDL_Image;


// Tritt ein Fehler beim Initalisieren von SDL oder OpenGL auf, wird die Exception geworfen.
type
  EEasySDLGraphicFail = Exception;

const
  // Breite des Fensters, sofern nicht anders angegeben
  DEFAULT_WINDOW_WIDTH   = 640;
  // Hoehe des Fensters, sofern nicht anders angegeben
  DEFAULT_WINDOW_HEIGHT  = 480;
  // Farbtiefe des Fensters, sofern nicht anders angegeben
  DEFAULT_WINDOW_BPP     = 32;

  // Caption des Fensters, falls keine andere uebergeben wurde
  DEFAULT_WINDOW_CAPTION  = 'www.delphigl.com';
  // Symbol des Fensters, falls keine anderes uebergeben wurde
  DEFAULT_WINDOW_ICON     = '../data/gfx/dgl_icon.png';

  WINDOW_FULLSCREEN = FALSE;
  WINDOW_RESIZE   = FALSE;

type
  TEasySDL  = class
  private
    surface: PSDL_Surface;
    videoInfo: TSDL_VideoInfo;
    videoFlags: Uint32;
    TimeAtLastUpdate: Uint32;

    // window
    fwindowCaption: String;
    fwindowIcon: String;

		fScreenWidth: Integer;
		fScreenHeight: Integer;

    Frames: Uint32;
    Framestarttick: Uint32;

    KeyState: array [0..SDLK_EURO] of Boolean;
    MouseState: array[0..5] of Boolean;

    // Fuehert die Initalisierung von SDL durch
    procedure initSDL;
    // Fuehert die Initalisierung von OpenGL durch
    procedure initGL;

    procedure handleEvents;
  protected
    done: Integer;


    { @abstract @name Ermoeglicht Timebased movement!

      Ueberschreiben sie in einem Nachfahren von @classname, diese Methode um
      auf die vergangene Zeit zu reagieren.

      Moechte man zum Beispiel ein Objekt mit konstanter Geschwindigkeit Bewegen so koennte man hier
      die Position um Geschwindigkeit mal Parameter Seconds erhoehen.

      Wenn der direkte Vorfahr einer Klasse @classname ist braucht inherited in @name nicht aufgerufen werden.

      @param Seconds Die vergangene Zeit in Sekunden.
    }
    procedure UpdateWorld(Seconds: TGLFloat); virtual;



    { @abstract @name Zeichnet die Szene.

      @name wird von Loop aufgerufen und soll die Szehne mit OpenGL Befehlen zeichnen.@br
      SDL_GL_SwapBuffers braucht nicht und soll auch nicht in @name aufgerufen werden.
    }
    procedure DrawScene; virtual; abstract;



	 // todo: depreciate
    { @abstract @name Behandelt Tastatur Ereignisse

      @name wird von handleEvent aufgerufen um auf ein Tastatur Ereignis, 
      wie etwa das druecken einer Taste, zu reagieren.

      Beim ueberschreiben dieser Methode ist drauf zu achten, das mit inherited die
      @name Methode des Vorfahrens aufgerufen wird.

      @param keysym beinhaltet Informationen ueber das Maustasten Ereignis
    }
    //procedure handleKey(keysym: TSDL_keysym); virtual;

    procedure handleKeyPressed(event: TSDL_KeyboardEvent); virtual;
    procedure handleKeyReleased(event: TSDL_KeyboardEvent); virtual;



	//todo: depreciate
    { @abstract @name Behandelt Maustasten Ereignisse

      @name wird von handleEvent aufgerufen um auf ein Maus Ereignis, 
      wie etwa das druecken einer Maustaste, zu reagieren.

      Beim ueberschreiben dieser Methode ist drauf zu achten, das mit inherited die
      @name Methode des Vorfahrens aufgerufen wird.

      @param event beinhaltet Informationen ueber das Maustasten Ereignis
    }
    //procedure handleMouse(event: TSDL_MouseButtonEvent); virtual;

    procedure handleMousePressed(event: TSDL_MouseButtonEvent); virtual;
    procedure handleMouseReleased(event: TSDL_MouseButtonEvent); virtual;
    procedure handleMouseMotion(event: TSDL_MouseMotionEvent); virtual;


    { @abstract @name Behandelt SDL Ereignisse

      @name behandelt SDL Ereignisse und ruft dazu unter anderem handleKey und handleMouse auf

      Beim ueberschreiben dieser Methode ist drauf zu achten, das mit inherited die
      @name Methode des Vorfahrens aufgerufen wird.
    }
    procedure handleEvent(var event: TSDL_Event); virtual;



    { @abstract @name Behandelt Groessen-Aenderungen des Fensters

      @name wird von handleEvent aufgerufen um auf eine
      groessen Aenderung des Fensters zu reagieren.

      Beim ueberschreiben dieser Methode ist drauf zu achten, das mit inherited die
      @name Methode des Vorfahrens aufgerufen wird. 
    }
    function handleResize(width: integer; height: integer): Boolean; virtual;
    


    // TODO: Evt. Methoden namen abaendern oder Methode ganz entfernen. Immerhin wird hier nichts gezeichnet.
    procedure Render;

  public
    // Die aktuelle Bilder pro Sekunde, die die Anwendung dargestellt hat.
    FPS: Single;

    // Erzeugt ein easySDL-Fenster
    constructor create(); overload;
    // Erzeugt ein easySDL-Fenster mit der uebergebenden Caption
    constructor create(const caption: String); overload;
    // Erzeugt ein easySDL-Fenster mit der uebergebenden Caption und Symbol
    constructor create(const caption: String; const icon: String); overload;

    // Die easySDL-Anwendung begibt sich in eine Loop und horcht nach auftretenden Ereignissen. Zuvor muss Start aufgerufen worden sein.
    procedure Loop;
    // Leitet die Initalisierung aller Schnittstellen ein
    procedure Start;
    // Gibt die verwendeten Ressourcen wieder frei
    procedure Stop;

    // Fragt ab, ob eine Taste gedrueckt ist. Als Parameter wird ein SDL-Tastenbezeichner erwartet.
    function keyDown(key: Integer):Boolean;

    function mouseDown(button: Integer):Boolean;


    // Wie keyDown, allerdings wird die Taste anschließend in den Status ungedrueckt versetzt.
    function keyDownOnce(key: Integer):Boolean;
    function mouseDownOnce(button:Integer): boolean;

    procedure setDisplay(width, height: Integer; fullscreen: Boolean);
    property ScreenWidth: Integer read fScreenWidth;
    property ScreenHeight: Integer read fScreenHeight;
  end;

implementation

constructor TEasySDL.create();
begin
  create(DEFAULT_WINDOW_CAPTION, DEFAULT_WINDOW_ICON);
end;

constructor TEasySDL.create(const caption: String);
begin
  create(caption, DEFAULT_WINDOW_ICON);
end;

constructor TEasySDL.create(const caption: String; const icon: String);
begin
  fwindowCaption := caption;
  fwindowIcon := icon;

  fScreenWidth := DEFAULT_WINDOW_WIDTH;
  fScreenHeight:= DEFAULT_WINDOW_HEIGHT;
end;

function TEasySDL.handleResize(width: integer; height: integer): Boolean;
begin
  result := false;

  // prevent a division by zero
  if ( height = 0 ) then height := 1;

  fScreenWidth := width;
  fScreenHeight := height;

  // refresh viewport
  glViewport( 0, 0, fScreenWidth, fScreenHeight);

  glMatrixMode( GL_PROJECTION );
    glLoadIdentity;
    gluPerspective( 45.0, (width / height), 0.1, 1000.0 );
  glMatrixMode( GL_MODELVIEW );

  // reset world matrix
  glLoadIdentity;
  result := true;
end;

(* depreciate
procedure TEasySDL.handleKey(keysym: TSDL_keysym);
begin

end;*)

procedure TEasySDL.handleKeyPressed(event: TSDL_KeyboardEvent);
begin
  case event.keysym.sym of
    SDLK_ESCAPE : done := -1;
  end;
end;

procedure TEasySDL.handleKeyReleased(event: TSDL_KeyboardEvent);
begin
end;
(*
procedure TEasySDL.handleMouse(event: TSDL_MouseButtonEvent);
begin
end;*)

procedure TEasySDL.handleMousePressed(event:TSDL_MouseButtonEvent);
begin
end;

procedure TEasySDL.handleMouseReleased(event:TSDL_MouseButtonEvent);
begin
end;

procedure TEasySDL.handleMouseMotion(event:TSDL_MouseMotionEvent);
begin
end;

procedure TEasySDL.handleEvent(var event : TSDL_Event);
begin
  case event.type_ of

    // leave application
    SDL_QUITEV :
    begin
      Done := -1;
    end;

    // key was pressed
    SDL_KEYDOWN :
    begin
      //handleKey( event.key.keysym );
      handleKeyPressed(event.key);
      if (event.key.keysym.sym >=0) and (event.key.keysym.sym <=SDLK_EURO) then
        KeyState[event.key.keysym.sym] := true;
    end;

    // key was released
    SDL_KEYUP :
    begin
      handleKeyReleased(event.key);
      if (event.key.keysym.sym >=0) and (event.key.keysym.sym <=SDLK_EURO) then
        KeyState[event.key.keysym.sym] := false;
    end;

    // mouse button was pressed
    SDL_MOUSEBUTTONDOWN:
    begin
      //handleMouse(event.button);
      handleMousePressed(event.button);
      if (event.button.button >=0) and (event.button.button <= 5) then
        MouseState[event.button.button] :=true;
    end;

    // mouse button was released
    SDL_MOUSEBUTTONUP:
    begin
       handleMouseReleased(event.button);
       if (event.button.button >=0) and (event.button.button <= 5) then
         MouseState[event.button.button] := false;
    end;

    SDL_MOUSEMOTION:
    begin
      handleMouseMotion(event.motion);	
    end;
    // window resized
    SDL_VIDEORESIZE :
    begin
      surface := SDL_SetVideoMode( event.resize.w, event.resize.h, DEFAULT_WINDOW_BPP, videoflags );

      if ( surface = nil ) then
      begin
        Writeln('Surface lost');
        Stop;
      end;
      handleResize( event.resize.w, event.resize.h );
    end;
  end;
end;


procedure TEasySDL.handleEvents;
var event : TSDL_Event;
begin
  // process events
  while ( SDL_PollEvent( @event ) = 1 ) do
  begin
    handleEvent(event);
  end;
end;

procedure TEasySDL.initSDL;
begin
  // Init SDL
  WriteLn('Init SDL...');
  if (SDL_Init(SDL_INIT_VIDEO) < 0) then raise EEasySDLGraphicFail.Create('Fail to init SDL:'+SDL_GetError);

  // detect graphic card
  videoInfo := SDL_GetVideoInfo^;
  if (@videoInfo = nil) then raise EEasySDLGraphicFail.Create('Unable to detect graphic card:'+SDL_GetError);

  // set flags
  videoFlags := SDL_OPENGL or SDL_DOUBLEBUF or SDL_HWPALETTE;

  if (videoInfo.hw_available <> 0) then
  begin
    WriteLn('Hardware Surface enabled...');
    videoFlags := videoFlags or SDL_HWSURFACE;
  end else
  begin
    WriteLn('Software Surface enabled...');
    videoFlags := videoFlags or SDL_SWSURFACE
  end;

  // hardware blit
  if (videoInfo.blit_hw <> 0) then
  begin
    WriteLn('Hardware support');
    videoFlags := videoFlags or SDL_HWACCEL;
  end else WriteLn('Software support');

  // set attributes
  SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
  SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
  SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

  // window caption
  SDL_WM_SetCaption(pchar(fwindowCaption), nil);
  SDL_WM_SetIcon(IMG_Load(pchar(fwindowIcon)), 0);

  // Init OpenGL surface
  surface := SDL_SetVideoMode(fScreenWidth, fScreenHeight, DEFAULT_WINDOW_BPP,videoflags);
  if not assigned(surface) then raise EEasySDLGraphicFail.Create('Fail to create surface:'+SDL_GetError);

  WriteLn('SDL done.');
end;

procedure TEasySDL.initGL;
begin
  // Init OpenGL
  WriteLn('Init OpenGL...');
  InitOpenGL;
  ReadExtensions;

  // Show information about opengl system
  //WriteLn('Extensions:  '+glGetString(GL_EXTENSIONS));
  //WriteLn('------');
  //WriteLn('Renderer: '+glGetString(GL_RENDERER));
  //WriteLn('Version: '+glGetString(GL_VERSION));
  //WriteLn('Vendor:  '+glGetString(GL_VENDOR));


  // some good start values
  glEnable(GL_TEXTURE_2D);                  // Aktiviert Texture Mapping
  glShadeModel(GL_SMOOTH);                  // Aktiviert weiches Shading
  glClearColor(0.0, 0.0, 0.0, 0.5);            // Bildschirm loechen (schwarz)
  glClearDepth(1.0);                      // Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);                  // Aktiviert Depth Testing
  glDepthFunc(GL_LEQUAL);                    // Bestimmt den Typ des Depth Testing
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);  // Qualitativ bessere Koordinaten Interpolation

  WriteLn('OpenGL done.');
end;

// Initialisiert SDL und OpenGL
procedure TEasySDL.Start;
begin
  WriteLn('.:. Init easySDL .:.');

  initSDL;
  initGL;

  handleResize(DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT);
end;

procedure TEasySDL.Stop;
begin
  WriteLn('Releasing easySDL');
end;

procedure TEasySDL.Loop;
var
  TimeNow:Uint32;
begin
  WriteLn('.:. easySDL Loop .:.');

  TimeAtLastUpdate := SDL_GetTicks();
  FrameStarttick := SDL_GetTicks;
  while ( Done <> -1 ) do
  begin
    handleEvents;
    TimeNow := SDL_GetTicks();

    if TimeNow >= TimeAtLastUpdate then
    begin
      UpdateWorld((TimeNow - TimeAtLastUpdate)/1000);
    end
    else
    begin
      // This code will only be only executed after about 32 days runtime
      UpdateWorld(((high(Uint32) - TimeNow) + TimeAtLastUpdate)/1000);
    end;
    TimeAtLastUpdate := TimeNow;
    DrawScene;
    Render;
  end;
end;

procedure TEasySDL.Render;
begin
  Inc(Frames);
  if SDL_GetTicks-FrameStarttick >= 500 then
  begin
    FPS:= Frames/(SDL_GetTicks-FrameStarttick)*1000;
    Frames:= 0;
    FrameStarttick:=SDL_GetTicks;
  end;
  SDL_GL_SwapBuffers;
end;

function TEasySDL.keyDown(key: Integer): Boolean;
begin
  result := false;
  if not((key >= 0) and (key <= SDLK_EURO)) then exit;

  result := KeyState[key];
end;

function TEasySDL.mouseDown(button: Integer):Boolean;
begin
  result := false;
  if not((button >= 0) and (button <= 5)) then exit;

  result := MouseState[button];
end;

function TEasySDL.keyDownOnce(key: Integer): Boolean;
begin
  result := false;
  if not((key >= 0) and (key <= SDLK_EURO)) then exit;

  result := KeyState[key];
  KeyState[key] := false;
end;

function TEasySDL.mouseDownOnce(button:Integer): boolean;
begin
  result := false;
  if not((button >= 0) and (button <= 5)) then exit;

  result := MouseState[button];
  MouseState[button] := false;
end;

procedure TEasySDL.UpdateWorld(Seconds: TGLFloat);
begin
  //Empty procedure. It could be used but needn't in the final program
end;

procedure TEasySDL.setDisplay(width, height: Integer; fullscreen: Boolean);
begin
	fScreenWidth := width;
	fScreenHeight := height;

	handleResize(width,height);
end;

end.
