unit easySDLFont;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Jonas Scholz (i0n0s)
  JonasScholz@web.de
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2005 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications,
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original
  software. If you use this software in a product, an acknowledgment in the product documentation would be
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  Hier brauchen wir nochmal ne gute Beschreibung ;)

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - SDL_TTF (http://www.libsdl.org/projects/SDL_ttf/ or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------

  2006-05-16 Fixed Bug, which cause a crash (JS)
  2006-05-04 Added ability to draw 3D fonts, will scale automatically based on fontsize (JS) 
  2006-04-25 Changed Drawcoordparameter from Cardinal to Integer (JS)
  2006-03-28 Changed TFont from record to class (JS)
  2006-03-22 Added faultmessages (JS)
  2006-03-15 Switched from solid to blended for anti aliasing (JS)
  2006-02-27 Fixed several bugs, italic fonts now works correct (JS)
  2006-01-25 Initial version (JS)


  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  want to say hello, feel free to write to the contributors:

  JS
    Jonas Scholz (i0n0s)
    email: JonasScholz@web.de

*******************************************************************************)

{
@author(Jonas Scholz <i0n0s@delphigl.com>)
}


interface

uses dglOpenGL, SDL, SysUtils;

type
  {Exception, die im Falle eine Fehlers ausgelst wird. @br
  Folgender Fehler knnen auftreffen: @br
  @unorderedList(
    @item Unable to find fontfile @br TTF-Datei konnte nicht gefunden werden.
    @item Unable to create font: +TTF_GetError @br Schrift konnte von SDL_TTF nicht erstellt werden.
    @item Fontsize to high @br Die gewnschte Schriftgrsse ist zu gross und bertritt die maximale Texturgrsse der Grafikkarte.
    @item Creating a surface failed: +SDL_GetError @br Das Erstellen einer SDL_Surface ist fehlgeschlagen.
    @item Can not init SDL_TTF: +TTF_GetError @br SDL_TTF konnte nicht initialisiert werden.
    @item Internal fault, font hasn't been registered @br @bold(Fehler sollte nie auftreten!) @br Eine Schrift (TFont) hat sich nicht bei Fontsystem registriert.
  )
  }
  EEasySDLFontFail = Exception;

  {Die an @link(TFontsystem) zu berreichende Schriftdefinition.}
  TFont = Class(TObject)
    protected
      fName   : String;
      fHeight : Byte;
      fColor  : TSDL_Color;
      fStyle  : Integer;
      procedure SetHeight(nHeight: Byte);
      procedure SetStyle(nStyle: Integer);
    public
      {Pfad und Dateiname der Schriftdatei}
      property Name: String Read fName;
      {Hhe des Textes, nach SDL_TTF. @br
      @bold(Dies ist nicht die Hhe des Textes in Pixeln!)@br
      Genauere Details: http://wiki.delphigl.com/index.php?title=TTF_GlyphMetrics}
      property Height: Byte Read fHeight Write SetHeight;
      {Farbe des Textes. Bei der Ausgabe des Textes wird dieses spter durch OpenGL eingefrbt}
      property Color: TSDL_Color Read fColor Write fColor;
      {Styl der Schriftart. Zulssige Typen sind:
      @unorderedList(
      @item TTF STYLE BOLD Der Text wird fett geschrieben
      @item TTF STYLE ITALIC Der Text wird kursiv geschrieben
      @item TTF STYLE UNDERLINE Der Text wird unterstrichen
      @item TTF STYLE NORMAL (=0) Normaler Text
      )}
      property Style: Integer Read fStyle Write SetStyle;

      constructor Create(tfile: String; tHeight: Byte; tColor: TSDL_Color; tStyle: Integer);
      procedure Draw(Text: String; x,y: Integer);
      procedure Draw3D(Text: String; x,y,z: Single);
      destructor Destroy; override;
  end;

  {Interner Speicherort fr die verschiedenen Schriften}
  TinternalFont = Class(TObject)
    protected
      Fontwidth      : Array[0..255] of Byte;
      FontBase       : Cardinal;
      Fonttexture    : GLUint;
      Width          : Byte;
      Height         : Byte;
      TextureWidth   : Cardinal;
      TextureHeight  : Cardinal;
    private
      FontName       : String;
      FontHeight     : Byte;
      FontStyle      : Integer;
    public
      constructor Create(nFont: TFont);
      procedure Draw(Text: String; x,y: Integer; r,g,b: Byte);
      procedure Draw3D(Text: String; x,y,z: Single; r,g,b: Byte);
      destructor Destroy; override;
  end;

  {Internes Fontsystem@br
  Verwaltet und erstellt die einzelnen Schriften. Wird durch TFont aufgerufen}
  TFontsystem = Class(TObject)
    protected
      Fonts : Array of TinternalFont;
    private
      {Hier durch wird eine Schrift in den Speicher geladen. Sollte vor dem ersten Zeichnen passieren}
      procedure RegisterFont(Font: TFont);
      {Hier wird eine schon geladene Schrift entsorgt}
      procedure UnRegisterFont(Font: TFont);
      {Gibt Text an der gewnschten Stelle aus. @br
      Der Orthomodus sollte vorher schon aktiv sein. @bold(Draw) ndert den Rendermodus nicht!}
      procedure Draw(Font: TFont; x,y: Integer; Text: String);
      procedure Draw3D(Font: TFont; x,y,z: Single; Text: String);
    public
      constructor Create;
      destructor Destroy; override;
  end;

{Wenn @true werden alte Schriften im Speicher behalten, falls sie nochmal benutzt werden sollten. @br
Man knnte theoretisch nur mit einer TFont arbeiten und hat keinen Geschwindigskeitsverlust.@br
Bei @false werden alte Schriften entsorgt und freigegeben. @br
Verbraucht weniger Speicher, dafr werden Schriften immer neuerstellt, auch wenn man schonmal eine alte hatte.}
var easySDLFont_SaveFonts: Boolean = False;

implementation

uses sdl_ttf, math, sdlutils;

var Fontsystem: TFontsystem;

//Erstellt aus einer Surface eine OpenGL-Textur.
function SDL_GL_LoadTexture(surface : PSDL_Surface) : GLuint;
var
	texture : GLuint;
begin
	glGenTextures(1, @texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, Surface.w, Surface.h, 0, GL_RGBA, GL_UNSIGNED_BYTE, surface.pixels);
	result := texture;
end;

//Eigene Blitroutine, da SDL_BlitSurface den Alphachannal verrechnet anstatt weiterzugeben.
function BlitSurface(src: PSDL_Surface; srcrect: PSDL_Rect; dst: PSDL_Surface; dstrect: PSDL_Rect): Integer;
var
  x,y: Integer;
  pixel: UInt32;
  pred, pgreen, pblue, palpha: PUInt8;
  red, green, blue, alpha: UInt8;
  delsrc, deldst: Boolean;
begin
  Result:=0;
  delsrc:=false;
  deldst:=false;
  if (src=nil) or (dst=nil) then
  begin
    Result:=-1;
    exit;
  end;
  if srcrect=nil then
  begin
    delsrc:=True;
    New(srcrect);
    srcrect.x:=0;
    srcrect.y:=0;
    srcrect.w:=src.w;
    srcrect.h:=src.h;
  end;
  if dstrect=nil then
  begin
    deldst:=True;
    New(dstrect);
    dstrect.x:=0;
    dstrect.y:=0;
    dstrect.w:=src.w;
    dstrect.h:=src.h;
  end;
  if (srcrect.x+srcrect.w>src.w) or (dstrect.x+dstrect.w>dst.w)
  or (srcrect.y+srcrect.h>src.h) or (dstrect.y+dstrect.h>dst.h) then
  begin
    Result:=-1;
    exit;
  end;
  if (srcrect.w>dstrect.w) or (srcrect.h>dstrect.h) then
  begin
    Result:=-1;
    exit;
  end;
  for y:=0 to srcrect.h-1 do
    for x:=0 to srcrect.w-1 do
    begin
      pixel:=SDL_GetPixel(src, x+srcrect.x, y+srcrect.y);

      pred:=@red;
      pgreen:=@green;
      pblue:=@blue;
      palpha:=@alpha;

      //read and swap color
      SDL_GetRGBA(pixel, src.format, pred, pgreen, pblue, palpha);
      pixel:=SDL_MapRGBA(dst.format, blue, green, red, alpha);

      SDL_PutPixel(dst, x+dstrect.x, y+dstrect.y, pixel);
    end;

  if delsrc then
    Dispose(srcrect);
  if deldst then
    Dispose(dstrect);
  Result:=0;
end;

//berprft TFonts auf Gleichheit.
//Auf den Farbwert wird NICHT berprft.
function Equal(aFont: TinternalFont; bFont: TFont): Boolean;
begin
  Result:=(aFont.FontName=bFont.Name) and (aFont.FontHeight=bFont.Height) and (aFont.FontStyle=bFont.Style);
end;

////////////////////////////////////////////////////////////////////////////////
// TFont                                                                      //
////////////////////////////////////////////////////////////////////////////////

constructor TFont.Create(tfile: String; tHeight: Byte; tColor: TSDL_Color; tStyle: Integer);
begin
  fName   := tfile;
  fHeight := tHeight;
  fColor  := tColor;
  fStyle  := tStyle;
  Fontsystem.RegisterFont(self);
end;


procedure TFont.SetHeight(nHeight: Byte);
begin
  if fHeight=nHeight then exit;
  if not easySDLFont_SaveFonts then Fontsystem.UnRegisterFont(self);
  fHeight:=nHeight;
  Fontsystem.RegisterFont(self);
end;


procedure TFont.SetStyle(nStyle: Integer);
begin
  if fStyle=nStyle then exit;
  if not easySDLFont_SaveFonts then Fontsystem.UnRegisterFont(self);
  fStyle:=nStyle;
  Fontsystem.RegisterFont(self);
end;


procedure TFont.Draw(Text: String; x,y: Integer);
begin
  Fontsystem.Draw(self, x, y, Text);
end;

procedure TFont.Draw3D(Text: String; x,y,z: Single);
begin
  Fontsystem.Draw3D(self, x, y, z, Text);
end;


destructor TFont.Destroy;
begin
  Fontsystem.UnRegisterFont(self);
  Inherited;
end;


////////////////////////////////////////////////////////////////////////////////
// TinternalFont                                                              //
////////////////////////////////////////////////////////////////////////////////

constructor TinternalFont.Create(nFont: TFont);
var pFont : PTTF_Font;
    i : Integer;
    x, y, advance, maxx, minx, maxw: Integer;
    surface: PSDL_Surface;
    charsurface: PSDL_Surface;
    Color: TSDL_Color;
    rmask, gmask, bmask, amask: Uint32;
    rect: SDL_Rect;
    str: String;
begin
  Inherited Create;

  FontName   := nFont.Name;
  FontHeight := nFont.Height;
  FontStyle  := nFont.Style;
  for i:=0 to 255 do
  begin
    Fontwidth[i]    :=0;
  end;
  FontBase      :=0;
  Fonttexture   :=0;
  Width         :=0;
  Height        :=0;
  TextureWidth  :=1;
  TextureHeight :=1;
  if not fileexists(FontName) then
  begin
    raise EEasySDLFontFail.Create('Unable to find fontfile '+Fontname);
    exit;
  end
  else
  begin
    //Einlesen der Schrift
  	pFont := TTF_OpenFont(PChar(FontName), FontHeight);
  	if (pFont = nil) then
  	begin
  		raise EEasySDLFontFail.Create('Unable to create font: '+TTF_GetError);
  		exit;
  	end;
    //Setzen der Styles der Schrift
    TTF_SetFontStyle(pFont, FontStyle);
    //Einlesen der Buchstabenbreite
    maxw:=0;
    for i := 0 to 255 do
    begin
      TTF_GlyphMetrics(pFont,i,minx,maxx,x,x,advance);
      Fontwidth[i]    := advance;
      if max(advance, maxx)>maxw then maxw:=max(advance, maxx);
    end;
    //'Berechnen' der Texturgrsse
    Width  :=maxw;
    Height :=TTF_FontHeight(pFont);
    i:=0;
    repeat
      Inc(i);
    until (Width*16)<=power(2,i);
    TextureWidth:=Trunc(power(2,i));
    i:=0;
    repeat
      Inc(i);
    until (Height*16)<=power(2,i);
    TextureHeight:=Trunc(power(2,i));
    //berprfung ob zu gross
    glGetIntegerv(GL_MAX_TEXTURE_SIZE,@i);
    if max(TextureWidth,TextureHeight)>i then
    begin
      raise EEasySDLFontFail.Create('Fontsize to high');
      exit;
    end;

    //Erstellen der Surface fr die Textur
    if SDL_BYTEORDER = SDL_BIG_ENDIAN then
    begin
      rmask:=$ff000000;
      gmask:=$00ff0000;
      bmask:=$0000ff00;
      amask:=$000000ff;
    end
    else
    begin
      rmask:=$000000ff;
      gmask:=$0000ff00;
      bmask:=$00ff0000;
      amask:=$ff000000;
    end;
    surface:=SDL_CreateRGBSurface(SDL_SWSURFACE, TextureWidth, TextureHeight, 32, rmask, gmask, bmask, amask);
    if (surface=nil) then
    begin
      raise EEasySDLFontFail.Create('Creating a surface failed: '+SDL_GetError);
      exit;
    end ;
    Color.r:=255;
    Color.g:=255;
    Color.b:=255;
    Color.unused:=255;
    //Erstellen der einzelnen Buchstaben
    for i:=1 to 255 do
    begin
      str:=chr(i);
      CharSurface:=TTF_RenderText_Blended(pFont, PChar(str), Color);
      //je nach TTF sind nicht alle Zeichen definiert.
      if CharSurface<>nil then
      begin
        rect.x:=(i mod 16)*Width;
        rect.y:=(i div 16)*Height;
        rect.w:=CharSurface.w;
        rect.h:=CharSurface.h;
        Blitsurface(CharSurface,nil,Surface,@rect);
        SDL_FreeSurface(CharSurface);
        CharSurface:=nil;
      end;
    end;
    //Erstellen der Textur
    Fonttexture:=SDL_GL_LoadTexture(Surface);
    //Freigeben des Bildes
    SDL_FreeSurface(Surface);
    //Freigeben der Schrift
    TTF_CloseFont(pFont);

    //Erstellen der Texturefont
    FontBase:= glGenLists(256);
    for y := 0 to 15 do
      for x := 0 to 15 do
      begin
        glNewList(FontBase+x+y*16, GL_COMPILE);
        glBegin(GL_QUADS);
          glTexCoord2f(x*Width/TextureWidth    , (y+1)*Height/TextureHeight);  glVertex3f(    0,     0, 0);
          glTexCoord2f((x+1)*Width/TextureWidth, (y+1)*Height/TextureHeight);  glVertex3f(Width,     0, 0);
          glTexCoord2f((x+1)*Width/TextureWidth, y*Height/TextureHeight);      glVertex3f(Width, Height, 0);
          glTexCoord2f(x*Width/TextureWidth    , y*Height/TextureHeight);      glVertex3f(    0, Height, 0);
        glEnd;
        glTranslatef(Fontwidth[x+y*16], 0, 0);
        glEndList;
      end;
  end;
end;


procedure TinternalFont.Draw(Text: String; x,y: Integer; r,g,b: Byte);
begin
  glPushMatrix; //Matrix speichern
  glTranslatef(x,y,0);
  glBindTexture(GL_TEXTURE_2D,fonttexture); //passende Textur einladen
  //Schwarzvorzeichenen mit Hilfe von Alphatest um Farbabweichungen zu vermeiden
  glEnable(GL_ALPHA_TEST);
  glAlphaFunc(GL_GREATER, 0.4);
  glColor3f(0,0,0);
  glPushmatrix;
    glListBase(FontBase); //passende Liste einlesen
    glCallLists(Length(Text), GL_UNSIGNED_BYTE, Pointer(Text)); //Buchstaben schreiben
  glPopmatrix;
  glDisable(GL_ALPHA_TEST);
  //Zeichnen der Schrift per Blending auf den vorbereiteten Grund
  glBlendFunc(GL_SRC_ALPHA, GL_ONE);
  glEnable(GL_BLEND);
  glColor3ub(r, g, b);  //Farben der Schrift setzen
    glListBase(FontBase); //passende Liste einlesen
    glCallLists(Length(Text), GL_UNSIGNED_BYTE, Pointer(Text)); //Buchstaben schreiben
  glDisable(GL_BLEND);
  glPopMatrix; //Matrix wieder herstellen
end;


procedure TinternalFont.Draw3D(Text: String; x,y,z: Single; r,g,b: Byte);
var i,j : byte;
    Matrix : array[0..15] of single;
begin
  glPushMatrix; //Matrix speichern
  glTranslatef(x,y,z);
  //delete rotation part - replace it with a 3x3 Identity Matrix
  glGetFloatv(GL_MODELVIEW_MATRIX, @Matrix); //get matrix
  for i := 0 to 2 do
    for j := 0 to 2 do
      if i=j then Matrix[i*4+j] := 1 else Matrix[i*4+j] := 0;
  glLoadMatrixf(@Matrix);
  glScalef(1/Width/10,1/Width/10,1/Width/10);
  glBindTexture(GL_TEXTURE_2D,fonttexture); //passende Textur einladen
  //Schwarzvorzeichenen mit Hilfe von Alphatest um Farbabweichungen zu vermeiden
  glEnable(GL_ALPHA_TEST);
  glAlphaFunc(GL_GREATER, 0.4);
  glColor3f(0,0,0);
  glPushmatrix;
    glListBase(FontBase); //passende Liste einlesen
    glCallLists(Length(Text), GL_UNSIGNED_BYTE, Pointer(Text)); //Buchstaben schreiben
  glPopmatrix;
  glDisable(GL_ALPHA_TEST);
  //Zeichnen der Schrift per Blending auf den vorbereiteten Grund
  glBlendFunc(GL_SRC_ALPHA, GL_ONE);
  glEnable(GL_BLEND);
  glColor3ub(r, g, b);  //Farben der Schrift setzen
    glListBase(FontBase); //passende Liste einlesen
    glCallLists(Length(Text), GL_UNSIGNED_BYTE, Pointer(Text)); //Buchstaben schreiben
  glDisable(GL_BLEND);
  glPopMatrix; //Matrix wieder herstellen
end;


destructor TinternalFont.Destroy;
begin
  glDeleteLists(Fontbase, 256);
  glDeleteTextures(1, @Fonttexture);
  Inherited;
end;


////////////////////////////////////////////////////////////////////////////////
// TFontsystem                                                                //
////////////////////////////////////////////////////////////////////////////////

constructor TFontsystem.Create;
begin
  Inherited;

  if ( TTF_Init < 0 ) then
	begin
		raise EEasySDLFontFail.Create('Can not init SDL_TTF: '+TTF_GetError);
    exit;
	end;
end;


destructor TFontsystem.Destroy;
var i: Integer;
begin
  for i:=0 to High(Fonts) do
    Fonts[i].Free;
  Inherited;
end;


procedure TFontsystem.RegisterFont(Font: TFont);
var i: Integer;
    b: Boolean;
begin
  //Schauen ob die Schrift schon registriert wurde
  b:=false;
  for i:=0 to High(Fonts) do
    if Equal(Fonts[i],Font) then
    begin
      b:=true;
      break;
    end;
  if not b then
  begin
    //Erstellen der Schrift
    SetLength(Fonts,High(Fonts)+2);
    Fonts[High(Fonts)]:=TinternalFont.Create(Font);
  end;
end;

procedure TFontsystem.UnRegisterFont(Font: TFont);
var i,j: Integer;
    b: Boolean;
begin
  //Schauen ob die Schrift schon registriert wurde
  b:=false;
  for i:=0 to High(Fonts) do
    if Equal(Fonts[i],Font) then
    begin
      b:=true;
      break;
    end;
  if b then
  begin
    Fonts[i].Free;
    for j:=i+1 to High(Fonts) do
      Fonts[j-1]:=Fonts[j];
    SetLength(Fonts,High(Fonts));
  end;
end;


procedure TFontsystem.Draw(Font: TFont; x,y: Integer; Text: String);
var i: Integer;
    b: Boolean;
begin
  b:=false;
  //berprfen ob die Schrift schon geladen wurde
  for i:=0 to High(Fonts) do
    if Equal(Fonts[i],Font) then
    begin
      b:=true;
      break;
    end;
  if not b then
  begin
		raise EEasySDLFontFail.Create('Internal fault, font hasn''t been registered');
    exit;
  end
  else
    Fonts[i].Draw(Text, x, y, Font.Color.r, Font.Color.g, Font.Color.b);
end;


procedure TFontsystem.Draw3D(Font: TFont; x,y,z: Single; Text: String);
var i: Integer;
    b: Boolean;
begin
  b:=false;
  //berprfen ob die Schrift schon geladen wurde
  for i:=0 to High(Fonts) do
    if Equal(Fonts[i],Font) then
    begin
      b:=true;
      break;
    end;
  if not b then
  begin
		raise EEasySDLFontFail.Create('Internal fault, font hasn''t been registered');
    exit;
  end
  else
    Fonts[i].Draw3D(Text, x, y, z, Font.Color.r, Font.Color.g, Font.Color.b);
end;


initialization
  Fontsystem := TFontsystem.Create;
finalization
  Fontsystem.Free;

end.
