{Texturloader fr Windows und Linux}
unit easySDLTextures;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}

//todo: Texturen mit Breiten/Hhen ungleich 2^n (JS)
//todo: Setzen der Filteroptionen (JS)

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Jonas Scholz (i0n0s)
  JonasScholz@web.de
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2005 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications,
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original
  software. If you use this software in a product, an acknowledgment in the product documentation would be
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  Hier brauchen wir nochmal ne gute Beschreibung ;)

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - SDL_Image 1.2 (http://www.libsdl.org/projects/SDL_image/ or yourd distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2006-01-23 Minor changes for better use. (JS)
  2005-09-20 Added PasDoc documentation. (JS)
  2005-09-11 Warns when using \ (JS)
  2005-09-10 Added support vor BGR for user with OpenGL<1.2 (JS)
             Added texturcompression
  2005-09-09 Path created cointains only /, nomore \ (JS)
  2005-09-07 Fixed creating absolute path (JS)
  2005-09-06 Fixed some minor bugs (JS)
  2005-09-06 Added some minor improvements (FS)
  2005-09-02 Initial version (JS)


  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  want to say hello, feel free to write to the contributors:

  FS
    Florian Sievert (Phobeus)
    email: Phobeus@DelphiGL.com
    Visit: http://www.delphigl.com

  JS
    Jonas Scholz (i0n0s)
    email: JonasScholz@web.de

*******************************************************************************)

interface

uses
  SysUtils,
  Classes,

  dglOpenGL,

  sdl,
  sdlutils,
  sdl_image;

type
  {Exception, die im Falle eine Fehlers ausgelst wird. @br
  Folgender Fehler knnen auftreffen: @br
  @unorderedList(
    @item Unable to bind Texture ID: @br Texturid ist unbekannt.
    @item Texture name already in use: '@br Textur wurde schonmal geladen oder der Name wurde schonmal vergeben.
    @item File not found:
    @item Unable to find texture with the name: @br Textur konnte unter dem angegebenen Pfad nicht gefunden werden.
    @item Cannot read file: @br Datei konnte von SDL nicht gelesen werden.
    @item Cannot close file: @br Datei konnte von SDL nicht geschlossen werden.
    @item Unsupported colordepth: @br Farbtiefe der Textur wird nicht untersttzt.
    @item Image error: @br Fehlermeldung von SDL_Image.
  )
  }
  EEasySDLTextureFail = Exception;

  {Texturmanager, dieser wird automatisch beim Programmstart erstellt. @br
  Per Variable @bold(Textures) kann auf ihm zugegriffen werden}
  TEasySDLTextures = class
	private
    fNames                : TStringList;
    fTexture_Id           : Array of GLUInt;
    fApplicationDirectory : String;
    fUseAppDirectory      : Boolean;
    fCompression          : GLint;
    fuseCompression       : Boolean;

    procedure Flip(Surface: PSDL_Surface);
    procedure SwapBGR(Surface: PSDL_Surface);
    procedure Convert8Bitto32Bit(var Surface: PSDL_Surface);
    function CreateTexture(path: String): GLUInt;
    function GetTextureID(Name: String): GLUInt;

    procedure SetCompression(nCompression: GLint);
    procedure SetUseCompression(use: Boolean);
  public
    constructor Create;
    {Bindet an Textur in GL_TEXTURE_2D.@br
    @bold(Name) ist der Name, die die Textur bekommen hat.@br
    Wenn nicht vergeben wurde, ist es der Dateipfad}
    procedure Bind(Name: String); overload;
    {Ldt eine Textur. @br
    Untersttze Formate: @br
    @unorderedList(
      @item BMP
      @item GIF
      @item JPEG @italic (Bentigt die JPEG-Bibliothek: http://www.ijg.org/)
      @item LBM
      @item PCX
      @item PNG  @italic (Bentigt die PNG- und die Zlib-Bibliothek: http://www.libpng.org/pub/png/libpng.html & http://www.gzip.org/zlib/)
      @item PNM
      @item TGA
      @item TIFF  (@italic Bentigt die Tiff-Bibliothek: ftp://ftp.sgi.com/graphics/tiff/)
      @item XPM
    )}
    function Load(Path: String): GLUInt; overload;
    {Siehe @link(TEasySDLTextures.Load).@br
    @bold(texture_name) ist ein beliebiger Bezeichner.
    }
    function Load(Path: String; texture_name: String): GLUInt; overload;
    {Erstellt aus relativen absolute Pfade.}
    function CorrectPath(oldPath: String): String;
    {Erlaubt das Lesen der OpenGL ID der Textur.}
    property Texture[Name: String]: GLUInt Read GetTextureID;
    {@true: Pfadangaben werden zu absoluten Pfaden korrigiert. (Default)@br
    @false: Pfadangaben werden einfach bernommen.}
    property UseApplicationDirectory: Boolean Read fUseAppDirectory Write fUseAppDirectory;
    {Definiert die Art der Texturkompression:@br
    Erlaubte Verfahren: @br
    @unorderedList(
    @item GL_COMPRESSED_RGB_ARB, GL_COMPRESSED_RGBA_ARB
    @item GL_COMPRESSED_RGBA_S3TC_DXT5_EXT, GL_COMPRESSED_RGBA_S3TC_DXT3_EXT, GL_COMPRESSED_RGBA_S3TC_DXT1_EXT
    )
    @italic(Hinweis: Bei allen Verfahren wird berprft ob diese untersttzt werden.)
    }
    property Compression: GLInt Read fCompression Write SetCompression;
    {@true: Texturkompression wird bei neuen Texturen angewandt.@br
    @italic(Hinweis: Vorher muss mit Compression ein Verfahren angegeben werden.)@br
    @false: Texturkompression ist deaktiviert.
    }
    property UseCompression: Boolean Read fuseCompression Write SetUseCompression;
end;

var
  {Eine automatisch erstellte Version des Texturmanagers. @br
   Ermglicht den Zugriff auch ausserhalb des Hauptprogrammes z.b. fr Modelloader}
  Textures: TEasySDLTextures;

implementation

////////////////////////////////////////////////////////////////////////////////
// constructor                                                                //
////////////////////////////////////////////////////////////////////////////////

constructor TEasySDLTextures.Create;
begin
  Inherited;
  fNames := TStringList.Create;
  SetLength(fTexture_ID, 0);
  fApplicationDirectory := ExtractFilePath(ParamStr(0));
  {$ifdef win32}
  //replace \ with /
  fApplicationDirectory:=StringReplace(fApplicationDirectory,'\','/',[rfReplaceAll]);
  {$endif}

  fUseAppDirectory      := true;
  fuseCompression       := false;
  fCompression          := 0;
end;


////////////////////////////////////////////////////////////////////////////////
// public                                                                     //
////////////////////////////////////////////////////////////////////////////////

procedure TEasySDLTextures.Bind(Name: String);
var
  i: Integer;
begin
  i:=fNames.IndexOf(Name);
  if (i<>-1) then
  begin
    glBindTexture(GL_TEXTURE_2D, fTexture_ID[i]);
  end
  else
  begin
    raise EEasySDLTextureFail.Create('Unable to find texture with the name '+name);
  end;
end;

function TEasySDLTextures.Load(path: String; texture_name: String): glUInt;
begin
  Result:=0;
  //checking if texturename is alread in use
  if fNames.IndexOf(texture_name)<>-1 then
  begin //texture_name already in use
    raise EEasySDLTextureFail.Create('Texture name already in use: '+texture_name);
    exit;
  end;

  //correcting path for linux user if not started per console
  //if path contains a ../ it is definitly not absolute
  if fUseAppDirectory then path:=CorrectPath(path);

  if FileExists(path) then
  begin
    SetLength(fTexture_ID, High(fTexture_ID)+2);
    //creating texture
    fTexture_ID[High(fTexture_ID)]:=CreateTexture(path);
    Result:=fTexture_ID[High(fTexture_ID)];
    if (Result<>0) then
    begin
      //stores texture name
      fNames.Add(texture_name)
    end
    else
    begin
      //removes unused GLUint
      SetLength(fTexture_ID, High(fTexture_ID));
    end;
  end
  else
  raise EEasySDLTextureFail.Create('File not found: '+path+'('+texture_name+')');
end;


function TEasySDLTextures.Load(path: String): GLUint;
begin
  Result := Load(path, path);
end;


//makes relative to absolut paths
function TEasySDLTextures.CorrectPath(oldPath: String): String;
const SLASH='/';

  //if Path given is using ../ these are counted and cut off
  function CutPath(var Path: String): Integer;
  begin
    if AnsiCompareText(Copy(Path,1,3),'..'+SLASH)=0 then
    begin
      Delete(Path,1,3);
      Result:=1+CutPath(Path);
    end
    else Result:=0;
  end;

  //own ReverseString for not including strutils
  function ReverseString(const oldstring: String): String;
  var i,j: Integer;
  begin
    j:=Length(oldstring);
    SetLength(Result,j);
    for i:=0 to j do
    begin
      Result[j-i+1]:=oldstring[i];
    end;
  end;

var i,j,k: Integer;
    apppath: String;
begin
  if Pos('\',oldPath)<>0 then
  begin
    Writeln('Illegal character \ in: '+oldpath);
    oldPath:=StringReplace(oldPath,'\','/',[rfReplaceAll]);
  end;
  i:=CutPath(oldPath);
  //can use ApplicationDirectory?
  if i>0 then
  begin
    //reversing String to cut off Directory not wanted by User
    apppath:=ReverseString(fApplicationDirectory);
    for j:=0 to i do
    begin
      k:=Pos(SLASH,apppath);
      apppath:=Copy(apppath,k+1,Length(apppath)-k);
    end;
    //SLASH is added because cut off before
    Result:=ReverseString(apppath)+SLASH+oldPath;
  end
  else
  Result:=fApplicationDirectory+oldPath;
end;


////////////////////////////////////////////////////////////////////////////////
// private                                                                    //
////////////////////////////////////////////////////////////////////////////////

function TEasySDLTextures.GetTextureID(Name: String): GLUInt;
var
  i: Integer;
begin
  result:=0;
  i:=fNames.IndexOf(Name);
  if (i<>-1) then
  begin
    Result:=fTexture_ID[i];
  end
  else
  begin
    raise EEasySDLTextureFail.Create('Unable to find texture with the name '+name);
  end;
end;


procedure TEasySDLTextures.SetCompression(nCompression: GLint);
begin
 case nCompression of
  GL_COMPRESSED_RGB_ARB,
  GL_COMPRESSED_RGBA_ARB: if GL_VERSION_1_3 or GL_ARB_texture_compression
                          then fCompression:= GL_COMPRESSED_RGB_ARB
                          else Writeln('Generell Texturcompression is not supported');

  GL_COMPRESSED_RGBA_S3TC_DXT5_EXT,
  GL_COMPRESSED_RGBA_S3TC_DXT3_EXT,
  GL_COMPRESSED_RGBA_S3TC_DXT1_EXT: if GL_EXT_texture_compression_s3tc
                                    then fCompression:= nCompression
                                    else Writeln('S3TC is not supported');
  else Writeln('Unknown format: '+InttoStr(nCompression));
  end;
end;


procedure TEasySDLTextures.SetUseCompression(use: Boolean);
begin
  if use then
  begin
    if GL_VERSION_1_3 or GL_ARB_texture_compression or GL_EXT_texture_compression_s3tc
    then fUseCompression:=true
    else Writeln('Texturcompression is not supported');
  end
  else fUseCompression:=false;
end;


//flips a surface
procedure TEasySDLTextures.Flip(Surface: PSDL_Surface);
var 
  x,y: Integer;
  a,b: UInt32;
begin
  for y:=0 to ((surface.h-1) div 2) do
    for x:=0 to surface.w-1 do
    begin
      //reads the pixels and swaps them
      a:=SDL_GetPixel(Surface, x, y);
      b:=SDL_GetPixel(Surface, x, surface.h-1-y);
      SDL_PutPixel(Surface, x, y, b);
      SDL_PutPixel(Surface, x, surface.h-1-y, a);
    end;
end;


procedure TEasySDLTextures.SwapBGR(Surface: PSDL_Surface);
var
  x,y: Integer;
  pixel: UInt32;
  pred, pgreen, pblue, palpha: PUInt8;
  red, green, blue, alpha: UInt8;
begin
  for y:=0 to (Surface.h-1) do
    for x:=0 to (Surface.w-1) do
    begin
      //reads the pixels them
      pixel:=SDL_GetPixel(Surface, x, y);

      pred:=@red;
      pgreen:=@green;
      pblue:=@blue;
      palpha:=@alpha;

      //read and swap color
      SDL_GetRGBA(pixel,Surface.format, pred, pgreen, pblue, palpha);
      pixel:=SDL_MapRGBA(Surface.format, blue, green, red, alpha);

      SDL_PutPixel(Surface, x, y, pixel);
    end;
end;


procedure TEasySDLTextures.Convert8Bitto32Bit(var Surface: PSDL_Surface);
var fmt: PSDL_PixelFormat;
    temp: PSDL_Surface;
begin
  if Surface<>nil then
  begin
    New(fmt);
    fmt.palette:=nil;
    fmt.BytesPerPixel:=4;
    fmt.BitsPerPixel:=32;
    fmt.colorkey:=0;
    fmt.alpha:=255;
    if (SDL_BYTEORDER = SDL_BIG_ENDIAN) then
    begin
      fmt.RMask:=$ff000000;
      fmt.GMask:=$00ff0000;
      fmt.BMask:=$0000ff00;
      fmt.AMask:=$000000ff;
    end
    else
    begin
      fmt.RMask:=$000000ff;
      fmt.GMask:=$0000ff00;
      fmt.BMask:=$00ff0000;
      fmt.AMask:=$ff000000;
    end;
    temp:=SDL_ConvertSurface(Surface,fmt,0);
    SDL_FreeSurface(Surface);
    Surface:=temp;
  end;
end;


function TEasySDLTextures.CreateTexture(path: String): GLUint;
var
  BGR         : Boolean;
  rwop        : PSDL_RWops;
  tex_surface : PSDL_Surface;
  texture     : GLUint;
  format      : GLenum;
  bpp         : GLint;
begin
  Result:=0;
  BGR:=false;

  //checking for BGR
    //testing for TGA
  if copy(Uppercase(path), length(path)-3, 4) = '.TGA' then
  begin
    BGR:=True
  end
  else
  begin
    //testing for BMP, using SDL_Image
    rwop:=SDL_RWFromFile(PChar(path), 'rb');

    if rwop<>nil then
    begin
      if Img_isBMP(rwop)<>0 then BGR:=True;
    end
    else
    begin
      raise EEasySDLTextureFail.Create('Cannot read file: '+path);
      exit;
    end;

    if SDL_RWClose(rwop)<>0 then
    begin
      raise EEasySDLTextureFail.Create('Cannot close file: '+path);
      exit;
    end;
  end;

  //loading image into surface
  tex_surface := IMG_Load(PChar(path));
  if assigned(tex_surface) then
  begin
    //flips surface
    Flip(tex_surface);

    glGenTextures(1, @texture);
    glBindTexture(GL_TEXTURE_2D, texture);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR{GL_LINEAR});
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    //glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    //glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

    //BGR is supported form 1.2, lower version need to swap manuall
    if (not GL_VERSION_1_2) and (BGR)
    and (tex_surface.format.BytesPerPixel>2) then
    begin
      SwapBGR(tex_surface);
      BGR:=False;
    end;

    //glTexImage2D is depending on bpp and rgb
    case tex_surface.format.BytesPerPixel of
      1:
      begin
        //8 bit format in SDL is not supported by OpenGL, so create a new 24 bit surface
        Convert8Bitto32Bit(tex_surface);
        format:=GL_RGBA;
      end;
      3: if BGR then format:=GL_BGR else format:=GL_RGB;
      4: if BGR then format:=GL_BGRA else format:=GL_RGBA;
      else
      begin
        raise EEasySDLTextureFail.Create('Unsupported colordepth: '+path);
        exit;
      end;
    end;

    //should texture get compressed?
    if not fuseCompression then bpp:=tex_surface.format.BytesPerPixel
    else
      if fCompression<>0 then
      begin
        if fCompression=GL_COMPRESSED_RGB_ARB then
          case format of
            GL_RGB, GL_BGR:   bpp:=GL_COMPRESSED_RGB_ARB;
            GL_RGBA, GL_BGRA: bpp:=GL_COMPRESSED_RGBA_ARB;
          end
        else bpp:=fCompression;
      end;

    //glTexImage2D(GL_TEXTURE_2D, 0, tex_surface.format.BytesPerPixel,
    //             tex_surface^.w, tex_surface^.h, 0, format, GL_UNSIGNED_BYTE, tex_surface^.pixels);

    if gluBuild2DMipmaps(GL_TEXTURE_2D, bpp , tex_surface^.w, tex_surface^.h,
                         format, GL_UNSIGNED_BYTE, tex_surface^.pixels)<>0
    then raise EEasySDLTextureFail.Create('gluBuild2DMipmaps failed');

    //frees surface, texture is stored intern by OpenGL, surface no longer required
    SDL_FreeSurface(tex_surface);
    Result:=texture;
  end
  else
  begin
    raise EEasySDLTextureFail.Create('Image error: '+IMG_GetError);
  end;
end;

initialization
  textures :=  TEasySDLTextures.Create;
finalization
  textures.free;

end.
