program boundingsphere;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE CONSOLE}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Florian Sievert (Phobeus)
  Phobeus@DelphiGL.com
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2006 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
	This sample demonstrate how to realize a collision detection using a bound sphere. Move the second sphere toward the static sphere
	in the center. If a hit is detected the color of the second sphere will become red. Note, that in this example you cannot move the
	sphere on the z-axis, however the detection will also work in a real 3d world.

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2006-01-21 Created (FS)

  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 


  FS
    Florian Sievert (Phobeus)
    email : Phobeus@DelphiGL.com
    Visit: http://www.delphigl.com

*******************************************************************************)

uses 
	dglOpenGL,

	SDL,
	easySDL,

	Geometry;

{$I ../dglsdk.inc}

type
  TMyProgram = class(TEasySDL)
		q: PGluQuadric;
		sphere1, sphere2: TVector3f; // Positions
		procedure Init;
    procedure DrawScene;override;
		procedure renderAxis;
		procedure UpdateWorld(Seconds: TGLFloat); override;
		function testSphere(pos1, pos2 : TVector3f; r1,r2: Single): Boolean;
  end;

var
  MyProgram :TMyProgram;

// setup material
const
	mat_ambient_red		: Array[0..3] of GlFloat = (1.0, 0.0, 0.0, 1.0);
	mat_ambient_green	: Array[0..3] of GlFloat = (0.0, 1.0, 0.0, 1.0);
	mat_ambient_blue	: Array[0..3] of GlFloat = (0.0, 0.0, 1.0, 1.0);


procedure TMyProgram.UpdateWorld(Seconds: TGLFloat);
begin
	if KeyDown(SDLK_LEFT) then sphere2[0] := sphere2[0] - 3*seconds;
	if KeyDown(SDLK_RIGHT) then sphere2[0] := sphere2[0] + 3*seconds;
	if KeyDown(SDLK_DOWN) then sphere2[1] := sphere2[1] - 3*seconds;
	if KeyDown(SDLK_UP) then sphere2[1] := sphere2[1] + 3*seconds;
end;

function TMyProgram.testSphere(pos1, pos2 : TVector3f; r1,r2: Single): Boolean;
var 
	mindistance, distance: single;
	relPos: TVector4f;	
	obj1, obj2 : TVector4f;
begin
	result := false;
	
	// TVector3f to TVector4f
	obj1[0] := pos1[0];
	obj1[1] := pos1[1];
	obj1[2] := pos1[2];
	obj1[3] := 0;
	
	obj2[0] := pos2[0];
	obj2[1] := pos2[1];
	obj2[2] := pos2[2];
	obj2[3] := 0;

	relPos := VectorSubtract(Obj1,Obj2);
	distance := sqr(relPos[0])+sqr(relPos[1])+sqr(relPos[2]);	
	mindistance := r1+r2;	
	result := (distance<=sqr(minDistance));
end;

procedure TMyProgram.renderAxis;
begin
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);
	glLineWidth(3);

	glBegin(GL_LINES);
		// x-axis
		glColor3f(1,0,0);
		glVertex3f(0,0,0);
		glVertex3f(100,0,0);

		glColor3f(0.5,0,0);
		glVertex3f(-100,0,0);
		glVertex3f(0,0,0);

		// y-axis
		glColor3f(0,1,0);
		glVertex3f(0,0,0);
		glVertex3f(0,100,0);

		glColor3f(0,0.5,0);
		glVertex3f(0,-100,0);
		glVertex3f(0,0,0);

	glEnd;

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glLineWidth(1);
	glColor3f(1,1,1);
end;

procedure TMyProgram.DrawScene;
begin
	glClearColor(0.0,0.0,0.0,1);
	glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );

	glLoadIdentity();	
	glTranslatef( 0.0, 0.0, -10);
	glDisable(GL_LIGHTING);
	renderAxis;

	// draw static sphere
	glEnable(GL_LIGHTING);
	glMaterialfv(GL_FRONT, GL_AMBIENT,   @mat_ambient_blue[0]);
	gluSphere(q,1,40,40);

	// test bound spheres
	if testSphere(sphere1,sphere2,1,1) then glMaterialfv(GL_FRONT, GL_AMBIENT,   @mat_ambient_red[0])
	else glMaterialfv(GL_FRONT, GL_AMBIENT,   @mat_ambient_green[0]);

	// draw moveable sphere
	glTranslatef(sphere2[0],sphere2[1],0);
	gluSphere(q,1,40,40);
end;

procedure TMyProgram.Init;
const
	mat_specular   : Array[0..3] of GlFloat = (1.0, 1.0, 1.0, 1.0);
	mat_shininess  : Array[0..0] of GlFloat = (50.0);
	mat_diffuse    : Array[0..3] of GlFloat = (0.4, 0.8, 0.4, 1.0);
 
	light_position : Array[0..3] of GlFloat = (0.0, 3.0, 0.0, 1.0);
	light_ambient  : Array[0..3] of GlFloat = (0.8, 0.8, 0.8, 1.0);
	light_diffuse  : Array[0..3] of GlFloat = (0.8, 0.8, 0.8, 1.0);
begin
	// sphere 1 is in the center
	sphere1[0] := 0;
	sphere1[1] := 0;
	sphere1[2] := 0;

	// sphere 2 starts next to sphere 1 on the right
	sphere2[0] := 3;
	sphere2[1] := 0;
	sphere2[2] := 0;

	// setup light
	glMaterialfv(GL_FRONT, GL_SPECULAR,  @mat_specular[0]);
	glMaterialfv(GL_FRONT, GL_SHININESS, @mat_shininess[0]);
	glMaterialfv(GL_FRONT, GL_DIFFUSE,   @mat_diffuse[0]);
	
	glLightfv(GL_LIGHT0, GL_AMBIENT,  @light_ambient[0]);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,  @light_diffuse[0]);
	glLightfv(GL_LIGHT0, GL_POSITION, @light_position[0]);

	glEnable(GL_LIGHT0);
	glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
	glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.001);
	glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.004);
	
	// misc init
	q := gluNewQuadric();
	gluQuadricNormals(q,GLU_SMOOTH);
end;

begin
	MyProgram := TMyProgram.Create('www.delphigl.com .:. Sphere Test');
	MyProgram.Start;
	MyProgram.Init;

	MyProgram.Loop;
	MyProgram.Stop;
end.