program heightmap;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE CONSOLE}
{$ENDIF}

//todo: first version, please check

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Florian Sievert (Phobeus)
  Phobeus@DelphiGL.com
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2006 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
	This sample combines several technologies in one scene: heightmaps, vertex buffer objects, linear interpolation (for height)

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2006-03-28 Minor change for easySDL (JS)
  2006-01-21 Created (FS)

  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 


  FS
    Florian Sievert (Phobeus)
    email : Phobeus@DelphiGL.com
    Visit: http://www.delphigl.com

*******************************************************************************)

uses 
	dglOpenGL,

	SysUtils,
	SDL,
	easySDL,
	easySDLTextures,

	Geometry,
	map;

{$I ../dglsdk.inc}

type
	TMyProgram = class(TEasySDL)
	private
		TM: TEasySDLTextures;
		hmap: THeightMap;

		q: PGluQuadric;
		typ: Byte;
		obj1, obj2: TVector3f; // Positions
		procedure Init;
    procedure DrawScene;override;
		procedure renderAxis;
		procedure UpdateWorld(Seconds: TGLFloat); override;
		function getPlayerHeight:Single;
  end;

var
	MyProgram :TMyProgram;
var
	rot: Single;
	camx, camy: Single;
	px, py: Single;

procedure TMyProgram.UpdateWorld(Seconds: TGLFloat);
begin
	//rot := rot + seconds*60;
	if KeyDown(SDLK_LEFT) then camx := camx - 3*seconds;
	if KeyDown(SDLK_RIGHT) then camx := camx + 3*seconds;
	if KeyDown(SDLK_DOWN) then camy := camy - 3*seconds;
	if KeyDown(SDLK_UP) then camy := camy + 3*seconds;

	if keyDown(SDLK_A) then px := px - 3*seconds;
	if keyDown(SDLK_D) then px := px + 3*seconds;
	if keyDown(SDLK_W) then py := py + 3*seconds;
	if keyDown(SDLK_S) then py := py - 3*seconds;

	if KeyDown(SDLK_R) then rot :=  rot + Seconds*60;
	if KeyDown(SDLK_F) then rot :=  rot - Seconds*60;

	if KeyDownOnce(SDLK_SPACE) then writeLn(Formatfloat('0.000',camy));

	// border check
	if px < 0 then px := 0;
	if py < 0 then py := 0;
	if px > 10 then px := 10;
	if py > 10 then py := 10;
end;

procedure TMyProgram.renderAxis;
begin
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);
	glLineWidth(3);

	glBegin(GL_LINES);
		// x-axis
		glColor3f(1,0,0);
		glVertex3f(0,0,0);
		glVertex3f(100,0,0);

		glColor3f(0.5,0,0);
		glVertex3f(-100,0,0);
		glVertex3f(0,0,0);

		// y-axis
		glColor3f(0,1,0);
		glVertex3f(0,0,0);
		glVertex3f(0,100,0);

		glColor3f(0,0.5,0);
		glVertex3f(0,-100,0);
		glVertex3f(0,0,0);
	glEnd;

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glLineWidth(1);
	glColor3f(1,1,1);
end;

function TMyProgram.getPlayerHeight:Single;
var
	x,y: Integer;
	v1,v2: Single;

	h1,h2,h3,h4: Single;
begin
	x := trunc(px);
	y := trunc(py);

	with hmap do
	begin
		h1 := height[x,y];
		h2 := height[x+1,y];
		h3 := height[x,y+1];
		h4 := height[x+1,y+1];

		v1 := h1*(1-frac(px))+h2*frac(px);
		v2 := h3*(1-frac(px))+h4*frac(px);		
	end;

	result := 0.2+v1*(1-frac(py))+v2*frac(py);
end;

procedure TMyProgram.DrawScene;
begin
	glClearColor(0.0,0.0,0.0,1);
	glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	gluLookat(camx,camy,5,
				camx,camy,0,
				0,1,0);

	glRotatef(rot,1,0,0);

	renderAxis;
	
	TM.bind('mars');
	hmap.render;

	TM.bind('wiki');
	glTranslatef(px,py,getPlayerHeight);
	gluSphere(q,0.2,10,10);
end;

procedure TMyProgram.Init;
begin
	hmap := THeightmap.Create;

	q := gluNewQuadric();
	gluQuadricTexture(q,true);
	gluQuadricNormals(q,GLU_SMOOTH);

	TM := TEasySDLTextures.Create;
	TM.load(DGLSDK_DATA+'gfx/venus.jpg','mars');
	TM.load(DGLSDK_DATA+'gfx/wiki.jpg','wiki');

	rot := -70;
	camx := 2.5;
	camy := 2.0;
end;

begin
	MyProgram := TMyProgram.Create('www.delphigl.com .:. Heightmap');
	MyProgram.Start;
	MyProgram.Init;

	MyProgram.Loop;
	MyProgram.Stop;
end.