unit map;

interface

uses
	dglOpenGL;

type
	PVertex = ^TVertex;
	TVertex = packed record
 		S,T,X,Y,Z : TGLFloat;
end;

type
	THeightmap = class
	private
		vbo: gluInt;
		VBOPointer: PVertex;
		procedure addPoint(x,y,z,s,t:glFloat);
	public
		height:array[0..10,0..10] of Single;
		constructor create;
		procedure render;
end;


implementation	

constructor THeightmap.create;
var
	x,y: Integer;
begin
	glGenBuffersARB(1, @VBO);
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, VBO);
	glEnableClientState(GL_VERTEX_ARRAY);

	glBufferDataARB(GL_ARRAY_BUFFER_ARB, 4*100*sizeof(TVertex),nil, GL_STATIC_DRAW_ARB);
	VBOPointer := glMapBufferARB(GL_ARRAY_BUFFER_ARB, GL_WRITE_ONLY_ARB);

	for x:=0 to 10 do
	for y:=0 to 10 do
		height[x,y] := sin(x)*cos(y);
 
	for x:=0 to 9 do
	for y:=0 to 9 do
	begin
		addPoint(x  ,y  ,height[x,y]    ,x*0.1    ,y*0.1);
		addPoint(x+1,y  ,height[x+1,y]  ,(x+1)*0.1,y*0.1);
		addPoint(x+1,y+1,height[x+1,y+1],(x+1)*0.1,(y+1)*0.1);
		addPoint(x  ,y+1,height[x,y+1]  ,x*0.1    ,(y+1)*0.1);
	end;

	glUnMapBufferARB(GL_ARRAY_BUFFER_ARB);
end;

procedure THeightmap.addPoint(x,y,z,s,t:glFloat);
begin
	VBOPointer^.X := x;
	VBOPointer^.Y := y;
	VBOPointer^.Z := z;
	VBOPointer^.S := s;
	VBOPointer^.T := t;

	inc(Integer(VBOPointer), SizeOf(TVertex));
end;

procedure THeightmap.render;
begin
	glInterleavedArrays(GL_T2F_V3F, SizeOf(TVertex), nil);
	glDrawArrays(GL_QUADS, 0, 4*10*10);
end;

end.