unit partikelsystem;

interface

uses dglOpengl, SDL, Classes;

type
  TfxVector = record
    x     : single;
    y     : single;
    z     : single;
  end;

  TfxColor = record
    r     : byte;
    g     : byte;
    b     : byte;
    a     : byte;
  end;

  PfxParticle = ^TfxParticle;
  TfxParticle = record
    Position : TfxVector;
    Velocity : TfxVector;
    Color    : TfxColor;
    LiveSpan : Int64;
    Age      : Cardinal;
  end;

  PvboParticle = ^TvboParticle;
  TvboParticle = packed record
    S,T: TGLFloat;
    R,G,B,A: Byte;
    X,Y,Z: TGLFLoat;
  end;

  TfxContainer = class(TObject)
    protected
      NumParticle : Integer;
      //wieviele Partikel noch existieren
      Particles  : TList;
      ParticleTex : TGluInt;
      Time : Cardinal;
      Size : Cardinal;
      VBO : TGLuint;
      va : Array of TvboParticle;

      fRender: procedure of object;
      fSaveParticle: procedure of object;

      function fDone: Boolean;

      procedure CreateVBO;
      procedure SaveParticletoVBO;
      procedure RenderVBO;

      procedure CreateVBO_ARB;
      procedure SaveParticletoVBO_ARB;
      procedure RenderVBO_ARB;

      procedure CreateVA;
      procedure SaveParticletoVA;
      procedure RenderVA;

      procedure CreatenonVBO;
      procedure RendernonVBO;

      procedure FXBillboardBegin;
      procedure FXBillboardEnd;
    public
      property Done: Boolean Read fDone;

      constructor Create(aSize : word; aParticleTex: TGLUint);
      procedure Advance(aTime : Cardinal);
      procedure Clean;
      procedure Render;
    end;

  TFire = class(TfxContainer)
    protected
      function FDone: Boolean;
    public
      constructor Create(aSize : word; aParticleTex: TGLUint);
      procedure Render;
      procedure Advance(aTime : Cardinal);
    end;

implementation

{------------------------------------------------------------------------------}
{ TPfxContainer                                                                }
{------------------------------------------------------------------------------}

function TfxContainer.fDone: Boolean;
begin
 Result:=(Numparticle<=0);
end;

////////////////////////////////////////////////////////////////////////////////
// VBO                                                                        //
////////////////////////////////////////////////////////////////////////////////

procedure TfxContainer.CreateVBO;
begin
  fRender:=RenderVBO;
  fSaveParticle:=SaveParticletoVBO;

  glGenBuffers(1, @VBO);
  glBindBuffer(GL_ARRAY_BUFFER, VBO);
  glEnableClientState(GL_VERTEX_ARRAY);
  glBufferData(GL_ARRAY_BUFFER, 4*NumParticle*SizeOf(TvboParticle), nil, GL_DYNAMIC_DRAW);
  glDisableClientState(GL_VERTEX_ARRAY);
end;

procedure TfxContainer.SaveParticletoVBO;
var VertexPointer: PvboParticle;
    VBOPointer: Pointer;
    i: Integer;
begin
  glBindBuffer(GL_ARRAY_BUFFER, VBO);
  glEnableClientState(GL_VERTEX_ARRAY);
  VBOPointer:= glMapBuffer(GL_ARRAY_BUFFER, GL_WRITE_ONLY);

  for i:=0 to NumParticle-1 do
  begin
    VertexPointer:= VBOPointer;
    VertexPointer^.S:=0;
    VertexPointer^.T:=0;
    VertexPointer^.R:=TfxParticle(Particles[i]^).Color.r;
    VertexPointer^.G:=TfxParticle(Particles[i]^).Color.g;
    VertexPointer^.B:=TfxParticle(Particles[i]^).Color.b;
    VertexPointer^.A:=TfxParticle(Particles[i]^).Color.a;
    VertexPointer^.X:=TfxParticle(Particles[i]^).Position.x-0.05;
    VertexPointer^.Y:=TfxParticle(Particles[i]^).Position.y-0.05;
    VertexPointer^.Z:=TfxParticle(Particles[i]^).Position.z;
    inc(Integer(VBOPointer), Sizeof(TvboParticle));

    VertexPointer:= VBOPointer;
    VertexPointer^.S:=0;
    VertexPointer^.T:=1;
    VertexPointer^.R:=TfxParticle(Particles[i]^).Color.r;
    VertexPointer^.G:=TfxParticle(Particles[i]^).Color.g;
    VertexPointer^.B:=TfxParticle(Particles[i]^).Color.b;
    VertexPointer^.A:=TfxParticle(Particles[i]^).Color.a;
    VertexPointer^.X:=TfxParticle(Particles[i]^).Position.x-0.05;
    VertexPointer^.Y:=TfxParticle(Particles[i]^).Position.y+0.05;
    VertexPointer^.Z:=TfxParticle(Particles[i]^).Position.z;
    inc(Integer(VBOPointer), Sizeof(TvboParticle));

    VertexPointer:= VBOPointer;
    VertexPointer^.S:=1;
    VertexPointer^.T:=1;
    VertexPointer^.R:=TfxParticle(Particles[i]^).Color.r;
    VertexPointer^.G:=TfxParticle(Particles[i]^).Color.g;
    VertexPointer^.B:=TfxParticle(Particles[i]^).Color.b;
    VertexPointer^.A:=TfxParticle(Particles[i]^).Color.a;
    VertexPointer^.X:=TfxParticle(Particles[i]^).Position.x+0.05;
    VertexPointer^.Y:=TfxParticle(Particles[i]^).Position.y+0.05;
    VertexPointer^.Z:=TfxParticle(Particles[i]^).Position.z;
    inc(Integer(VBOPointer), Sizeof(TvboParticle));

    VertexPointer:= VBOPointer;
    VertexPointer^.S:=1;
    VertexPointer^.T:=0;
    VertexPointer^.R:=TfxParticle(Particles[i]^).Color.r;
    VertexPointer^.G:=TfxParticle(Particles[i]^).Color.g;
    VertexPointer^.B:=TfxParticle(Particles[i]^).Color.b;
    VertexPointer^.A:=TfxParticle(Particles[i]^).Color.a;
    VertexPointer^.X:=TfxParticle(Particles[i]^).Position.x+0.05;
    VertexPointer^.Y:=TfxParticle(Particles[i]^).Position.y-0.05;
    VertexPointer^.Z:=TfxParticle(Particles[i]^).Position.z;
    inc(Integer(VBOPointer), Sizeof(TvboParticle));
  end;
  glUnMapBuffer(GL_ARRAY_BUFFER);
end;

procedure TfxContainer.RenderVBO;
begin
  Time:=SDL_Getticks;
  FXBillboardBegin;

  glInterleavedArrays(GL_T2F_C4UB_V3F, SizeOf(TvboParticle), nil);
  glDrawArrays(GL_QUADS, 0, 4*NumParticle);
  glDisableClientState(GL_VERTEX_ARRAY);

  FXBillboardEnd;
end;

////////////////////////////////////////////////////////////////////////////////
// VBO_ARB                                                                    //
////////////////////////////////////////////////////////////////////////////////

procedure TfxContainer.CreateVBO_ARB;
begin
  fRender:=RenderVBO_ARB;
  fSaveParticle:=SaveParticletoVBO_ARB;

  glGenBuffersARB(1, @VBO);
  glBindBufferARB(GL_ARRAY_BUFFER_ARB, VBO);
  glEnableClientState(GL_VERTEX_ARRAY);
  glBufferDataARB(GL_ARRAY_BUFFER_ARB, 4*NumParticle*SizeOf(TvboParticle), nil, GL_DYNAMIC_DRAW_ARB);
  glDisableClientState(GL_VERTEX_ARRAY);
end;

procedure TfxContainer.SaveParticletoVBO_ARB;
var VertexPointer: PvboParticle;
    VBOPointer: Pointer;
    i: Integer;
begin
  glBindBufferARB(GL_ARRAY_BUFFER_ARB, VBO);
  glEnableClientState(GL_VERTEX_ARRAY);
  VBOPointer:= glMapBufferARB(GL_ARRAY_BUFFER_ARB, GL_WRITE_ONLY_ARB);

  for i:=0 to NumParticle-1 do
  begin
    VertexPointer:= VBOPointer;
    VertexPointer^.S:=0;
    VertexPointer^.T:=0;
    VertexPointer^.R:=TfxParticle(Particles[i]^).Color.r;
    VertexPointer^.G:=TfxParticle(Particles[i]^).Color.g;
    VertexPointer^.B:=TfxParticle(Particles[i]^).Color.b;
    VertexPointer^.A:=TfxParticle(Particles[i]^).Color.a;
    VertexPointer^.X:=TfxParticle(Particles[i]^).Position.x-0.05;
    VertexPointer^.Y:=TfxParticle(Particles[i]^).Position.y-0.05;
    VertexPointer^.Z:=TfxParticle(Particles[i]^).Position.z;
    inc(Integer(VBOPointer), Sizeof(TvboParticle));

    VertexPointer:= VBOPointer;
    VertexPointer^.S:=0;
    VertexPointer^.T:=1;
    VertexPointer^.R:=TfxParticle(Particles[i]^).Color.r;
    VertexPointer^.G:=TfxParticle(Particles[i]^).Color.g;
    VertexPointer^.B:=TfxParticle(Particles[i]^).Color.b;
    VertexPointer^.A:=TfxParticle(Particles[i]^).Color.a;
    VertexPointer^.X:=TfxParticle(Particles[i]^).Position.x-0.05;
    VertexPointer^.Y:=TfxParticle(Particles[i]^).Position.y+0.05;
    VertexPointer^.Z:=TfxParticle(Particles[i]^).Position.z;
    inc(Integer(VBOPointer), Sizeof(TvboParticle));

    VertexPointer:= VBOPointer;
    VertexPointer^.S:=1;
    VertexPointer^.T:=1;
    VertexPointer^.R:=TfxParticle(Particles[i]^).Color.r;
    VertexPointer^.G:=TfxParticle(Particles[i]^).Color.g;
    VertexPointer^.B:=TfxParticle(Particles[i]^).Color.b;
    VertexPointer^.A:=TfxParticle(Particles[i]^).Color.a;
    VertexPointer^.X:=TfxParticle(Particles[i]^).Position.x+0.05;
    VertexPointer^.Y:=TfxParticle(Particles[i]^).Position.y+0.05;
    VertexPointer^.Z:=TfxParticle(Particles[i]^).Position.z;
    inc(Integer(VBOPointer), Sizeof(TvboParticle));

    VertexPointer:= VBOPointer;
    VertexPointer^.S:=1;
    VertexPointer^.T:=0;
    VertexPointer^.R:=TfxParticle(Particles[i]^).Color.r;
    VertexPointer^.G:=TfxParticle(Particles[i]^).Color.g;
    VertexPointer^.B:=TfxParticle(Particles[i]^).Color.b;
    VertexPointer^.A:=TfxParticle(Particles[i]^).Color.a;
    VertexPointer^.X:=TfxParticle(Particles[i]^).Position.x+0.05;
    VertexPointer^.Y:=TfxParticle(Particles[i]^).Position.y-0.05;
    VertexPointer^.Z:=TfxParticle(Particles[i]^).Position.z;
    inc(Integer(VBOPointer), Sizeof(TvboParticle));
  end;
  glUnMapBufferARB(GL_ARRAY_BUFFER_ARB);
end;

procedure TfxContainer.RenderVBO_ARB;
begin
  Time:=SDL_Getticks;
  FXBillboardBegin;

  glInterleavedArrays(GL_T2F_C4UB_V3F, SizeOf(TvboParticle), nil);
  glDrawArrays(GL_QUADS, 0, 4*NumParticle);
  glDisableClientState(GL_VERTEX_ARRAY);

  FXBillboardEnd;
end;


////////////////////////////////////////////////////////////////////////////////
// Vertex Array                                                               //
////////////////////////////////////////////////////////////////////////////////

procedure TfxContainer.CreateVA;
begin
  fRender:=RenderVA;
  fSaveParticle:=SaveParticletoVA;

  SetLength(va,4*NumParticle);
end;

procedure TfxContainer.SaveParticletoVA;
var i: Integer;
begin
  for i:=0 to NumParticle-1 do
  begin
    VA[4*i].S:=0;
    VA[4*i].T:=0;
    VA[4*i].R:=TfxParticle(Particles[i]^).Color.r;
    VA[4*i].G:=TfxParticle(Particles[i]^).Color.g;
    VA[4*i].B:=TfxParticle(Particles[i]^).Color.b;
    VA[4*i].A:=TfxParticle(Particles[i]^).Color.a;
    VA[4*i].X:=TfxParticle(Particles[i]^).Position.x-0.05;
    VA[4*i].Y:=TfxParticle(Particles[i]^).Position.y-0.05;
    VA[4*i].Z:=TfxParticle(Particles[i]^).Position.z;

    VA[4*i+1].S:=0;
    VA[4*i+1].T:=1;
    VA[4*i+1].R:=TfxParticle(Particles[i]^).Color.r;
    VA[4*i+1].G:=TfxParticle(Particles[i]^).Color.g;
    VA[4*i+1].B:=TfxParticle(Particles[i]^).Color.b;
    VA[4*i+1].A:=TfxParticle(Particles[i]^).Color.a;
    VA[4*i+1].X:=TfxParticle(Particles[i]^).Position.x-0.05;
    VA[4*i+1].Y:=TfxParticle(Particles[i]^).Position.y+0.05;
    VA[4*i+1].Z:=TfxParticle(Particles[i]^).Position.z;

    VA[4*i+2].S:=1;
    VA[4*i+2].T:=1;
    VA[4*i+2].R:=TfxParticle(Particles[i]^).Color.r;
    VA[4*i+2].G:=TfxParticle(Particles[i]^).Color.g;
    VA[4*i+2].B:=TfxParticle(Particles[i]^).Color.b;
    VA[4*i+2].A:=TfxParticle(Particles[i]^).Color.a;
    VA[4*i+2].X:=TfxParticle(Particles[i]^).Position.x+0.05;
    VA[4*i+2].Y:=TfxParticle(Particles[i]^).Position.y+0.05;
    VA[4*i+2].Z:=TfxParticle(Particles[i]^).Position.z;

    VA[4*i+3].S:=1;
    VA[4*i+3].T:=0;
    VA[4*i+3].R:=TfxParticle(Particles[i]^).Color.r;
    VA[4*i+3].G:=TfxParticle(Particles[i]^).Color.g;
    VA[4*i+3].B:=TfxParticle(Particles[i]^).Color.b;
    VA[4*i+3].A:=TfxParticle(Particles[i]^).Color.a;
    VA[4*i+3].X:=TfxParticle(Particles[i]^).Position.x+0.05;
    VA[4*i+3].Y:=TfxParticle(Particles[i]^).Position.y-0.05;
    VA[4*i+3].Z:=TfxParticle(Particles[i]^).Position.z;
  end;
end;

procedure TfxContainer.RenderVA;
begin
  Time:=SDL_Getticks;
  FXBillboardBegin;

  glInterleavedArrays(GL_T2F_C4UB_V3F, SizeOf(TvboParticle), @VA[0]);
  glDrawArrays(GL_QUADS, 0, 4*NumParticle);

  FXBillboardEnd;
end;

////////////////////////////////////////////////////////////////////////////////
// nonVBO                                                                     //
////////////////////////////////////////////////////////////////////////////////

procedure TfxContainer.CreatenonVBO;
begin
  fRender:=RendernonVBO;
  fSaveParticle:=nil;
end;

procedure TfxContainer.RendernonVBO;
var i : integer;
begin
  Time:=SDL_Getticks;
  FXBillboardBegin;
  for i := 0 to numParticle-1 do
    with TfxParticle(Particles[i]^) do
    begin
      //Set Particle Color and Alpha (sat)
      glColor4ub(color.r,color.g,color.b,color.a);
      glTranslatef(Position.x,Position.y,Position.z);
      //Render Quad
      glBegin(GL_QUADS);
        glTexCoord2f(1,0); glVertex3f(0.05,-0.05,0);
        glTexCoord2f(0,0); glVertex3f(-0.05,-0.05,0);
        glTexCoord2f(0,1); glVertex3f(-0.05,0.05,0);
        glTexCoord2f(1,1); glVertex3f(+0.05,+0.05,0);
      glEnd;
      glTranslatef(-Position.x,-Position.y,-Position.z);
    end;
  FXBillboardEnd;
end;



procedure TfxContainer.FXBillboardBegin;
var x,y : byte;
    Matrix : array[0..15] of single;
begin
  //save original Matrix
  glPushMatrix;
  //delete rotation part - replace it with a 3x3 Identity Matrix
  glGetFloatv(GL_MODELVIEW_MATRIX, @Matrix); //get matrix
  for x := 0 to 2 do
    for y := 0 to 2 do
      if x=y then Matrix[x*4+y] := 1 else Matrix[x*4+y] := 0;
  glLoadMatrixf(@Matrix);
end;

procedure TfxContainer.FXBillboardEnd;
begin
  glPopMatrix;
end;

constructor TfxContainer.Create(aSize : word; aParticleTex: TGLUint);
begin
  Particles:=TList.Create;
  NumParticle:=aSize;
  ParticleTex:=aParticleTex;
  Time:=SDL_Getticks;
  Size:=aSize;

  if GL_VERSION_1_5 then CreateVBO
  else if GL_ARB_vertex_buffer_object then CreateVBO_ARB
  else if GL_VERSION_1_1 then CreateVA
  else CreatenonVBO;
end;

procedure TfxContainer.Advance(aTime : Cardinal);
var i : Integer;
    b : Boolean;
begin
  b:=False;
  for i := 0 to NumParticle-1 do
  begin
    with TfxParticle(Particles[i]^) do
      begin
        //S := S_0 + v * t
        Position.X := Position.X + Velocity.X*aTime / 1000;
        Position.Y := Position.Y + Velocity.Y*aTime / 1000;
        Position.Z := Position.Z + Velocity.Z*aTime / 1000;
        LiveSpan := LiveSpan - aTime;
        Age := Age + aTime;
      end;
    if TfxParticle(Particles[i]^).LiveSpan<=0
      then b:=True;
  end;
  if b then Clean;
end;

procedure TfxContainer.Clean;
var i: integer;
begin
  i:=0;
  repeat
    if TfxParticle(Particles[i]^).LiveSpan < 0
      then
      begin
        Dispose(PfxParticle(Particles[i]));
        Particles.Delete(i)
      end
      else Inc(i);
  until i>Particles.Count-1;
  NumParticle:=Particles.Count;
end;

procedure TfxContainer.Render;
begin
  fRender;
end;

{------------------------------------------------------------------------------}
{ TFire                                                                        }
{------------------------------------------------------------------------------}

function TFire.FDone: Boolean;
begin
  Result:=False;
end;

constructor TFire.Create(aSize : word; aParticleTex: TGLUint);
var i: Integer;
    Particle : PfxParticle;
begin
  Inherited;

  for i:=0 to aSize-1 do
  begin
  new(Particle);
  with Particle^ do
    begin
      //Set Particle-Template
      Position.x := random(20)/100-0.10;
      Position.y := random(5)/100;
      Position.z := random(5)/100;
      Velocity.x := random(60)/100-0.3;
      Velocity.y := random(80)/100+0.2;
      Velocity.z := random(40)/100-0.2;
      Color.r := 255;
      Color.g := round(random(255)*0.7);
      Color.b := round(random(255)*0.25);
      Color.a := 255;
      LiveSpan := random(1000);
      Age := 0;
    end;
  Particles.Add(Particle);
  end;
  if Assigned(fSaveParticle) then fSaveParticle;
end;

procedure TFire.Render;
begin
  glBindTexture(GL_TEXTURE_2D,ParticleTex);
  Advance(SDL_Getticks-Time);

  Inherited;
end;

procedure TFire.Advance(aTime : Cardinal);
var i : Integer;
begin
  for i := 0 to NumParticle-1 do
  begin
    with TfxParticle(Particles[i]^) do
      begin
        //S := S_0 + v * t
        Position.X := Position.X + Velocity.X*aTime/1000;
        Position.Y := Position.Y + Velocity.Y*aTime/1000;
        Position.Z := Position.Z + Velocity.Z*aTime/1000;
        LiveSpan := LiveSpan - aTime;
        Age := Age + aTime;
        Color.a:=trunc(Livespan/1500*255);
        //if age > 100 then color.a:=trunc((1-Age/1000)*255) else color.a:= trunc(Age/100*255);
        if Livespan<0 then
          begin
            //Set Particle-Template
            Position.x := random(20)/100-0.10;
            Position.y := random(5)/100;
            Position.z := random(5)/100;
            Velocity.x := random(60)/100-0.3;
            Velocity.y := random(80)/100+0.2;
            Velocity.z := random(40)/100-0.2;
            Color.r := 255;
            Color.g := round(random(255)*0.7);
            Color.b := round(random(255)*0.25);
            Color.a := 255;
            LiveSpan := 1000+random(500);
            Age := 0;
          end;
      end;
  end;
  if Assigned(fSaveParticle) then fSaveParticle;
end;

end.
