program pointsprite;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE CONSOLE}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Florian Sievert (Phobeus)
  Phobeus@DelphiGL.com
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2006 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  This sample try to demonstrate how to use the arb point sprite. A point sprite might be usefull for a particle system.
	Instead of using several quads with the particle texture, you can define the position with a point. This should also save 
	bandwidth of the pipeline allowing more particles ;)

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2006-01-18 Minor changes for compatibility (JS)
  2006-01-18 Created (FS)

  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 


  FS
    Florian Sievert (Phobeus)
    email : Phobeus@DelphiGL.com
    Visit: http://www.delphigl.com

*******************************************************************************)

uses 
  dglOpenGL,

  SDL,
  easySDL,
	easySDLTextures;

{$I ../dglsdk.inc}

type
  TMyProgram = class(TEasySDL)
    procedure DrawScene;override;
		procedure renderPointSprites;
		procedure renderAxis;
		procedure UpdateWorld(Seconds: TGLFloat); override;
  end;

var
  MyProgram :TMyProgram;

var
	Cam_Rot: SIngle;


procedure TMyProgram.UpdateWorld(Seconds: TGLFloat);
begin
	Cam_Rot := Cam_Rot + seconds*50;
end;

procedure TMyProgram.renderPointSprites;
var
	quadratic: array[0..2] of Single;
	maxSize: Single;
begin
	glEnable( GL_BLEND );
	glEnable(GL_DEPTH_TEST);
	glBlendFunc( GL_SRC_ALPHA, GL_ONE );

	quadratic[0] := 0.01;
	quadratic[1] := 0.01;
	quadratic[2] := 0.01;

	glPointParameterfvARB( GL_POINT_DISTANCE_ATTENUATION_ARB, @quadratic );

	// Abfrage der maximalen point size
	maxSize := 0.0;
	glGetFloatv( GL_POINT_SIZE_MAX_ARB, @maxSize );

	// Maximal einen Wert von 100 verwenden
	if( maxSize > 100.0 ) then maxSize := 100.0;
	glPointSize( maxSize );

	glPointParameterfARB( GL_POINT_FADE_THRESHOLD_SIZE_ARB, 60.0 );
	glPointParameterfARB( GL_POINT_SIZE_MIN_ARB, 1.0 );
	glPointParameterfARB( GL_POINT_SIZE_MAX_ARB, maxSize );

	glTexEnvf( GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE );

	// Punkte als Point Sprite rendern
	glEnable( GL_POINT_SPRITE_ARB );

	glBegin( GL_POINTS );
		glColor3f( 1.0, 0.0, 0.0); glVertex3f(-2, 0, 0);
		glColor3f( 0.0, 1.0, 0.0); glVertex3f( 0, 0, 0);
		glColor3f( 0.0, 0.0, 1.0); glVertex3f( 2, 0, 0);
	glEnd();

	// Punkte wieder normal zeichnen
	glDisable( GL_POINT_SPRITE_ARB );
	glDisable(GL_BLEND);
end;

procedure TMyProgram.renderAxis;
begin
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);
	glLineWidth(3);

	glBegin(GL_LINES);
		// x-axis
		glColor3f(1,0,0);
		glVertex3f(0,0,0);
		glVertex3f(100,0,0);

		glColor3f(0.5,0,0);
		glVertex3f(-100,0,0);
		glVertex3f(0,0,0);

		// y-axis
		glColor3f(0,1,0);
		glVertex3f(0,0,0);
		glVertex3f(0,100,0);

		glColor3f(0,0.5,0);
		glVertex3f(0,-100,0);
		glVertex3f(0,0,0);

	glEnd;

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glLineWidth(1);
	glColor3f(1,1,1);
end;

procedure TMyProgram.DrawScene;
begin
	glClearColor(0.0,0.0,0.0,1);
	glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );
	glMatrixMode( GL_MODELVIEW );

	// Rotierende Kamera
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -8.0+3*sin(Cam_rot/25) );
	glRotatef(45,1,0,0);
	glRotatef(Cam_Rot,0,1,0);

	renderAxis;
	renderPointSprites();
end;

begin
  MyProgram := TMyProgram.Create('www.delphigl.com .:. Point Sprite Sample');
  MyProgram.Start;

	textures.Load(DGLSDK_DATA+'gfx/particle.png');

  MyProgram.Loop;
  MyProgram.Stop;
end.
