program renderpass;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE CONSOLE}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Florian Sievert (Phobeus)
  Phobeus@DelphiGL.com
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2005 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
	This is the source code for the renderpass tutorial. It demonstrates how to render a scene into a texture to use it again later.
	You'll find a scene with some rotating pyramids in the center and three screen in the background. With F1-F3 you can active or 
	deactivate the screen. If active, they will display the offscene texture, else just a noise. If you press Space, you can activate
	recursive rendering that shows a pretty nice effect.

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2006-02-18 Port from WinAPI (FS)


  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 

  FS
    Florian Sievert (Phobeus)
    email : Phobeus@DelphiGL.com
    Visit: http://www.delphigl.com

*******************************************************************************)

uses 
	dglOpenGL,

	SDL,
	easySDL,
	easySDLTextures;

{$I ../dglsdk.inc}

type
  TMyProgram = class(TEasySDL)
		TM: TEasySDLTextures;
		screen: gluInt; 		// our offscreen texture
		s1,s2,s3: Boolean; 	// screen status
		recursive: Boolean;	// recursive scene status
    procedure drawScene;override;
		procedure updateWorld(seconds: Single);override;
		procedure init;

		procedure offScreen;
		procedure monitorMode(active:boolean);

		procedure drawPyramidScene;
		procedure drawPyramid;
		procedure drawScreens;
  end;

var
  MyProgram :TMyProgram;


	// dummy
	x: Single;
	outrot: Single;

procedure TMyProgram.updateWorld(seconds: Single);
begin
	x := x + seconds*50;
	outrot := outrot + seconds*50;

	if keyDownOnce(SDLK_F1) then s1 := not s1;
	if keyDownOnce(SDLK_F2) then s2 := not s2;
	if keyDownOnce(SDLK_F3) then s3 := not s3;

	if keyDownOnce(SDLK_SPACE) then recursive := not recursive;
end;

procedure TMyProgram.drawPyramid;
begin
	glBegin(GL_TRIANGLES);
		//left
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(   0,  1.0,    0);

		//right
		glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(   0,  1.0,    0);

		//front
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f( 0.0,  1.0,  0.0);

		//back
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f( 0.0,  1.0,  0.0);
	glEnd;
end;

procedure TMyProgram.drawPyramidScene;
begin
	// prepare chrome effect
	glMatrixMode(GL_TEXTURE);
		glLoadIdentity;
		glColor3f(1,1,1);
		glTranslatef(0,-(x/100),0);
	glMatrixMode(GL_MODELVIEW);


	// small center pyramid 
	TM.bind('chrome');
	glLoadIdentity;
	glTranslatef(0,-2,-14);

	glPushMatrix;
		glTranslatef(0,sin(x/50),0);
		glRotatef(x,0,1,0);

		DrawPyramid;
	glPopMatrix;

	// big center pyramid
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glDisable(GL_DEPTH_TEST);

	glPushMatrix;
		glRotatef(x,0,1,0);
		glScalef(2,2,2);

		if Sin(x/50) < 0.9 then glColor4f(1,1,1,0.4) else glColor4f(1,0.3,0.3,0.5);

		DrawPyramid;
	glPopMatrix;

	// blue pyramid
	glPushMatrix;
		glColor4f(0.3,0.3,0.8,0.8);
		glRotatef(outrot,0,1,0);
		glTranslatef(5,0,0);
		glRotatef(outRot,-0.5,1,0);
		DrawPyramid;
	glPopMatrix;

	// green pyramid
	glPushMatrix;
		glColor4f(0.3,0.8,0.3,0.8);
		glRotatef(outrot,0,1,0);
		glTranslatef(-5,0,0);
		glRotatef(OutRot,0.5,1,0);
		drawPyramid;
	glPopMatrix;

	// reset to default
	glMatrixMode(GL_TEXTURE);
		glLoadIdentity;
	glMatrixMode(GL_MODELVIEW);
	glColor4f(1,1,1,0);

	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
end;

procedure TMyProgram.monitorMode(active:Boolean);
begin
	if active then
	begin
		glBindTexture(GL_TEXTURE_2D,screen);
		glMatrixMode(GL_TEXTURE);
		glLoadIdentity;
		glMatrixMode(GL_MODELVIEW);
	end else
	begin
		TM.bind('noise');
		glMatrixMode(GL_TEXTURE);
		glLoadIdentity;
		gltranslatef(random(100)/10,x,0);
		glMatrixMode(GL_MODELVIEW);
	end;
end;

procedure TMyProgram.drawScreens;
begin
 (* Wir rendern alle Bildschirme *)
	glLoadIdentity;
	glTranslatef(0,0,-10);

	monitorMode(s1);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex3f(-6, 0,-1);
		glTexCoord2f(1,0); glVertex3f(-2, 0,-6);
		glTexCoord2f(1,1); glVertex3f(-2, 4,-6);
		glTexCoord2f(0,1); glVertex3f(-6, 4,-1);
	glEnd();

	monitorMode(s2);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex3f( 2, 0,-6);
		glTexCoord2f(1,0); glVertex3f( 6, 0,-1);
		glTexCoord2f(1,1); glVertex3f( 6, 4,-1);
		glTexCoord2f(0,1); glVertex3f( 2, 4,-6);
	glEnd();

	monitorMode(s3);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex3f(-2, 3,-14);
		glTexCoord2f(1,0); glVertex3f( 2, 3,-14);
		glTexCoord2f(1,1); glVertex3f( 2, 6,-10);
		glTexCoord2f(0,1); glVertex3f(-2, 6,-10);
	glEnd();

	monitorMode(false);
end;

procedure TMyProgram.offScreen;
begin
	// clear with black color
	glClearColor(0.0,0.0,0.0,0);
	glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

	// set viewport to texture size
	glViewport(0, 0, 256, 256);
	glBindTexture(GL_TEXTURE_2D,Screen);

	// draw the scene
	if recursive then drawScreens;
	DrawPyramidScene;

	// copy scene into texture
	glBindTexture(GL_TEXTURE_2D, Screen);
	glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 0, 0, 256, 256, 0);
end;

procedure TMyProgram.drawScene;
begin
	// render offscreen texture first
	offScreen;

	// draw the normal scene
	glClearColor(0,0.0,0.3,0);
	glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
	glViewport(0, 0, 640, 480);
	drawScreens;
	drawPyramidScene;
end;

procedure TMyProgram.init;
var pTexData: Pointer;
begin
	// allocate memory for offscreen texture (width*height*RGB)
	GetMem(pTexData, 256*256*3);

	glGenTextures(1, @Screen);
	glBindTexture(GL_TEXTURE_2D, Screen);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 256, 256, 0, GL_RGB, GL_UNSIGNED_BYTE, pTexData);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	FreeMem(pTexData);

	// using texture manager for all other textures
	TM := TEasySDLTextures.Create;
	TM.Load(DGLSDK_DATA+'/gfx/chrome.jpg','chrome');
	TM.Load(DGLSDK_DATA+'/gfx/noise.jpg','noise');
end;

begin
	Randomize;
	MyProgram := TMyProgram.Create('www.delphigl.com .:. Renderpass Tutorial');

	MyProgram.Start;
	MyProgram.Init;
	MyProgram.Loop;
	MyProgram.Stop;
end.
