// Unit gepostet von Noop aus der DGL-Community
unit mHelperLib;

interface

uses dglOpenGL;

type
  float = double;
  TPoint3DF = record x,y,z: float; end;
  PPoint3DF=^TPoint3DF;
  TMatrix4x4 = record
    _11,_12,_13,_14: float;
    _21,_22,_23,_24: float;
    _31,_32,_33,_34: float;
    _41,_42,_43,_44: float;
  end;
  PMatrix4x4 = ^TMatrix4x4;
  TMatrix = array of array of float;
  PMatrix = ^TMatrix;

function Point3DF(x, y, z: float): TPoint3DF;
function _oglGetMatrix(mmode: cardinal): TMatrix4x4;
procedure _oglSetMatrix(mmode: cardinal; m: TMatrix4x4);
function _Mul(a, b: TMatrix4x4): TMatrix4x4;
function _App(a: TPoint3DF; m: TMatrix4x4): TPoint3DF;
function _New4x4: TMatrix4x4; overload;
function _RotX(d: float): TMatrix4x4;
function _RotY(d: float): TMatrix4x4;
function _RotZ(d: float): TMatrix4x4;
function _RotN(n: TPoint3DF; d: float): TMatrix4x4;

implementation

function Point3DF(x, y, z: float): TPoint3DF;
begin
  result.x:=x;
  result.y:=y;
  result.z:=z;
end;

function _oglGetMatrix(mmode: cardinal): TMatrix4x4;
var
  f: array[0..15] of GLDouble;
begin
  FillChar(f[0], SizeOf(f), 0);
  glMatrixMode(mmode);
  if (mmode = GL_MODELVIEW) or (mmode = GL_MODELVIEW_MATRIX) then
   glGetDoublev(GL_MODELVIEW_MATRIX, @f);
  if (mmode = GL_PROJECTION) or (mmode = GL_PROJECTION_MATRIX) then
   glGetDoublev(GL_PROJECTION_MATRIX, @f);
  if (mmode = GL_TEXTURE) or (mmode = GL_TEXTURE_MATRIX) then
   glGetDoublev(GL_TEXTURE_MATRIX, @f);
  result._11 := f[00]; result._12 := f[01]; result._13 := f[02]; result._14 := f[03];
  result._21 := f[04]; result._22 := f[05]; result._23 := f[06]; result._24 := f[07];
  result._31 := f[08]; result._32 := f[09]; result._33 := f[10]; result._34 := f[11];
  result._41 := f[12]; result._42 := f[13]; result._43 := f[14]; result._44 := f[15];
end;

procedure _oglSetMatrix(mmode: cardinal; m: TMatrix4x4);
var
  f: array[0..15] of GLDouble;
begin
  f[00] := m._11; f[01] := m._12; f[02] := m._13; f[03] := m._14;
  f[04] := m._21; f[05] := m._22; f[06] := m._23; f[07] := m._24;
  f[08] := m._31; f[09] := m._32; f[10] := m._33; f[11] := m._34;
  f[12] := m._41; f[13] := m._42; f[14] := m._43; f[15] := m._44;
  glMatrixMode(mmode);
  if (mmode = GL_MODELVIEW) or (mmode = GL_MODELVIEW_MATRIX) then
   glLoadMatrixd(@f);
  if (mmode = GL_PROJECTION) or (mmode = GL_PROJECTION_MATRIX) then
   glLoadMatrixd(@f);
  if (mmode = GL_TEXTURE) or (mmode = GL_TEXTURE_MATRIX) then
   glLoadMatrixd(@f);
end;

function _Mul(a, b: TMatrix4x4): TMatrix4x4;
begin
  result._11 := b._11*a._11 + b._12*a._21 + b._13*a._31 + b._14*a._41;
  result._12 := b._11*a._12 + b._12*a._22 + b._13*a._32 + b._14*a._42;
  result._13 := b._11*a._13 + b._12*a._23 + b._13*a._33 + b._14*a._43;
  result._14 := b._11*a._14 + b._12*a._24 + b._13*a._34 + b._14*a._44;

  result._21 := b._21*a._11 + b._22*a._21 + b._23*a._31 + b._24*a._41;
  result._22 := b._21*a._12 + b._22*a._22 + b._23*a._32 + b._24*a._42;
  result._23 := b._21*a._13 + b._22*a._23 + b._23*a._33 + b._24*a._43;
  result._24 := b._21*a._14 + b._22*a._24 + b._23*a._34 + b._24*a._44;

  result._31 := b._31*a._11 + b._32*a._21 + b._33*a._31 + b._34*a._41;
  result._32 := b._31*a._12 + b._32*a._22 + b._33*a._32 + b._34*a._42;
  result._33 := b._31*a._13 + b._32*a._23 + b._33*a._33 + b._34*a._43;
  result._34 := b._31*a._14 + b._32*a._24 + b._33*a._34 + b._34*a._44;

  result._41 := b._41*a._11 + b._42*a._21 + b._43*a._31 + b._44*a._41;
  result._42 := b._41*a._12 + b._42*a._22 + b._43*a._32 + b._44*a._42;
  result._43 := b._41*a._13 + b._42*a._23 + b._43*a._33 + b._44*a._43;
  result._44 := b._41*a._14 + b._42*a._24 + b._43*a._34 + b._44*a._44;
end;

function _App(a: TPoint3DF; m: TMatrix4x4): TPoint3DF;
begin
  result.x := a.x*m._11 + a.y*m._21 + a.z*m._31 + m._41;
  result.y := a.x*m._12 + a.y*m._22 + a.z*m._32 + m._42;
  result.z := a.x*m._13 + a.y*m._23 + a.z*m._33 + m._43;
end;

function _New4x4: TMatrix4x4;
begin
  result._11 := 1.0; result._12 := 0.0; result._13 := 0.0; result._14 := 0.0;
  result._21 := 0.0; result._22 := 1.0; result._23 := 0.0; result._24 := 0.0;
  result._31 := 0.0; result._32 := 0.0; result._33 := 1.0; result._34 := 0.0;
  result._41 := 0.0; result._42 := 0.0; result._43 := 0.0; result._44 := 1.0;
end;

function _RotX(d: float): TMatrix4x4;
begin
  Result := _New4x4;
  Result._22 := Cos(d); Result._32 :=-Sin(d);
  Result._23 := Sin(d); Result._33 := Cos(d);
end;

function _RotY(d: float): TMatrix4x4;
begin
  Result := _New4x4;
  Result._11 := Cos(d); Result._31 := Sin(d);
  Result._13 :=-Sin(d); Result._33 := Cos(d);
end;

function _RotZ(d: float): TMatrix4x4;
begin
  Result := _New4x4;
  Result._11 := Cos(d); Result._21 :=-Sin(d);
  Result._12 := Sin(d); Result._22 := Cos(d);
end;

function _RotN(n: TPoint3DF; d: float): TMatrix4x4;
var
  xx, yy, zz, xy, yz, zx, xs, ys, zs, one_c, s, c, mag: float;
  optimized: boolean;
  m: TMatrix4x4;
begin
(*
 * Generate a 4x4 transformation matrix from glRotate parameters, and
 * post-multiply the input matrix by it.
 *
 * \author
 * This function was contributed by Erich Boleyn (erich@uruk.org).
 * Optimizations contributed by Rudolf Opalla (rudi@khm.de).
 *
 * Translated into Delphi by Stefan (slude007 [at] web [d0t] de)
*)
  s := Sin(d);
  c := Cos(d);
  optimized := False;
  m._11 := 1.0; m._12 := 0.0; m._13 := 0.0; m._14 := 0.0;
  m._21 := 0.0; m._22 := 1.0; m._23 := 0.0; m._24 := 0.0;
  m._31 := 0.0; m._32 := 0.0; m._33 := 1.0; m._34 := 0.0;
  m._41 := 0.0; m._42 := 0.0; m._43 := 0.0; m._44 := 1.0;
  if n.x = 0 then
  begin
    if n.y = 0 then
    begin
      if n.z <> 0 then
      begin
        //Rotate only around Z axis
        optimized := True;
        m._11 := c;
        m._22 := c;
        if n.z < 0 then
        begin
          m._12 := s;
          m._21 := -s;
        end else
        begin
          m._12 := -s;
          m._21 := s;
        end;
      end
    end else
    if n.z = 0 then
    begin
      //Rotate only around Y axis
      optimized := True;
      m._11 := c;
      m._33 := c;
      if n.y < 0 then
      begin
        m._13 := -s;
        m._31 := s;
      end else
      begin
        m._13 := s;
        m._31 := -s;
      end;
    end;
  end else
  if n.y = 0 then
  begin
    if n.z = 0 then
    begin
      //Rotate only around X axis
      optimized := True;
      m._22 := c;
      m._33 := c;
      if (n.x < 0) then
      begin
        m._23 := s;
        m._32 := -s;
      end else
      begin
        m._23 := -s;
        m._32 := s;
      end;
    end;
  end;
  if not optimized then
  begin
    mag := Sqrt(n.x*n.x + n.y*n.y + n.z*n.z);
    if mag <= 0.0001 then exit;
    n.x := n.x / mag;
    n.y := n.y / mag;
    n.z := n.z / mag;
(*
       *     Arbitrary axis rotation matrix.
       *
       *  This is composed of 5 matrices, Rz, Ry, T, Ry', Rz', multiplied
       *  like so:  Rz * Ry * T * Ry' * Rz'.  T is the final rotation
       *  (which is about the X-axis), and the two composite transforms
       *  Ry' * Rz' and Rz * Ry are (respectively) the rotations necessary
       *  from the arbitrary axis to the X-axis then back.  They are
       *  all elementary rotations.
       *
       *  Rz' is a rotation about the Z-axis, to bring the axis vector
       *  into the x-z plane.  Then Ry' is applied, rotating about the
       *  Y-axis to bring the axis vector parallel with the X-axis.  The
       *  rotation about the X-axis is then performed.  Ry and Rz are
       *  simply the respective inverse transforms to bring the arbitrary
       *  axis back to it's original orientation.  The first transforms
       *  Rz' and Ry' are considered inverses, since the data from the
       *  arbitrary axis gives you info on how to get to it, not how
       *  to get away from it, and an inverse must be applied.
       *
       *  The basic calculation used is to recognize that the arbitrary
       *  axis vector (x, y, z), since it is of unit length, actually
       *  represents the sines and cosines of the angles to rotate the
       *  X-axis to the same orientation, with theta being the angle about
       *  Z and phi the angle about Y (in the order described above)
       *  as follows:
       *
       *  cos ( theta ) = x / sqrt ( 1 - z^2 )
       *  sin ( theta ) = y / sqrt ( 1 - z^2 )
       *
       *  cos ( phi ) = sqrt ( 1 - z^2 )
       *  sin ( phi ) = z
       *
       *  Note that cos ( phi ) can further be inserted to the above
       *  formulas:
       *
       *  cos ( theta ) = x / cos ( phi )
       *  sin ( theta ) = y / sin ( phi )
       *
       *  ...etc.  Because of those relations and the standard trigonometric
       *  relations, it is pssible to reduce the transforms down to what
       *  is used below.  It may be that any primary axis chosen will give the
       *  same results (modulo a sign convention) using thie method.
       *
       *  Particularly nice is to notice that all divisions that might
       *  have caused trouble when parallel to certain planes or
       *  axis go away with care paid to reducing the expressions.
       *  After checking, it does perform correctly under all cases, since
       *  in all the cases of division where the denominator would have
       *  been zero, the numerator would have been zero as well, giving
       *  the expected result.
*)
    xx := n.x * n.x;
    yy := n.y * n.y;
    zz := n.z * n.z;
    xy := n.x * n.y;
    yz := n.y * n.z;
    zx := n.z * n.x;
    xs := n.x * s;
    ys := n.y * s;
    zs := n.z * s;
    one_c := 1.0 - c;
    m._11 := (one_c * xx) + c;
    m._12 := (one_c * xy) - zs;
    m._13 := (one_c * zx) + ys;

    m._21 := (one_c * xy) + zs;
    m._22 := (one_c * yy) + c;
    m._23 := (one_c * yz) - xs;

    m._31 := (one_c * zx) - ys;
    m._32 := (one_c * yz) + xs;
    m._33 := (one_c * zz) + c;
  end;
  Result := m;
end;
end.
