program rot_template;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE CONSOLE}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Jonas Scholz (i0n0s)
  jonasscholz@web.de
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2006 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  This sample shows how to use rotation and translation with fixed axis.
  Based on the example "CAD-Template" postet by attragie here:
  http://www.delphigl.com/forum/viewtopic.php?t=5452


  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2006-05-05 Initial release (JS)

  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 


  JS
    Jonas Scholz (i0n0s)
    email : jonasscholz@web.de

*******************************************************************************)

uses
  dglOpenGL,
  SDL, SDL_TTF,
  easySDL,
  easySDLFont,
  easySDLTextures,
  mHelperLib,
  SysUtils;

{$I ../dglsdk.inc}

type
  TMyProgram = class(TEasySDL)
    Scale: Single;                                 //Faktor fr den Zoom
    Movex, Movey, Movez: Single;                   //Vektor fr die Translation
    MouseX, MouseY: Word;                          //Zwischenspeicher fr die Mauskoordinaten
    cs: Boolean;                                   //Boolean fr die Anzeige des Koordinatensystems
    WinkelX, WinkelY, WinkelZ: Single;             //Winkel des Krpers, wird nur fr die Ausgabe verwendet
    viewmat: TMatrix4x4;                           //Ansichtmatrix

    RightMousePressed, LeftMousePressed,
    MiddleMousePressed, MouseCapture : Boolean;    //Boolean fr die Mausverarbeitung

    ZDFont, BZDFont, DDFont: TFont;                //Schriften fr die Darstellung
    procedure ShowLegende;                         //Zeigt die "Legende" an
    procedure ShowCs;                              //Zeigt das Koordinatensystem an
    procedure FitView;                             //Setzt die Ansicht zurck
    procedure handleKeyPressed(event: TSDL_KeyboardEvent); override;
    procedure handleMousePressed(event: TSDL_MouseButtonEvent); override;
    procedure handleMouseReleased(event: TSDL_MouseButtonEvent); override;
    procedure handleMouseMotion(event: TSDL_MouseMotionEvent); override;
    procedure DrawScene; override;
  end;

var
    MyProgram   : TMyProgram;

procedure TMyProgram.ShowLegende;
begin
  ZDFont.Draw('FPS: '+InttoStr(trunc(FPS)),5,0);
  BZDFont.Draw('Rotation- & Translationbeispiel',640-180,480-30);

  ZDFont.Draw('Help:',5,480-20);
  ZDFont.Draw('ESC : Programm beenden',5,480-45);
  ZDFont.Draw('v   : Ansicht zurcksetzen',5,480-65);
  ZDFont.Draw('k   : Koordinatenkreuz',5,480-85);
  ZDFont.Draw('X-Rotation: '+InttoStr(trunc(WinkelX*180/pi)),640-80,65);
  ZDFont.Draw('Y-Rotation: '+InttoStr(trunc(WinkelY*180/pi)),640-80,45);
  ZDFont.Draw('Z-Rotation: '+InttoStr(trunc(WinkelZ*180/pi)),640-80,25);
end;


procedure TMyProgram.ShowCs;
begin
  glBindTexture(GL_TEXTURE_2D,0);
  glLineWidth(3);
  glbegin(gl_lines);   //Koordiantenkreuz
    glcolor3f(1,0,0);
    glvertex3f(-1,0,0);
    glvertex3f(1,0,0);
    glcolor3f(0,1,0);
    glvertex3f(0,-1,0);
    glvertex3f(0,1,0);
    glcolor3f(0,0,1);
    glvertex3f(0,0,-1);
    glvertex3f(0,0,1);
  glend;
  //Beschriftung des Koordiantenkreuzes
  DDFont.Draw3D('-x',-1,0,0);
  DDFont.Draw3D('+x',1,0,0);

  DDFont.Draw3D('-y',0,-1,0);
  DDFont.Draw3D('+y',0,1,0);

  DDFont.Draw3D('-z',0,0,-1);
  DDFont.Draw3D('+z',0,0,1);
end;

//----------------------------------------------------------------------------
// Zeichnen der Szene
//----------------------------------------------------------------------------
procedure TMyProgram.DrawScene;
const
  x: double = 0.4;
  y: double = 0.4;
  z: double = 0.4;
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  //Bewegen zur Position des Wrfels
  glTranslatef(movex,movey,movez);
  //Rotieren und wechseln in die Modellansicht
  _oglSetMatrix(GL_MODELVIEW,viewmat);
  //Setzen des Zoomfaktors
  glScalef(scale,scale,scale);

  //Laden der Textur
  Textures.Bind('Wiki');

  //Zeichnen des Wrfels
  glBegin(GL_QUADS);
    glTexCoord2f(1,0); glVertex3f(-x, -y,  z);
    glTexCoord2f(0,0); glVertex3f( x, -y,  z);
    glTexCoord2f(0,1); glVertex3f( x,  y,  z);
    glTexCoord2f(1,1); glVertex3f(-x,  y,  z);

    glTexCoord2f(1,0); glVertex3f( x, -y, -z);
    glTexCoord2f(0,0); glVertex3f(-x, -y, -z);
    glTexCoord2f(0,1); glVertex3f(-x,  y, -z);
    glTexCoord2f(1,1); glVertex3f( x,  y, -z);

    glTexCoord2f(1,0); glVertex3f( x, -y,  z);
    glTexCoord2f(0,0); glVertex3f(-x, -y,  z);
    glTexCoord2f(0,1); glVertex3f(-x, -y, -z);
    glTexCoord2f(1,1); glVertex3f( x, -y, -z);

    glTexCoord2f(1,0); glVertex3f( x,  y, -z);
    glTexCoord2f(0,0); glVertex3f(-x,  y, -z);
    glTexCoord2f(0,1); glVertex3f(-x,  y,  z);
    glTexCoord2f(1,1); glVertex3f( x,  y,  z);

    glTexCoord2f(1,1); glVertex3f(-x,  y, -z);
    glTexCoord2f(1,0); glVertex3f(-x, -y, -z);
    glTexCoord2f(0,0); glVertex3f(-x, -y,  z);
    glTexCoord2f(0,1); glVertex3f(-x,  y,  z);

    glTexCoord2f(1,1); glVertex3f( x,  y,  z);
    glTexCoord2f(1,0); glVertex3f( x, -y,  z);
    glTexCoord2f(0,0); glVertex3f( x, -y, -z);
    glTexCoord2f(0,1); glVertex3f( x,  y, -z);
  glEnd;

  //Koordiantensystem anzeigen
  if CS then showCS;

  //Ansichten fr die Legende zurcksetzen
  glLoadIdentity;
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  //In den Orthomoduswechseln
  glOrtho(0, 640, 0, 480, 0, 100);
  //Legende anzeigen
  ShowLegende;
end;

//Anfangszustand wieder herstellen
procedure TMyProgram.FitView;
begin
  movex:=0; movey:=0;
  WinkelX:=0; WinkelY:=0; WinkelZ:=0;
  scale:=1;
  viewmat := _New4x4;
end;

procedure TMyProgram.handleKeyPressed(event: TSDL_KeyboardEvent);
begin
  case event.keysym.sym of
    SDLK_ESCAPE : done := -1;
    SDLK_v : FitView;
    SDLK_k : Cs:=not Cs;
  end;
end;

procedure TMyProgram.handleMousePressed(event: TSDL_MouseButtonEvent);
var f: Double;
begin
  MouseCapture := true;

  MouseX := event.x;
  MouseY := event.y;
  // feststellen welcher Mausbutton gedrckt wurde
  if event.button=SDL_BUTTON_LEFT then LeftMousePressed:=true else LeftMousePressed:=false;
  if event.button=SDL_BUTTON_RIGHT then RightMousePressed:=true else RightMousePressed:=false;
  if event.button=SDL_BUTTON_MIDDLE then MiddleMousePressed:=true else MiddleMousePressed:=false;
  if event.button=SDL_BUTTON_WHEELUP then
  begin
    //Ranzoomen
    if scale>1 then f:=scale*0.08 else f:=0.08;
    scale:=scale-f;
    if scale<0 then scale:=0.01;
  end;
  if event.button=SDL_BUTTON_WHEELDOWN then
  begin
    //Rauszoomen
    if scale>1 then f:=scale*0.08 else f:=0.08;
    scale:=scale+f;
    if scale<0 then scale:=0.01;
  end;
end;

procedure TMyProgram.handleMouseReleased(event: TSDL_MouseButtonEvent);
begin
  MouseCapture:=False;
end;

procedure TMyProgram.handleMouseMotion(event: TSDL_MouseMotionEvent);
var f: double;
    XRotation, YRotation ,ZRotation: Single;
begin
  //Rotation um X- und Y-Achste bei rechter Maustaste
  if MouseCapture and RightMousePressed then
  begin
    //Berechnen des Winkels der durch die Bewegung entstanden ist
    YRotation := ((MouseX - event.x) div 2);
    XRotation := ((MouseY - event.y) div 2);
    //Einberechnen der Rotation in die Matrix
    viewmat := _Mul(_RotY(YRotation/25), viewmat);
    viewmat := _Mul(_RotX(XRotation/25), viewmat);

    //Speichern der Werte fr die Ausgabe
    WinkelX:=WinkelX+XRotation/25;
    If WinkelX>2*pi then WinkelX:=WinkelX-2*pi;
    If WinkelX<-2*pi then WinkelX:=WinkelX+2*pi;
    WinkelY:=WinkelY+YRotation/25;
    If WinkelY>2*pi then WinkelY:=WinkelY-2*pi;
    If WinkelY<-2*pi then WinkelY:=WinkelY+2*pi;
  end;

  //Z-Achse bei mittlere Maustaste
  if MouseCapture and MiddleMousePressed then
  begin
    //Berechnen des Winkels der durch die Bewegung entstanden ist
    YRotation := ((MouseX - event.x) div 2);
    XRotation := ((MouseY - event.y) div 2);

    //Mauszoom, wenn CTRL gedrckt ist
    if ((SDL_GetModState and KMOD_LCTRL)=KMOD_LCTRL)
    or ((SDL_GetModState and KMOD_RCTRL)=KMOD_RCTRL) then
    begin
      if scale>1 then f:=scale*0.02 else f:=0.02;
      if event.yrel<0 then scale:=scale-f else scale:=scale+f;
      if scale<0 then scale:=0.01;
    end
    else
    begin
      //Einberechnen der Rotation in die Matrix
      viewmat := _Mul(_RotZ((XRotation+YRotation)/25), viewmat);

      //Speichern der Werte fr die Ausgabe
      WinkelZ:=WinkelZ+(XRotation+YRotation)/25;
      If WinkelZ>2*pi then WinkelZ:=WinkelZ-2*pi;
      If WinkelZ<-2*pi then WinkelZ:=WinkelZ+2*pi;
    end;
  end;

  //Bewegen bei linker Maustaste
  if MouseCapture and LeftMousePressed then
  begin
    movex := movex - ((MouseX - event.x) div 2)*0.01;
    movey := movey + ((MouseY - event.y) div 2)*0.01;
  end;

  //letzten Koordinaten merken
  MouseX := event.x;
  MouseY := event.y;
end;

(******************************************************************************)
(******************************************************************************)
//  Hauptprogramm
(******************************************************************************)
(******************************************************************************)
var Color: TSDL_Color;

begin
  // Initalisierung
  MyProgram := TMyProgram.Create('www.delphigl.com .:. Rotation- & Translationbeispiel');
  MyProgram.Start;

  with MyProgram do
  begin
    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
    glClearColor(0,0,0,0);
    //Initialisieren der Matrix
    viewmat := _New4x4;
    // Voreinstellungen der Bewegungsvariablen
    movex:=0; movey:=0; movez:=0;
    scale:=1;
    cs:=false;

    //Erstellen der Schriften
    Color.r:=255;
    Color.g:=255;
    Color.b:=255;
    Color.unused:=0;
    ZDFont:=TFont.Create(DGLSDK_DATA+'fonts/steelfis.ttf',18, Color, 0);
    BZDFont:=TFont.Create(DGLSDK_DATA+'fonts/steelfib.ttf',22, Color, TTF_STYLE_UNDERLINE);
    DDFont:=TFont.Create(DGLSDK_DATA+'fonts/monofont.ttf',38, Color, 0);
    //Einladen des Wiki-Bildes
    Textures.Load(DGLSDK_DATA+'gfx/wiki.jpg','Wiki');
  end;
  //In den Mainloop gehen
  MyProgram.Loop;
  //Ressourcen entladen
  MyProgram.Stop;
end.
