program acceleration;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE CONSOLE}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Florian Sievert (Phobeus)
  Phobeus@DelphiGL.com
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2006 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
	This sample demonstrate how to realize a collision detection using a bound sphere. Move the second sphere toward the static sphere
	in the center. If a hit is detected the color of the second sphere will become red. Note, that in this example you cannot move the
	sphere on the z-axis, however the detection will also work in a real 3d world.

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2006-01-21 Created (FS)

  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 


  FS
    Florian Sievert (Phobeus)
    email : Phobeus@DelphiGL.com
    Visit: http://www.delphigl.com

*******************************************************************************)

uses 
  dglOpenGL,

  SDL,
  easySDL;

type

  TVector2d =
  record
    X,Y:Double;
  end;
	
  TColor=
  record
    red,green,blue:Single;
  end;

  TBullet = class
  private
    aEngine:TVector2d; // Beschleunigung des Antriebes
    full:Double; // Wieviel Treibstoff hat der Antrieb noch
    pos:TVector2d;// Position
    speed:TVector2d;// Geschwindigkeit
    color:TColor;
  public
    constructor create();
    procedure Update(Seconds:TGLFloat);
    procedure draw;
  end;

  TMyProgram = class(TEasySDL)
    bulletList:array of TBullet;
    procedure DrawScene;override;
		procedure UpdateWorld(Seconds: TGLFloat); override;
    procedure handleKeyPressed(event: TSDL_KeyboardEvent); override;
    destructor destroy;
    // Erzeugt neue Geschosse und entfernt falls vorhanden alte.
    procedure NewBullets;

  end;

var
  MyProgram :TMyProgram;

const
  g=9.81;// Beschleunigung durch die Gravitation

constructor TBullet.create;
const
  START_FULL=1.0;// Wie viele Sekunden soll der Treibstoff reichen
var
  angle:double;
begin
  pos.x := 0;
  pos.y := 0;
  speed.x := 0;
  speed.y := 0;
  
  // Geschosse werden in zufeallige Richtung beschleunigt. Jedoch immer ungefaehr nach oben.

  angle := pi*(random-0.5);
  aEngine.x := random*10*sin(angle);
  aEngine.y := random*10*cos(angle) + 10;

  color.red := 1;
  color.green := random;
  color.blue := 0.5*random;

  full := START_FULL;
end;

procedure TBullet.Update(Seconds:TGLFloat);
  procedure accelerate(time:TGLFloat;aX,aY:double);
  begin
    if time <= 0 then exit;

    // neue Position berechnen:
    pos.x := 0.5*aX*sqr(time) + speed.x*time + pos.x; 
    pos.y := 0.5*aY*sqr(time) + speed.y*time + pos.y;

    // neue Geschwindigkeit berechnen(wichtig: muss nach der Positionsberechnung stehen):
    speed.x := aX*time + speed.x;
    speed.y := aY*time + speed.y;
  end;

begin

  if full >= seconds then
  begin
    accelerate(seconds,aEngine.x,aEngine.y -g); // die ganze Zeit mit vollem Antrieb
  end
  else
  begin // full < seconds
    accelerate(full,aEngine.x,aEngine.y -g);// Beschleunigung mit Antrieb
    accelerate(seconds -full,0,-g); // Beschleunigung ohne Antrieb
  end;

  // Jede Sekunde wird eine Einheit Treibstoff verbraucht:
  full := full - Seconds;
  if full < 0 then full := 0;
end;

procedure TBullet.draw;
begin
  glBegin(GL_POINTS);
    glColor3fv(@color);
    glVertex2f(pos.x,pos.y);
  glEnd();
end;

procedure TMyProgram.NewBullets;
const
  BULLETS_COUNT=400;
var
  i:integer;
begin
  if length(bulletList) = 0 then
    setLength(bulletList,BULLETS_COUNT)
  else
    for i := 0 to high(bulletList) do bulletList[i].free;

  for i := 0 to high(bulletList) do 
  begin
    bulletList[i] := TBullet.create;
  end;
end;

destructor TMyProgram.destroy;
var
  i:integer;
begin
  for i := 0 to high(bulletList) do bulletList[i].free;
end;

procedure TMyProgram.UpdateWorld(Seconds: TGLFloat);
var 
  i:integer;
begin
	for i := 0 to high(bulletList) do bulletList[i].update(Seconds);
end;

procedure TMyProgram.handleKeyPressed(event: TSDL_KeyboardEvent);
begin
  case event.keysym.sym of
    SDLK_SPACE :
    begin
      NewBullets;
    end;
  end;
end;



procedure TMyProgram.DrawScene;
var 
  i:integer;
begin
  glClearColor(0.0,0.0,0.0,1);
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );
	glLoadIdentity();	
	glTranslatef( 0.0, -0.4, -1);
  glScalef(0.1,0.1,0.1);
  glPointSize(3);
  for i := 0 to high(bulletList) do bulletList[i].draw();


end;


begin
  MyProgram := TMyProgram.Create('www.delphigl.com .:. time based acceleration >> PRESS SPACE <<');
  MyProgram.Start;
  MyProgram.NewBullets;
  MyProgram.Loop;
  MyProgram.Stop;
end.
