program glsl_laboda;

{$IFDEF fpc}
  {$MODE Delphi}
{$ELSE}
  {$APPTYPE CONSOLE}
  {$R ..\data\gfx\dgl.RES}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Jonas Scholz (i0n0s)
  jonasscholz@web.de

  Original by La Boda
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2006 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  This sample try to demonstrate how to use the arb point sprite. A point sprite might be usefull for a particle system.
	Instead of using several quads with the particle texture, you can define the position with a point. This should also save 
	bandwidth of the pipeline allowing more particles ;)

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2006-05-01 Added licence (JS)

  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 


  JS
    Jonas Scholz (i0n0s)
    email : jonasscholz@web.de

*******************************************************************************)

uses
  dglOpenGL,
  easySDL,
  sdl,
  SysUtils,
  Classes,
  figures;

type
  TMyProgram = class(TEasySDL)
    procedure DrawScene; override;
    procedure UpdateWorld(Seconds: TGLFloat); override;
  end;

var
  MyProgram: TMyProgram;
  shader_phong, shader_blinn: GLHandleARB;

  counter: Extended;
  SPEEDVARIABLE: Extended;
  second: Extended;

  useShader : Boolean;
  usePong   : Boolean;

  Shininess: GLint;

function LoadFragmentandVertexShader(FFilename: string; VFilename: string): GLHandleARB;
 var
  ProgramObject, FragmentShaderObject, VertexShaderObject: GLHandleARB;
  FShader: TStringList;
  FShaderText: String;
  FShaderLength: Integer;
  VShader: TStringList;
  VShaderText: String;
  VShaderLength: Integer;
begin
if FileExists(FFilename)= True then
if FileExists(VFilename)= True then
begin
 ProgramObject:= glCreateProgramObjectARB;

 FragmentShaderObject:= glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
 VertexShaderObject:= glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);

 FShader:= TStringList.Create;
 FShader.LoadFromFile(FFilename);
 FShaderText:= FShader.Text;
 FShaderLength:= Length(FShaderText);

 VShader:= TStringList.Create;
 VShader.LoadFromFile(VFilename);
 VShaderText:= VShader.Text;
 VShaderLength:= Length(VShaderText);

 glShaderSourceARB(VertexShaderObject, 1, @VShaderText, @VShaderLength);
 glShaderSourceARB(FragmentShaderObject, 1, @FShaderText, @FShaderLength);

 glCompileShaderARB(FragmentShaderObject);
 glCompileShaderARB(VertexShaderObject);

 glAttachObjectARB(ProgramObject, FragmentShaderObject);
 glAttachObjectARB(ProgramObject, VertexShaderObject);

 
 glDeleteObjectARB(FragmentShaderObject);
 glDeleteObjectARB(VertexShaderObject);
 glLinkProgramARB(ProgramObject);

 Result:= ProgramObject;
end
else
 Writeln('Fehler: Shader-Datei konnte nicht gefunden werden!')
end;

procedure SetLightOptionsOnInit();
 const material_specular : Array[0..3] of GLfloat = (0, 0, 0, 1);
       material_ambient  : Array[0..3] of GLfloat = (0.5, 0.5, 0.5, 1);
       material_diffuse  : Array[0..3] of GLfloat = (0.5, 0.5, 0.5, 1);

       light_position    : Array[0..3] of GLfloat = (0, 0, 0, 1);
       light_ambient     : Array[0..3] of GLfloat = (0.5, 0.5, 0.5, 1);
       light_diffuse     : Array[0..3] of GLfloat = (1, 0, 0, 1);
       light_specular    : Array[0..3] of GLfloat = (1, 1, 1, 1);
var    material_shininess: Array[0..0] of GLfloat;
begin
 material_shininess[0]:=Shininess;

 glEnable(GL_LIGHTING);
 glEnable(GL_LIGHT0);

 glMaterialfv(GL_FRONT, GL_SPECULAR, @material_specular[0]);
 glMaterialfv(GL_FRONT, GL_SHININESS, @material_shininess[0]);
 glMaterialfv(GL_FRONT, GL_AMBIENT, @material_ambient[0]);
 glMaterialfv(GL_FRONT, GL_DIFFUSE, @material_diffuse[0]);

 glLightfv(GL_LIGHT0, GL_POSITION, @light_position[0]);
 glLightfv(GL_LIGHT0, GL_AMBIENT, @light_ambient[0]);
 glLightfv(GL_LIGHT0, GL_DIFFUSE, @light_diffuse[0]);
 glLightfv(GL_LIGHT0, GL_SPECULAR, @light_specular[0]);
end;

procedure TMyProgram.DrawScene;
begin
  // Farb- und Tiefenpuffer lchen
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  // In die Modelansichtsmatrix wechseln
  glMatrixMode(GL_MODELVIEW);
  // Identittsmatrix laden
  glLoadIdentity;

  if not useShader then
    glUseProgramObjectARB(0)
  else
    if usePong then
      glUseProgramObjectARB(shader_phong)
    else
      glUseProgramObjectARB(shader_blinn);

  counter:= counter + 1*SPEEDVARIABLE;

  glTranslatef(0, 0, -2);

  glColor3f(1, 1, 1);

  glPushMatrix;

  glRotatef(counter/10+90, 0, 1, 0);
  glRotatef(counter/10, 0, 0, 1);
  glTranslatef(-2, 0, 0);

  DrawSphere(0, 0, 0, 0.1, 5);
  SetLightOptionsOnInit;
  glPopMatrix;

  DrawSphere(0, 0, 0, 0.75, 40);
end;

procedure TMyProgram.UpdateWorld(Seconds: TGLFloat);
var Caption: String;
begin
  second:=second+Seconds;
  if second>0.25 then
  begin
    second:=0;
    if keydown(SDLK_S) then useShader:=not useShader;
    if keydown(SDLK_N) then usepong:=not usepong;
    if keydown(SDLK_KP_PLUS) or keydown(SDLK_PLUS) then Inc(Shininess);
    if keydown(SDLK_KP_MINUS) or keydown(SDLK_MINUS) then Dec(Shininess);

    Caption:='[S]hader: '+BooltoStr(useShader);
    if useShader then
      if usePong then Caption:=Caption+' | '+InttoStr(shininess)+' | pho[n]g-Shader'
      else Caption:=Caption+' | '+InttoStr(shininess)+' | bli[n]n-Shader + pho[n]g-Shader';

    SDL_WM_SetCaption(PChar('www.delphigl.com .:. FPS: '+InttoStr(Round(FPS))+
                            ' GLSL 2 von La_Boda | '+Caption),'');
  end;
  SPEEDVARIABLE:=Seconds*500;
end;

begin
  MyProgram:=TMyProgram.Create('www.delphigl.com .:. GLSL 2 von La_Boda');

  MyProgram.Start;

  useShader:=False;
  usePong:=True;

  shader_phong:= LoadFragmentAndVertexShader('lbphong.fp', 'lbphong.vs');
  shader_blinn:= LoadFragmentAndVertexShader('lbblinn.fp', 'lbblinn.vs');

  Shininess:= 120;

  MyProgram.Loop;
  MyProgram.Stop;
end.
