program glsl_sdl;

{$IFDEF fpc}
  {$MODE Delphi}
{$ELSE}
  {$APPTYPE CONSOLE}
  {$R ..\data\gfx\dgl.RES}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Jonas Scholz (i0n0s)
  jonasscholz@web.de

  Original by Soulchild
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2006 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  This sample try to demonstrate how to use the arb point sprite. A point sprite might be usefull for a particle system.
	Instead of using several quads with the particle texture, you can define the position with a point. This should also save 
	bandwidth of the pipeline allowing more particles ;)

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2006-05-01 Added licence (JS)

  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 


  JS
    Jonas Scholz (i0n0s)
    email : jonasscholz@web.de

*******************************************************************************)

uses
  dglOpenGL,
  easySDL,
  easySDLTextures,
  sdl,
  SysUtils,
  Classes;

{$I ../dglsdk.inc}

type
  TMyProgram = class(TEasySDL)
    procedure DrawScene; override;
    procedure UpdateWorld(Seconds: TGLFloat); override;
  end;

  TVertex3f=array[0..2]of GLfloat;

  TLight = record
    pos,color:TVertex3f;
    strength:glFloat;
  end;

var
  MyProgram: TMyProgram;

  Mytime: Extended;

  lightobj:PGLUQuadric;
  light:array [0..2]of TLight;        //Array fr die 3 Lichter
  ProgramObject: glHandleARB;

procedure glSlang_Load(pVShaderFile,pFShaderFile:String);
var
  FShader    : GLHandleARB;
  VShader    : GLHandleARB;
  SSource    : TStringList;
  Source     : String;
  SLen       : Integer;
begin

  ProgramObject:=glCreateProgramObjectARB;

  SSource      :=TStringList.Create;
  try
    if pVShaderFile<>'' then
    begin
      VShader:=glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
      SSource.LoadFromFile(pVShaderFile);
      Source:=SSource.Text;
      SLen  :=Length(Source);
      glShaderSourceARB(VShader, 1, @Source, @SLen);
      glCompileShaderARB(VShader);
      glAttachObjectARB(ProgramObject, VShader);
      glDeleteObjectARB(VShader);
    end;

    if pFShaderFile <>'' then
    begin
      FShader:=glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
      SSource.LoadFromFile(pFShaderFile);
      Source:=SSource.Text;
      SLen  :=Length(Source);
      glShaderSourceARB(FShader, 1, @Source, @SLen);
      glCompileShaderARB(FShader);
      glAttachObjectARB(ProgramObject, FShader);
      glDeleteObjectARB(FShader);
    end;

    glLinkProgramARB(ProgramObject);
  finally
    SSource.Free;
  end;
end;

procedure TMyProgram.DrawScene;
const
  w: glFloat=3;
begin
  // Farb- und Tiefenpuffer l�hen
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  // In die Modelansichtsmatrix wechseln
  glMatrixMode(GL_MODELVIEW);
  // Identit�smatrix laden
  glLoadIdentity;

  with light[0] do
  begin
    //Berechnung der Position der Lichtquelle
    pos[0]:=Cos(MyTime/1500)*Sin(MyTime/700-2)*4;
    pos[1]:=Sin(MyTime/1600)*Cos(MyTime/600-1)*4;
    pos[2]:=Sin(MyTime/1700)*Sin(MyTime/500-3)*4;
    //Bewegen auf die neue Position
    glTranslatef(pos[0],pos[1],pos[2]-14);
    //Setzen der Farbe
    glColor3fv(@color);
  end;
  //Zeichnen der Kugel (sp�ere Lichtquelle)
  gluSphere(lightobj,0.2,8,8);

  //Zurcksetzen der Position
  glLoadIdentity;
  with light[1] do
  begin
    pos[0]:=Cos(MyTime/1700)*Sin(MyTime/1500-3)*4;
    pos[1]:=Sin(MyTime/1600)*Cos(MyTime/1700-1)*4;
    pos[2]:=Sin(MyTime/1500)*Sin(MyTime/1600-2)*4;

    glTranslatef(pos[0],pos[1],pos[2]-14);
    glColor3fv(@color);
  end;
  gluSphere(lightobj,0.2,8,8);

  glLoadIdentity;
  with light[2] do
  begin
    pos[0]:=Cos(MyTime/1500)*Sin(MyTime/1600-1)*4;
    pos[1]:=Sin(MyTime/1700)*Cos(MyTime/1700-2)*4;
    pos[2]:=Sin(MyTime/1600)*Sin(MyTime/1500-3)*4;
    glTranslatef(pos[0],pos[1],pos[2]-14);
    glColor3fv(@color);
  end;
  gluSphere(lightobj,0.2,8,8);

  glLoadIdentity;
  //Setzen des Shaders, ab jetzt gezeichnete Objekte werden mit dem Shader behandelt
  glUseProgramObjectARB(ProgramObject);
  //�ergabe der Positionen der Lichtquellen an den Shader
  with light[0] do
    glUniform4fARB(glGetUniformLocationARB(ProgramObject,'LightPos0'),pos[0],pos[1],pos[2],1);
  with light[1] do
    glUniform4fARB(glGetUniformLocationARB(ProgramObject,'LightPos1'),pos[0],pos[1],pos[2],1);
  with light[2] do
    glUniform4fARB(glGetUniformLocationARB(ProgramObject,'LightPos2'),pos[0],pos[1],pos[2],1);

  //Zurcksetzen der Farbe in OpenGL
  glColor3f(1,1,1);
  glTranslatef(0,0,-14);
  //Zeichnen der W�de
  glBegin(GL_QUADS);
    glTexCoord2f(0,0);glVertex3f(-5, 5,-5);
    glTexCoord2f(0,1);glVertex3f(-5,-5,-5);
    glTexCoord2f(1,1);glVertex3f( 5,-5,-5);
    glTexCoord2f(1,0);glVertex3f( 5, 5,-5);

    glTexCoord2f(0,0);glVertex3f(-5, 5,-5);
    glTexCoord2f(1,0);glVertex3f(-5, 5, 5);
    glTexCoord2f(1,1);glVertex3f(-5,-5, 5);
    glTexCoord2f(0,1);glVertex3f(-5,-5,-5);

    glTexCoord2f(0,0);glVertex3f( 5, 5,-5);
    glTexCoord2f(0,1);glVertex3f( 5,-5,-5);
    glTexCoord2f(1,1);glVertex3f( 5,-5, 5);
    glTexCoord2f(1,0);glVertex3f( 5, 5, 5);

    glTexCoord2f(0,0);glVertex3f(-5,-5, 5);
    glTexCoord2f(1,0);glVertex3f( 5,-5, 5);
    glTexCoord2f(1,1);glVertex3f( 5,-5,-5);
    glTexCoord2f(0,1);glVertex3f(-5,-5,-5);

    glTexCoord2f(0,0);glVertex3f(-5, 5, 5);
    glTexCoord2f(0,1);glVertex3f(-5, 5,-5);
    glTexCoord2f(1,1);glVertex3f( 5, 5,-5);
    glTexCoord2f(1,0);glVertex3f( 5, 5, 5);
  glEnd;
  //Ausschalten des Shaders, wichtig!
  glUseProgramObjectARB(0);
end;

procedure TMyProgram.UpdateWorld(Seconds: TGLFloat);
begin
  Mytime:=Mytime + 1000*Seconds;
  SDL_WM_SetCaption(PChar('www.delphigl.com .:. GLSL by Soulchild     FPS: '+InttoStr(Round(FPS))),nil);
end;

begin
  MyProgram:=TMyProgram.Create('www.delphigl.com .:. GLSL by Soulchild');

  MyProgram.Start;
  //Laden des Shaders
  glSlang_Load('scVShader.vs','scFShader.fp');
  //Erstellen des gluQuadrics fr gluSphere
  lightobj:=gluNewQuadric;
  //Setzen der Lichtquellen (Farbe und St�ke)
  light[0].color[0]:=1.0;
  light[0].color[1]:=0.0;
  light[0].color[2]:=0.0;
  light[1].color[0]:=0.0;
  light[1].color[1]:=1.0;
  light[1].color[2]:=0.0;
  light[2].color[0]:=0.0;
  light[2].color[1]:=0.0;
  light[2].color[2]:=1.0;
  light[0].strength:=4;
  light[1].strength:=4;
  light[2].strength:=4;
  //Laden der Mauertexture
  textures.Load(DGLSDK_DATA+'/gfx/wall.jpg','Walltex');

  MyProgram.Loop;
  MyProgram.Stop;
end.
