program sample_sdl_input;

{$IFDEF FPC}
  {$LINKLIB c}
  {$mode Delphi}
  {$H+}
{$ELSE}
  {$APPTYPE Console}
{$ENDIF}

uses
	SDL,
	SysUtils,
	easySDLTextures,
	dglOpenGL;

{$I ../dglsdk.inc}

const
	WINDOW_CAPTION	=  'http://www.delphigl.com/ .:. SDL-Input Sample';

	SCREEN_WIDTH		= 320;
	SCREEN_HEIGHT		= 160;
	SCREEN_BPP			= 32;								// screen depth

	FPS_INTERVAL		= 250;							// calc every second FPS

var
	// SDL video
	surface : PSDL_Surface;
	videoInfo : TSDL_VideoInfo;
	videoflags : Uint32;

	// misc
	done        : Integer;                    // programm status (0 = rund, -1 abort)   
	FPSCount    : LongInt;

  duck : Boolean;
  dir  : Integer;
  anim : Integer;

  TuxCoord : array[0..7, 0..3] of Single = (
  (0.0000,0.5000,   0.1875,1.0000),
  (0.1875,0.5000,   0.3750,1.0000),
  (0.3750,0.5000,   0.5625,1.0000),
  (0.5625,0.5000,   0.7500,1.0000),
  (0.7500,0.5000,   0.9375,1.0000),

  (0.0000,0.0000,   0.1875,0.5000),
  (0.1875,0.0000,   0.3750,0.5000),
  (0.3750,0.0000,   0.5625,0.5000));

//----------------------------------------------------------------------------
// quiting app - get sure everything is released
//----------------------------------------------------------------------------
procedure Quit_App;
begin
	// release ressources
	Writeln('Release Ressources...');

	SDL_FreeSurface(surface);
	SDL_Quit;

	// time to say goodbye
	WriteLn('Bye Bye :)');
	Halt(0);
end;


//----------------------------------------------------------------------------
// resize action
//----------------------------------------------------------------------------
function glResizeWindow( width : integer; height : integer ) : Boolean;
begin
	// prevent division by zero
	if ( height = 0 ) then height := 1;

	// refresh viewport and projection matrix
	glViewport( 0, 0, width, height );

  glMatrixMode(GL_PROJECTION);  // Change Matrix Mode to Projection
  glLoadIdentity();             // Reset View
  glOrtho(0, 320, 0, 160, 0, 10);
  glMatrixMode(GL_MODELVIEW);   // Change Projection to Matrix Mode

	// reset world matrix
	glLoadIdentity;

	// action successfull
	result := true;
end;

//----------------------------------------------------------------------------
// timer action
//----------------------------------------------------------------------------
function glTimer( interval : UInt32; param : Pointer ) : UInt32;
begin
	SDL_WM_SetCaption(pchar( WINDOW_CAPTION + ' ['+IntToStr(FPSCount * 1000 DIV FPS_INTERVAL)+' FPS]') , nil);
	FPSCount := 0;
  if not duck then
  begin
    Inc(anim);
    if anim>5 then
    begin
    	anim := 0;
    end;
  end;
	Result := interval;
end;


//----------------------------------------------------------------------------
// Initialize SDL
//----------------------------------------------------------------------------
procedure Init_SDL;
begin
	// Init SDL
	WriteLn('Initalisiere SDL...');

	if ( SDL_Init( SDL_INIT_VIDEO ) < 0 ) then
	begin
		WriteLn('->Init SDL failed:'+SDL_GetError);
		Quit_App;
	end;

	// Get informations about graphic card
	videoInfo := SDL_GetVideoInfo^;
	if ( @videoInfo = nil ) then
	begin
		WriteLn('->Unable to detect graphc card');
		Quit_App;
	end else WriteLn('Graphic card: ', (videoInfo.video_mem DIV 1024), ' MB');

	// Initalisierung der Surface
	surface := SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, videoflags);
	if ( surface = nil ) then
	begin
		WriteLn('->Creating surface failed:'+SDL_GetError);
		Quit_App;
	end;
      
	// flags for SDL
	videoFlags := SDL_OPENGL or			// OpenGL support
					      SDL_DOUBLEBUF or			// Enable Double Buffering
					      SDL_HWPALETTE;				// hardware palette

	if (videoInfo.hw_available <> 0) then
	begin
		WriteLn('Hardware Surface enabled...');
		videoFlags := videoFlags or SDL_HWSURFACE;
	end
	else
	begin
		WriteLn('Software Surface enabled...');
		videoFlags := videoFlags or SDL_SWSURFACE
	end;

	// hardware support
	if ( videoInfo.blit_hw <> 0 ) then
	begin
		WriteLn('Hardware support');
		videoFlags := videoFlags or SDL_HWACCEL;
	end else WriteLn('Software support');
	      
	// setup sdl/opengl attributes
	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	
	// setup window caption
	SDL_WM_SetCaption( WINDOW_CAPTION , nil);

	// Init surface
	surface := SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, SDL_OPENGL );
	if ( surface = nil ) then
	begin
		WriteLn('->OpenGL-Support for Surface failed!');
		Quit_App;
	end;

	// Init timer
	SDL_Init(SDL_INIT_TIMER);
end;


//----------------------------------------------------------------------------
// Initialize OpenGL
//----------------------------------------------------------------------------
procedure Init_OpenGL;
begin
	// Init OpenGL
	InitOpenGL;
	ReadExtensions;

	// Give some feedback into console that might be usefull for debuggig
	WriteLn('Extensions:  '+glGetString(GL_EXTENSIONS));
	WriteLn('------');
	WriteLn('Renderer: '+glGetString(GL_RENDERER));
	WriteLn('Version: '+glGetString(GL_VERSION));
	WriteLn('Vendor:  '+glGetString(GL_VENDOR));

	// set up default values for state machine
	glEnable(GL_TEXTURE_2D);									// Enable Texture mapping
	glShadeModel(GL_SMOOTH);									// Enable Smooth Shading
	glClearColor(0.0, 0.0, 0.0, 0.5);						// Clear screen with black
	glClearDepth(1.0);											// setup Depth Buffer
	glEnable(GL_DEPTH_TEST);									// Enable Depth Testing
	glDepthFunc(GL_LEQUAL);										// Setup Depth Testing
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// better interpolation
end;


//----------------------------------------------------------------------------
// React on program events
//----------------------------------------------------------------------------
procedure glHandleEvents;
var event : TSDL_Event;
begin
	while ( SDL_PollEvent( @event ) = 1 ) do
	begin
		case event.type_ of

		// Quit applications
		SDL_QUITEV :
		begin
			Done := -1;
		end;

		// key was pressed
		SDL_KEYDOWN :
		begin
    	case event.key.keysym.sym of
    		SDLK_ESCAPE :
    		begin
    			Done := -1;
    		end;

    		SDLK_LEFT:
    		begin
    			dir := 0;
    		end;

    		SDLK_RIGHT:
    		begin
    			dir := 1;
		    end;

        SDLK_DOWN:
        begin
          duck:=True;
        end;
    	end;
		end;

    SDL_KEYUP :
    begin
      case event.key.keysym.sym of
        SDLK_DOWN:
        begin
          duck:=False;
        end;
      end;
    end;

		// change window size
		SDL_VIDEORESIZE :
		begin
			surface := SDL_SetVideoMode( event.resize.w, event.resize.h, SCREEN_BPP, videoflags );

			if ( surface = nil ) then
			begin
				Writeln('->Surface list');
				Quit_App;
			end;
			glResizeWindow( event.resize.w, event.resize.h );
		end;
		
		end;
    end;
end;


//----------------------------------------------------------------------------
// draw the scene
//----------------------------------------------------------------------------
procedure glDrawScene;
begin
	// clear screen and depth buffer
	glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );
	glClearColor(1,1,1,0.0);

  glLoadIdentity;

	glTranslatef(136,48,0);

  if duck then anim:=6;

  Textures.Bind('Animation');
  if dir=0 then
  begin
    glBegin(GL_QUADS);
  		glTexCoord2f(TuxCoord[anim][0],TuxCoord[anim][1]); glVertex2f( 48, 0 );
   		glTexCoord2f(TuxCoord[anim][2],TuxCoord[anim][1]); glVertex2f( 0 , 0 );
   		glTexCoord2f(TuxCoord[anim][2],TuxCoord[anim][3]); glVertex2f( 0 , 64);
   		glTexCoord2f(TuxCoord[anim][0],TuxCoord[anim][3]); glVertex2f( 48, 64);
   	glEnd();
  end
  else
  begin
    glBegin(GL_QUADS);
  		glTexCoord2f(TuxCoord[anim][0],TuxCoord[anim][1]); glVertex2f( 0 , 0 );
   		glTexCoord2f(TuxCoord[anim][2],TuxCoord[anim][1]); glVertex2f( 48, 0 );
   		glTexCoord2f(TuxCoord[anim][2],TuxCoord[anim][3]); glVertex2f( 48, 64);
   		glTexCoord2f(TuxCoord[anim][0],TuxCoord[anim][3]); glVertex2f( 0 , 64);
   	glEnd();
  end;
end;


begin
	// Init program
	Writeln('Starting...');
    
	Init_SDL;
	Init_OpenGL;

  textures.Load(DGLSDK_DATA+'gfx/tux.png','Animation');

	glResizeWindow( SCREEN_WIDTH, SCREEN_HEIGHT );

	// Activate timer for fps calculation
	SDL_AddTimer(FPS_INTERVAL,@glTimer,nil);

	// Enter main loop
	Writeln('Start rendering...');
	while ( Done <> -1 ) do
	begin
		glHandleEvents;
		glDrawScene;

		// show framebuffer on screen
		SDL_GL_SwapBuffers;
		Inc(FPSCount);
	end;

	// get sure programm gets terminated
	Quit_App;
end.
