program sample_sdl_mixer;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE Console}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Florian Sievert (Phobeus)
  Phobeus@DelphiGL.com
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2005 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  Documetation on SDL_mixer is poor. However you will find it pretty is to use, if you get the first lines running.
  hopefull this small sample will help to find a good start.

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2005-09-01 Add application icon (FS)
  2005-07-23 Created (FS)


  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 

  FS
    Florian Sievert (Phobeus)
    email : Phobeus@DelphiGL.com
    Visit: http://www.delphigl.com

*******************************************************************************)
uses
  SDL,
  SDL_mixer,
  SDL_image,
  SysUtils;

{$I ../dglsdk.inc}

const
  WINDOW_CAPTION		=       'http://www.delphigl.com/ .:. SDL_Mixer Sample'; 

var
  phaser: PMix_Chunk;
  boom : PMix_Chunk;
  music: PMix_Music;

  audio_rate: integer = MIX_DEFAULT_FREQUENCY;	
  audio_format: Uint16 = MIX_DEFAULT_FORMAT;
  audio_channels: integer = MIX_DEFAULT_CHANNELS;

  done : Integer;

  surface :PSDL_Surface;
  videoflags  : Uint32;

procedure glHandleEvents;
var event : TSDL_Event;
begin
  while ( SDL_PollEvent( @event ) = 1 ) do
  begin
    case event.type_ of
      SDL_QUITEV: done := -1;
      SDL_KEYDOWN : done := -1;
    end;
  end;
end;

begin
  Writeln('Starting...');
  Randomize;
  videoFlags := 0;

  // first we get SDL and SDL_Audio running
  WriteLn('Init audio...');
  if ((SDL_Init(SDL_INIT_AUDIO or SDL_INIT_VIDEO) < 0)) then
  begin
    WriteLn('Unable to init SDL_Audio or SDL_Video');
    halt;
  end;

  // Init surface. In fact we do not need a video output, but SDL
  // expect a surface for event handling.
  surface := SDL_SetVideoMode(600, 200, 32, videoflags);
  if not assigned(surface) then
  begin
    WriteLn('Unable to create surface:'+SDL_GetError);
    Halt;
  end;

  SDL_WM_SetCaption( WINDOW_CAPTION , nil);
  SDL_WM_SetIcon(IMG_Load('../data/gfx/dgl_icon.png'), 0); 

  // request a audio device with system default settings
  if ( Mix_OpenAudio( audio_rate, AUDIO_S16SYS, audio_channels, 4096) < 0) then
  begin
    WriteLn('Unable open audio:', SDL_GetError);
    halt;
  end;

  // if not avaible, we shall get something that works...
  Mix_QuerySpec( audio_rate, audio_format, audio_channels);
  WriteLn(Format('Opened audio at %d Hz %d bit %s', [audio_rate,
          (audio_format and $FF), 'stereo']));


  // loading effects and music
  music := Mix_LoadMUS(DGLSDK_DATA+'sounds/moon6.xm');
  phaser := Mix_LoadWAV(DGLSDK_DATA+'sounds/phaser.wav');
  boom := Mix_LoadWAV(DGLSDK_DATA+'sounds/boom.wav');

  // Start music, play in unfinite loop
  //Mix_PlayMusic(music, -1);

  // But this time we fade in music softly
  Mix_FadeInMusic(music,-1,3000);
  SDL_delay(3000);

  WriteLn('Press a key to leave.');
  while ( Done <> -1 ) do
  begin			
    // check if a key is pressed to leave app
    glHandleEvents;

    // yeah, yeah, I know. A nice battle szene would be nice, but
    // a small simulation should also do the work ;-)

    // play big gun 
    if random(40) = 13 then 
    begin
      // we are using a random sound to play trick on the user.
      // it will sound far or less near user yet :)
      Mix_Volume(1,random(40)+20);			
      Mix_PlayChannel(1, boom, 0);
    end;

    // play phaser
    if random(20) = 13 then 
    begin
      // same dirty trick here, but we also randomize the amount of loops
      Mix_Volume(2,random(40));
      Mix_PlayChannel(2, phaser, random(3)+1);
    end;

    // giving CPU a small "break"
    SDL_Delay(50);
  end;

  // fade smooth out.
  Mix_FadeOutMusic(2000);
  SDL_Delay(2000);

  // release everything
  Mix_HaltMusic();

  // release everything
  Mix_FreeMusic(music);
  Mix_FreeChunk(boom);
  Mix_FreeChunk(phaser);

  Mix_CloseAudio;

  // and say goodbye!
  SDL_Quit;
  WriteLn('Bye Bye ;)');
end.
