program sdl_rwops;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE Console}
{$ENDIF}

uses
  SysUtils,

  SDL,
  SDL_Image,
  SDLutils,
  dglOpenGL;

const
  WINDOW_CAPTION	= 'http://www.delphigl.com/ .:. SDL-RWops-Sample';
  WINDOW_RESIZE		= FALSE;  // allow window resize?
  FULLSCREEN			= FALSE;  // use fullscreen

  SCREEN_WIDTH		= 128;    // width of screen
  SCREEN_HEIGHT		= 128;    // height of screen
  SCREEN_BPP			= 32;     // bit per pixel

var
  surface     : PSDL_Surface;
  tgasurface  : PSDL_Surface;

  videoInfo   : TSDL_VideoInfo;
  videoflags  : Uint32;

  done        : Integer;


procedure Quit_App;
begin
  // release ressources
  Writeln('Release Ressources...');

  if (FULLSCREEN) then SDL_WM_ToggleFullScreen(surface);
  SDL_FreeSurface (surface) ;
  SDL_Quit;

  WriteLn('Bye Bye ;)');
  Halt(0);
end;


//----------------------------------------------------------------------------
// handle keyboard inpu
//----------------------------------------------------------------------------
procedure glHandleKeyPress(keysym: TSDL_keysym);
begin
  case keysym.sym of
    SDLK_ESCAPE : done := -1;
  end;
end;


//----------------------------------------------------------------------------
// handle sdl events
//----------------------------------------------------------------------------
procedure glHandleEvents;
var event : TSDL_Event;
begin
  // process events
  while (SDL_PollEvent(@event) = 1) do
  begin
    case event.type_ of
      // quit signal
      SDL_QUITEV :
      begin
        Done := -1;
      end;

      // key was pressed
      SDL_KEYDOWN :
      begin
        glHandleKeyPress( event.key.keysym );
      end;

      // window was resized
      SDL_VIDEORESIZE :
      begin
        surface := SDL_SetVideoMode(event.resize.w,event.resize.h,SCREEN_BPP,videoflags);

        if ( surface = nil ) then
        begin
          Writeln('surface lost');
          Quit_App;
        end;
      end;
    end;
  end;
end;


procedure Init_SDL;
begin
  // Init SDL
  WriteLn('Init SDL...');

  if ( SDL_Init( SDL_INIT_VIDEO ) < 0 ) then
  begin
    WriteLn('Failed to init SDL: '+SDL_GetError);
    Quit_App;
  end;

  // get information about graphic card
  videoInfo := SDL_GetVideoInfo^;
  if (@videoInfo = nil) then
  begin
    WriteLn('Unable to detect graphic card');
    Quit_App;
  end;

  // set sdl flags: we want a opengl surface
  videoFlags := SDL_DOUBLEBUF or SDL_HWPALETTE;

  if (videoInfo.hw_available <> 0) then
  begin
    WriteLn('Hardware Surface enabled...');
    videoFlags := videoFlags or SDL_HWSURFACE;
  end
  else
  begin
    WriteLn('Software Surface enabled...');
    videoFlags := videoFlags or SDL_SWSURFACE
  end;

  // hardware blit?
  if (videoInfo.blit_hw <> 0) then
  begin
    WriteLn('Hardware blit');
    videoFlags := videoFlags or SDL_HWACCEL;
  end else WriteLn('Software blit');

  // set opengl attributes
  SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

  if (WINDOW_RESIZE) then videoflags := videoFlags or SDL_RESIZABLE;    // Enable window resizing

  // set window propterties
  SDL_WM_SetCaption(WINDOW_CAPTION, nil);
  SDL_WM_SetIcon(IMG_Load('../data/gfx/dgl_icon.png'), 0);

  // Init a surface
  surface := SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, videoflags);
  if not assigned(surface) then
  begin
    WriteLn('Failed to init surface: '+SDL_GetError);
    Quit_App;
  end;

  // switch to fullscrene?
  if (FULLSCREEN) then SDL_WM_ToggleFullScreen(surface);

  // Init sdl timer
  SDL_Init(SDL_INIT_TIMER);
end;


//Spiegelt eine Surface, kopiert aus easySDLtextures
procedure Flip(Surface: PSDL_Surface);
var 
  x,y: Integer;
  a,b: UInt32;
begin
  for y:=0 to ((surface.h-1) div 2) do
    for x:=0 to surface.w-1 do
    begin
      //reads the pixels and swaps them
      a:=SDL_GetPixel(Surface, x, y);
      b:=SDL_GetPixel(Surface, x, surface.h-1-y);
      SDL_PutPixel(Surface, x, y, b);
      SDL_PutPixel(Surface, x, surface.h-1-y, a);
    end;
end;


function LoadTGA(Filename: String; out Surface: PSDL_Surface): Boolean;
var
  //TGA-Header aus dem Wiki
  TGAHEADER : packed record
    tfType : Byte;
    tfColorMapType : Byte;
    tfImageType : Byte;
    tfColorMapSpec : Array[0..4] of Byte;
    tfOrigX : Word;
    tfOrigY : Word;
    tfWidth : Word;
    tfHeight : Word;
    tfBpp : Byte;
    tfImageDes : Byte;
  end;
  //rwop fr die Datei
  TGArwop   : PSDL_RWops;
  rmask, gmask, bmask: Uint32;

  Width, Height : Integer;
  ColorDepth    : Integer;
  ImageSize     : Integer;
begin
  result:= false;
  //Datei muss existieren
  if FileExists(Filename) then
  begin
    //Erstellen des rwop mit Lesezugriff im binren Modus
    TGArwop:=SDL_RWfromFile(PChar(Filename),'rb');
    if TGArwop=nil then writeln('Kritischer Fehler');
    //Einlesen des TGA-Headers
    SDL_RWRead(TGARWop,@TGAHeader,SizeOf(TGAHeader),1);
  end
  else
  begin
    Writeln(Filename+' konnte nicht gefunden werden');
    Exit;
  end;

  //Wir untersttzen aktuell nur unkomprimierte Truecolorbilder
  if (TGAHeader.tfImageType <> 2) then
  begin
    SDL_RWClose(TGARWop);
    Writeln('Datei nicht untersttzt');
    Exit;
  end;

  Width  := TGAHeader.tfWidth;
  Height := TGAHeader.tfHeight;
  ColorDepth := TGAHeader.tfBPP;
  ImageSize  := Width*Height*(ColorDepth div 8);

  if SDL_BYTEORDER<>SDL_BIG_ENDIAN then
  begin
    //TGA benutzt Little Endian, SDL hier Big Endian, damit wird aus BGR->RGB
    rmask:=$ff0000;
    gmask:=$00ff00;
    bmask:=$0000ff;
  end else
  begin
    rmask:=$0000ff;
    gmask:=$00ff00;
    bmask:=$ff0000;
  end;
  //Erstellen der Surface. surface.pixels zeigt damit auf einem leeren Bereich mit ausreichend Platz
  surface:=SDL_CreateRGBSurface(SDL_SWSURFACE, Width, Height, Colordepth, rmask, gmask, bmask, 0);
  if (surface=nil) then
  begin
    writeln('CreateRGBSurface fehlgeschlagen');
    exit;
  end;
  //Lesen der Bildinformationen in die Surface
  SDL_RWRead(TGARWop, surface.pixels, ImageSize, 1);
  Result:=True;
end;

////////////////////////////////////////////////////////////////////////////////
//Aufgabe:                                                                    //
//  Schreiben einer Funktion die eine Surface in einer TGA speichert.         //
//Hinweise:                                                                   //
//  Informationen zum TGA-Format findet man auf www.wotsit.org                //
//  Endian sollte beachtet werden.                                            //
//  Surface muss gespiegelt werden.                                           //
////////////////////////////////////////////////////////////////////////////////


(******************************************************************************)
(******************************************************************************)
//  main program
(******************************************************************************)
(******************************************************************************)
begin
  // Init
  Writeln('Starting...');

  Init_SDL;

  //Laden der TGA in eine Surface
  if LoadTGA('wiki.tga',tgasurface) then
  begin
    //Spiegeln der Surface
    Flip(tgasurface);
    //Kopieren der Surface in die des Fensters
    if SDL_BlitSurface(tgasurface,nil,surface,nil)=0 then Writeln('Blitting Erfolgreich');
    //Freigeben der Surface
    SDL_FreeSurface(tgasurface);
  end;

  Writeln('Start rendering...');

  // enter main loop
  while ( Done <> -1 ) do
  begin
    glHandleEvents;
  end;

  // quite application
  Quit_App;
end.
