program sample_sdl_mutex;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE Console}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the dglsdk for linux
  Visit us @ http://www.delphigl.com
  Maintained by Florian Sievert (Phobeus)
  Phobeus@DelphiGL.com
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2005 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  We will just simulate a shared ressource usage. We increase target and decrease it after right after it. As long as we are locking
  our mutex, target should always be 0 after each loop call. If target is > 1 something had gone terrible wrong. Another thread is
  currently running and uses target. In application this would cause any kind of trouble. Try to check it out and remove mutex lock and
  unlock!

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)

  History
  -------
  2005-08-14 Created (FS)


  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 

  FS
    Florian Sievert (Phobeus)
    email : Phobeus@DelphiGL.com
    Visit: http://www.delphigl.com

*******************************************************************************)

uses	
  SDL;

var 
  done: Integer;									// application status
  threads:array[1..5] of PSDL_Thread;		// five threads
  target : Integer;								// our simulated common ressource 
  lock: PSDL_mutex;								// a mutex 
  c, times : Integer;							// some help vars

function thread_func:integer;
var count: Integer;
begin
  count := 0;

  // Thread enters his loop
  while (done <> 0) do
  begin
    // Lock Mutex, if mutex is already locked, it will not return until the mutex is unlocked
    SDL_LockMutex(lock);

    inc(target);
    // With Mutex lock, we should never see this!
    if (target > 1) then
    begin
      WriteLn('Uh oh, somebody else is using the variable!');
    end;
    dec(target);

    // Unlock the mutex. This is very important!
    SDL_UnLockMutex(lock);
    inc(count);
  end;

  // return how many times the thread loop was processed
  result := count;
end;

begin
  // app starts
  done := 1;
  times := 0;

  // Creating a mutex
  lock := SDL_CreateMutex();

  // Creating 5 threads
  WriteLn('Creating Threads...');
  for c:=1 to 5 do threads[c] := SDL_CreateThread(@thread_func, nil);

  // we just wait here for some seconds and then tell threads to stop working
  SDL_Delay(5000);
  SDL_LockMutex(lock);
  done := 0;
  SDL_UnLockMutex(lock);

  // now we wait until all thread are finished
  for c:=1 to 5 do
  begin
    SDL_WaitThread(threads[c], times);
    WriteLn('Thread ',c, ' used the target ',times,' times');
  end;

  // be a good boy! Release mutex ;)
  SDL_DestroyMutex(lock);

  SDL_Quit;
  WriteLn('ByeBye ;)');
end.
