unit uFonts;
(*
  Convert from c++ to delphi and made an object.

  font.cpp
  Cone3D SDL font routines.
  Made by Marius Andra 2002
  http://cone3d.gamedev.net

  You can use the code for anything you like.
  Even in a commercial project.
  But please let me know where it ends up.
  I'm just curious. That's all.
*)

interface
{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}
uses SysUtils,
     SDL;

type
  PByteArray       = ^TByteArray;
  TByteArray       = packed array[0..0] of Byte;

  TSDLFont         = class(TObject)
  protected
    fFont            : PSDL_Surface;     // The SDL Surface for the font image
    fWidth           : Integer;          // Width of the SDL Surface (same as the height)
    fcharWidth       : Integer;          // Width of one block character in the font (fontWidth/16)
    fWidths          : PByteArray;       // Real widths of all the fonts
    fData            : PByteArray;       // The raw font data
  public
    constructor Create(const aFontdir : String; r, g, b, a : Single); overload;
    constructor Create(const aFontdir : String; r, g, b : Single); overload;
    constructor Create(const aFontdir : String); overload;
    destructor Destroy; override;

    procedure fontDrawIMG(Screen, Img : PSDL_Surface; x, y, w, h, x2, y2 : Integer);

    procedure TextOut(aScreen : PSDL_Surface; x, y : Integer; const aText : String);
    function  TextWidth(const aText : String) : Integer;
  end;

implementation

// this function loads in our font file
constructor TSDLFont.Create(const aFontdir : String; r, g, b, a : Single);
var
  fpText           : System.Text;
  fpByte           : THandle;
  sFilename        : String;
  cAlpha           : Byte;
  Width            : Integer;
  iLoop            : Integer;
  tempSurface      : PSDL_Surface;
  rMask            : Cardinal;
  gMask            : Cardinal;
  bMask            : Cardinal;
  aMask            : Cardinal;
begin
  inherited Create;
  
  // find out about the size of a font from the ini file
  sFilename := Format('%s/%s', [afontDir, 'font.ini']);

  AssignFile(fpText, sFilename);
  Reset(fpText);
  try
    ReadLn(fpText, Width);
  finally
    CloseFile(fpText);
  end;

  // let's create our font structure now
  fData      := AllocMem(Width * Width * 4);
  fWidth     := Width;
  fcharWidth := Trunc(Width / 16);

  // open the font raw data file and read everything in
  sFilename := Format('%s/%s', [aFontDir, 'font.raw']);
  fpByte := FileOpen(sFilename, fmOpenRead);
  try
    for iLoop := 0 to Pred(Width * Width) do begin
      FileRead(fpByte, cAlpha, SizeOf(cAlpha));
      fData^[iLoop * 4 + 0] := Trunc(255 * r);
      fData^[iLoop * 4 + 1] := Trunc(255 * g);
      fData^[iLoop * 4 + 2] := Trunc(255 * b);
      fData^[iLoop * 4 + 3] := cAlpha;
    end;
  finally
    FileClose(fpByte);
  end;

  // now let's create a SDL surface for the font
  if (SDL_BYTEORDER <> SDL_BIG_ENDIAN) then begin
    rmask := $ff000000;
    gmask := $00ff0000;
    bmask := $0000ff00;
    amask := $000000ff;
  end else begin
    rmask := $000000ff;
    gmask := $0000ff00;
    bmask := $00ff0000;
    amask := $ff000000;
  end;

  tempSurface := SDL_CreateRGBSurfaceFrom(fData, Width, Width, 32, Width * 4, rMask, gMask, bMask, aMask);
  try
    fFont := SDL_DisplayFormatAlpha(tempSurface);
  finally
    SDL_FreeSurface(tempSurface);
  end;

  // let's create a variable to hold all the widths of the font
  fWidths := AllocMem(Width);

  // now read in the information about the width of each character
  sFilename := Format('%s/%s', [aFontDir, 'font.dat']);
  fpByte := FileOpen(sFilename, fmOpenRead);
  try
    for iLoop := 0 to Pred(fWidth) do begin
      FileRead(fpByte, fWidths^[iLoop], SizeOf(fWidths[iLoop]));
    end;
  finally
    FileClose(fpByte);
  end;
end;

constructor TSDLFont.Create(const aFontdir : String; r, g, b : Single);
begin
  Create(aFontDir, r, g, b, 0.0);
end;

constructor TSDLFont.Create(const aFontdir : String);
begin
  Create(aFontDir, 1.0, 1.0, 1.0, 0.0);
end;

destructor TSDLFont.Destroy;
begin
  SDL_FreeSurface(fFont);
  FreeMem(fData);
  FreeMem(fWidths);

  inherited Destroy;
end;

// this function draws one part of an image to some other part of an other image
// it's only to be used inside the font.cpp file, so it's not available to any
// other source files (no prototype in font.h)
procedure TSDLFont.fontDrawIMG(Screen, Img : PSDL_Surface; x, y, w, h, x2, y2 : Integer);
var
  src              : TSDL_Rect;
  dest             : TSDL_Rect;
begin
  dest.x := x;
  dest.y := y;
  src.x  := x2;
  src.y  := y2;
  src.w  := w;
  src.h  := h;
  SDL_BlitSurface(Img, @Src, Screen, @Dest);
end; 

// here we draw the string
procedure TSDLFont.TextOut(aScreen : PSDL_Surface; x, y : Integer; const aText : String);
var
  cBuffer                    : array[0..1024] of Char;
  iLen                       : Integer;
  iXX                        : Integer;
  iLoop                      : Integer;
  iWidth                     : Integer;
  iXPos, iYPos               : Integer;
begin
  StrPLCopy(cBuffer, aText, Pred(SizeOf(cBuffer)));
  ilen := StrLen(cBuffer); // Get the number of chars in the string
  ixx  := 0;               // This will hold the place where to draw the next char
  
  // Loop through all the chars in the string
  for iLoop := 0 to Pred(iLen) do begin
    // This may look scary, but it's really not.
    // We only draw one character with this code.
    // At the next run of the loop we draw the next character.

    // Remember, the fontDrawIMG function looks like this:
  	// void fontDrawIMG(SDL_Surface *screen, SDL_Surface *img, int x, int y,
    //                                         int w, int h, int x2, int y2)

    iWidth := fWidths[Ord(cBuffer[iLoop])];
    iXPos  := (Ord(cBuffer[iLoop]) mod fcharWidth);
    iYPos  := (Ord(cBuffer[iLoop]) div fcharWidth);
    // We draw onto the screen SDL_Surface a part of the font SDL_Surface.
    fontDrawIMG(
      aScreen,
      fFont,
      // We draw the char at pos [x+xx,y].
      // x+xx: this function's parameter x + the width of all the characters before
      // this one, so we wouldn't overlap any of the previous characters in the string
      // y: this function's y parameter
      iXX + x,
      y,
      // For the width of the to-be-drawn character we take it's real width + 2
      iWidth + 2,
      // And for the height we take the height of the character (height of the font/16)
      fcharWidth,

      // Now comes the tricky part
      // The font image DOES consist of 16x16 grid of characters. From left to
      // right in the image, the ascii values of the characters increase:
      // The character at block 0x0 in the font image is the character with the
      // ascii code 0, the character at the pos 1x0 has the ascii code 1, the char
      // at the pos 15x0 has the ascii code 15. And that's the end of the first row
      // Now in the second row on the image, the first character (0x1) has the ascii
      // value 16, the fourth character on the second row of the image (3x1) has the ascii
      // value 19. To calculate the ascii value of the character 3x1, we use the
      // really simple equation: row*[number of thing in one row (=16)]+column pos.
      // So the position 3x1 has the ascii value 1*16+3 = 19. The character in the
      // image at the position 8x12 has the ascii value 12*16+8=200, and so on.
      // But this isn't much of use to us since we KNOW the ascii value of a character,
      // but we NEED to find out it's position on the image.
      // First we'll get the column on the image. For that we'll divide the ascii value
      // with 16 (the number of columns) and get it's remainder (we'll use the modulus
      // operator). We'll do this equation to get the column: [ascii value]%16.
      // Now to get to the position of the column in pixels, we multiply the ascii
      // value by the width of one column ([font image width]/16)
      // And so the equation to get the first pixel of the block becomes:
      // [font image width]%16*[1/16th of the font image width]
      // Now, since all the letters are centered in each cell (1/16th of the image),
      // we need to get the center of the cell. This is done by adding half the width
      // of the cell to the number we got before. And the only thing left to do is to
      // subtract half of the character's real width and we get the x position from where
      // to draw our character on the font map :)
      iXPos * fcharWidth + (fcharWidth div 2) - (iWidth div 2),
      // To get the row of the character in the image, we divide the ascii value of
      // the character by 16 and get rid of all the numbers after the point (.)
      // (if we get the number 7.125257.., we remove the .125157... and end up with 7)
      // We then multiply the result with the height of one cell and voila - we get
      // the y position!
      iYPos * fcharWidth
    );

    // Now we increase the xx printed string width counter by the width of the
    // drawn character
    iXX := iXX + iWidth;
  end;
end; 

// This function returns the width of a string
function TSDLFont.TextWidth(const aText : String) : Integer;
var
  cBuffer          : array[0..1024] of Char;
  iLen             : Integer;
  iLoop            : Integer;
begin
  StrPLCopy(cBuffer, aText, Pred(SizeOf(cBuffer)));

  // Now we just count up the width of all the characters
  Result := 0;
  iLen   := StrLen(cBuffer);
  for iLoop := 0 to Pred(iLen) do begin
    // Add their widths together
    Result := Result + fWidths[Ord(cBuffer[iLoop])];
  end;
end;

end.
