unit uMain;
(*
  This is a extened version of a converted sample of a c++ SDL example.
  Part of code is taken from Marius Andra and the credits for the sprites
  go to Dann / Jotaroh and for the music to seablue.

  lesson6.cpp
  Cone3D GFX with SDL lesson 6.
  Made by Marius Andra 2001
  http://cone3d.gamedev.net

  You can use the code for anything you like.
  Even in a commercial project.
  But please let me know where it ends up.
  I'm just curious. That's all.
*)
{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}
interface

uses SysUtils,

     SDL,
     SDL_Mixer,
     Logger,

     uSprite,
     uFonts,
     uGame;

const
  cBULLETS         = 10;
  cENEMIES         = 10;

type
  TGameMain        = class(TGameApp)
  protected
    fBack            : PSDL_Surface;     // Showing mars surface
    fLogo            : PSDL_Surface;     // logo on screen top
    fSmallShip       : PSDL_Surface;     // bitmap for displaying lives
    fGameOverImg     : PSDL_Surface;     // GameOver screen
    fJoystickSens    : Double;
    fPauseTime       : Cardinal;

    fShipBase        : TSpriteBase;      // ship bitmaps
    fShip            : TSprite;          // Stores the ship

    fBulletBase      : TSpriteBase;      // bullet graphic data 
    fBullets         : array[0..(cBULLETS - 1)] of TSprite;
    fBulletTime      : Cardinal;         // fire frequenz of ship, mean 250ms between to shots
                                         // Shows which elements of the bullets
                                         // array are in use
    fExplosion1Base  : TSpriteBase;
    fExplosions      : array[0..(cENEMIES - 1)] of TSprite;
    fExplosionsSpeed : array[0..(cENEMIES - 1)] of Integer; // Speeds of each enemy
    fExplosionsLife  : array[0..(cENEMIES - 1)] of Integer;

    fExplosion2Base  : TSpriteBase;
    fExplosion2      : TSprite;

    fEnemyBase       : TSpriteBase;
    fEnemies         : array[0..(cENEMIES - 1)] of TSprite;
    felife           : array[0..(cENEMIES - 1)] of Integer; // Life of each enemy
    feinfo           : array[0..(cENEMIES - 1)] of Integer; // The type of enemy - bouncing or not

    fScroll          : Double;           // How much has the screen scrolled

    fLastEnemy       : Integer;          // Remembers when the last enemy came out
    fAliveTime       : Integer;          // Remembers when you were resurrected

    fscore           : Integer;          // The score and the lives
    fLives           : Integer;

    fShot            : TEffectData;      // This will store our shot sound
    fExplode1        : TEffectData;      // This will store our enemy "explosion" sound
    fExplode2        : TEffectData;      // This will store our ship "explosion" sound

    fFont            : TSDLFont;
    fYellowFont      : TSDLFont;

    function  Sprite_Collide(object1, object2 : TSprite) : Boolean;
    procedure ShipExplosionDone(aSprite : TSprite; aState : TAnimState);

    procedure DrawScroll;
    procedure ResetGame(aFirstCall : Boolean = False);
    procedure PauseKeyPressed;
    procedure AddBullet;
    procedure AddEnemy;
    procedure AddEnemyExplosion(aSprite : TSprite);
    procedure AddShipExplosion(aSprite : TSprite);
    procedure ClearObjects;
  public
    constructor Create(const aCaption : String);
    destructor Destroy; override;

    procedure GameInit; override;
    function  HandleEvents(var aEvent : TSDL_Event) : Boolean; override;
    procedure BeforeDrawScene; override;
    procedure DrawScene; override;
    procedure AfterDrawScene; override;
    procedure GameShutdown; override;
  end;

implementation

function TGameMain.Sprite_Collide(object1, object2 : TSprite) : Boolean;
begin
  Result := object1.SpriteHitPixel(object2);
end;

procedure TGameMain.ShipExplosionDone(aSprite : TSprite; aState : TAnimState);
begin
  if (aState = asAnimStop) then begin
    // stop animation
    fExplosion2.StopAnim;
    fExplosion2.Visible := False;
    
    // decrease our lives
    Dec(fLives);
    // if we are dead then
    if (fLives < 1) then begin
      ClearObjects;

      // mark the game as over
      SwitchToGameOver;

      // stop moving/checking the enemies,
      // eg: get out of this loop
      Exit;
    end;

    // Mark the time when we died
    fAliveTime := Trunc(GameTime);

    // If we didn't completely die then reset our position
    fShip.setXY(10, 245);

    // Make the ship blink (animate)
    fShip.AnimSpeed := 1;
    fShip.Visible   := True;
    fShip.setXY(10, 245);
    fShip.startAnim();
  end;
end;

// This function draws the scrolling background
procedure TGameMain.DrawScroll;
begin
  // We first check, if the background has scrolled more than it's own
  // width. And if so, then we must draw 2 images on the screen:
  // the end of one scrolling background and the start of another.
  if (fScroll > (3909 - 640)) then begin
    DrawIMG(fBack, 0, 60, Trunc(3909 - fScroll), 400, Trunc(fScroll), 0);
    DrawIMG(fBack, Trunc(3909 - fScroll - 1),60, Trunc(640 - (3909-fScroll)), 400, 0, 0);
  end else begin
    // If not then we draw only one background
    DrawIMG(fBack, 0, 60, 640, 400, Trunc(fScroll), 0);
  end;
end;

procedure TGameMain.ResetGame(aFirstCall : Boolean = False);
var
  iLoop            : Integer;
begin
  // Reset all the bullets
  for iLoop := 0 to Pred(High(fBullets)) do begin
    fBullets[iLoop].Visible := False;
  end;

  // and the enemies
  for iLoop := 0 to Pred(High(fEnemies)) do begin
    fEnemies[iLoop].Visible := False;
  end;

  fShip.Visible := True;
  fShip.setXY(50, 220);

  if aFirstCall then begin
    fShip.AnimSpeed := 0;
    fShip.Frame     := 0;
    fShip.stopAnim();
  end else begin
    // And make it blink
    fShip.AnimSpeed := 1;
    fShip.Frame     := 0;
    fShip.startAnim();
  end;

  // Reset the lives and score
  fLives      := 5;
  fScore      := 0;

  // Mark the time of the resurrection
  fAliveTime  := Trunc(GameTime);
  fBullettime := SDL_GetTicks();
end;

procedure TGameMain.PauseKeyPressed;
var
  iTime            : Cardinal;
  iDeath           : Cardinal;
begin
  iDeath := fPauseTime + 250;
  iTime  := SDL_GetTicks();
  if (iDeath > iTime) then begin
    Exit;
  end;
  fPauseTime := iTime;
  if IsState(gsPaused) then begin
    SwitchToPlaying;
  end else begin
    SwitchToPaused;
  end;
end;

procedure TGameMain.AddBullet;
var
  iLoop            : Integer;
  iTime            : Cardinal;
  iDeath           : Cardinal;
begin
  iDeath := fBullettime + 250;
  iTime  := SDL_GetTicks();
  if (iDeath > iTime) then begin
    Exit;
  end;
  fBullettime := iTime;

  // We loop through all the spots for bullets
  for iLoop := 0 to Pred(cBULLETS) do begin
    // If we find an empty spot
    if not fBullets[iLoop].Visible then begin
      // Set the bullet's position
      fbullets[iLoop].setXY(fShip.X + 70, fShip.Y + 23);
      Mix_PlayChannel(0, fShot.Effect, 0);

      // and mark the spot
      fBullets[iLoop].Visible := True;

      Exit;
    end;
  end;
end;

procedure TGameMain.AddEnemy;
var
  iLoop            : Integer;
begin
  // We loop through all the spots for enemies
  for iLoop := 0 to Pred(High(fEnemies)) do begin
    // If the spot is free
    if not fEnemies[iLoop].Visible then begin
      // we stop the animation on the enemy sprite for safety
      // (so that it wouldn's start blinking blue and purple all the time)
      fEnemies[iLoop].stopAnim;

      // We give the enemy a random y coordinate so that each enemy
      // pops up at a different location...
      fEnemies[iLoop].setXY(640, 60 + Random(350));

      // We also give the enemy random speed and life values
      fEnemies[iLoop].MoveSpeed := Random(4) + 1;
      felife[iLoop]   := Random(3) + 1;

      // Make one enemy out of two
      if (felife[iLoop] > 1) then begin
        // blue (frame 1 in the sprite animation)
        fEnemies[iLoop].Frame := 1;

        // mark him as blue (moving up and down)
        feinfo[iLoop] := Random(300) + 1;
      end else begin
        // and the others purple (frame 0 in the sprite animation)
        fEnemies[iLoop].Frame := 0;
        // mark him as purple
        feinfo[iLoop] := 0;
      end;

      // fill in the spot
      fEnemies[iLoop].Visible := True;
      Exit;
    end;
  end;
end;

procedure TGameMain.AddEnemyExplosion(aSprite : TSprite);
var
  iLoop            : Integer;
begin
  // We loop through all the spots for enemies
  for iLoop := 0 to Pred(High(fExplosionsLife)) do begin
    if (fExplosionsLife[iLoop] <= 0) then begin
      // If the spot is free
      fExplosionsLife[iLoop]       := 600;
      fExplosions[iLoop].Visible   := True;
      fExplosions[iLoop].Frame     := 0;
      fExplosions[iLoop].AnimSpeed := 2;
      fExplosions[iLoop].MoveSpeed := aSprite.MoveSpeed;
      fExplosions[iLoop].SetCenter(aSprite.CenterPos);
      fExplosions[iLoop].StartAnim;
      Exit;
    end;
  end;
end;

procedure TGameMain.AddShipExplosion(aSprite : TSprite);
begin
  // Zeige die Explostion an
  fExplosion2.Visible   := True;
  fExplosion2.AnimSpeed := 3;
  fExplosion2.OnAnimStateChange := ShipExplosionDone;
  fExplosion2.SetCenter(aSprite.CenterPos);
  fExplosion2.StartAnim;

  // play some spooky sound
  Mix_PlayChannel(1, fExplode2.Effect, 0);
end;

procedure TGameMain.ClearObjects;
var
  iLoop            : Integer;
begin
  for iLoop := 0 to Pred(High(fBullets)) do begin
    fBullets[iLoop].Visible := False;
  end;

  for iLoop := 0 to Pred(High(fEnemies)) do begin
    fEnemies[iLoop].Visible := False;
  end;
end;

constructor TGameMain.Create(const aCaption : String);
begin
  inherited Create(aCaption);

  fShipbase       := TSpriteBase.Create;
  fBulletBase     := TSpriteBase.Create;
  fEnemyBase      := TSpriteBase.Create;
  fExplosion1Base := TSpriteBase.Create;
  fExplosion2Base := TSpriteBase.Create;
  fShot           := nil;
  fExplode1       := nil;
  fExplode2       := nil;
  fPauseTime      := 0;
  fJoystickSens   := 3.0;

  fBullettime     := SDL_GetTicks();
  fAliveTime      := 0;
  fLastEnemy      := 0;
  fScroll         := 0;

  fScore          := 0;
  fLives          := 5;
  Randomize;
end;

destructor TGameMain.Destroy;
var
  iLoop            : Integer;
begin
  FreeAndNil(fFont);
  FreeAndNil(fYellowFont);

  // destroy enemies
  for iLoop := 0 to Pred(High(fEnemies)) do begin
    fEnemies[iLoop].Free;
  end;
  FreeAndNil(fEnemyBase);

  // destroy explosion 1
  for iLoop := 0 to Pred(High(fExplosions)) do begin
    fExplosions[iLoop].Free;
  end;
  FreeAndNil(fExplosion1Base);

  // destroy bulltes
  for iLoop := 0 to Pred(High(fBullets)) do begin
    fBullets[iLoop].Free;
  end;
  FreeAndNil(fBulletBase);

  // destroy player
  FreeAndNil(fShip);
  FreeAndNil(fShipbase);

  // destroy players explosion
  FreeAndNil(fExplosion2);
  FreeAndNil(fExplosion2Base);

  inherited Destroy;
end;

// Loading all game files
procedure TGameMain.GameInit;
var
  iLoop            : Integer;
begin
  inherited GameInit;

  // Joysticks.UseEventNotify := False;
  if (Joysticks.Count > 0) then begin
    Joysticks[0].Open;
  end;

  Log.LogStatus('Loading sound data', 'uMain');
  BackgroundMusic := LoadMusic('data/sound/(sblu)moon6.xm');
  fShot           := LoadSound('data/sound/shot.wav');
  fExplode1       := LoadSound('data/sound/explode1.wav');
  fExplode2       := LoadSound('data/sound/explode2.wav');

  // We also load in all the graphics...
  Log.LogStatus('Loading graphic data', 'uMain');
  fBack        := LoadImage('data/mars.jpg');
  fLogo        := LoadImage('data/logo.bmp');
  fSmallShip   := LoadImage('data/smallship.bmp');
  fGameOverImg := LoadImage('data/gameover.bmp');

  // And make the game over image transparent
  SDL_SetColorKey(fGameOverImg, SDL_SRCCOLORKEY, SDL_MapRGB(fGameOverImg.Format, 0, 255, 0));

  // players ship
  fShipbase.Init('data/ship');
  fShip := LoadSprite(fShipbase, 50, 220, 1);

  // explosion of ship
  fExplosion2Base.Init('data/explosion2');
  fExplosion2 := LoadSprite(fExplosion2base);

  // enemies data
  fEnemyBase.Init('data/enemy');
  for iLoop := 0 to Pred(High(fEnemies)) do begin
    fEnemies[iLoop] := LoadSprite(fEnemyBase);
  end;

  // enemies explosions
  fExplosion1Base.Init('data/explosion1');
  for iLoop := 0 to Pred(High(fExplosions)) do begin
    fExplosions[iLoop] := LoadSprite(fExplosion1Base);
  end;

  // bullets data
  fBulletBase.Init('data/bullet');
  for iLoop := 0 to Pred(High(fBullets)) do begin
    fBullets[iLoop] := LoadSprite(fBulletBase);
  end;

  // Now we load in the fonts - a white one and a yellow one
  Log.LogStatus('Loading font data', 'uMain');
  fFont       := TSDLFont.Create('data/font');
  fYellowFont := TSDLFont.Create('data/font', 1, 1, 0);

  ResetGame(True);

  Log.LogStatus('Init done', 'uMain');
end;

function TGameMain.HandleEvents(var aEvent : TSDL_Event) : Boolean;

  function JoystickButtonPressed : Boolean;
  begin
    Result := False;
    if (Joysticks.Count > 0) then begin
      Result := Joysticks[0].ButtonPressed[0];
    end;
  end;


begin
  Result := inherited HandleEvents(aEvent);
  if not Result then begin

    if (aEvent.Type_ = SDL_KEYDOWN) then begin
      if (aEvent.Key.KeySym.Sym = SDLK_P) then begin
        PauseKeyPressed;
        Result  := True;
      end;

      // If the user pressed ENTER and the game is over then
      if IsState(gsGameOver) then begin
        if (aEvent.Key.KeySym.Sym = SDLK_RETURN) then begin
          SwitchToPlaying;
          ResetGame;
        end;
      end;
    end else if (aEvent.Type_ = SDL_JOYBUTTONDOWN) then begin
      if (aEvent.button.button = 0) then begin
        if IsState(gsGameOver) then begin
          SwitchToPlaying;
          ResetGame;
        end;
      end else if (aEvent.button.button = 1) then begin
        if IsState([gsPlaying, gsPaused]) then begin
          PauseKeyPressed;
          Result := True;
        end;
      end;
    end;
  end;
end;

procedure TGameMain.BeforeDrawScene;

  function GetJoystickX : Double;
  begin
    Result := 0;
    if (Joysticks.Count > 0) then begin
      Result := Joysticks[0].AxisX;
    end;
  end;

  function GetJoystickY : Double;
  begin
    Result := 0;
    if (Joysticks.Count > 0) then begin
      Result := Joysticks[0].AxisY;
    end;
  end;

  function JoystickButtonPressed : Boolean;
  begin
    Result := False;
    if (Joysticks.Count > 0) then begin
      Result := Joysticks[0].ButtonPressed[0];
    end;
  end;

var
  iLoop, iJ        : Integer;
  uDT              : Double;
begin
  // uDT is used delta time between two frames
  uDT  := FrameTime * 0.1;

  if IsState(gsPlaying) then begin
    // If the game isn't paused nor over
    if (Keys[SDLK_SPACE] <> 0) or JoystickButtonPressed
    then begin
      // Then add a bullet
      addBullet;
    end;

    // If the keys UP, DOWN, LEFT and RIGHT have been pressed
    if (Keys[SDLK_UP] <> 0) or (GetJoystickY < -0.05) then begin
      // Move the ship up
      if (Keys[SDLK_UP] <> 0) then begin
        fShip.YAdd(Trunc(-2 * uDT));
      end else begin
        fShip.YAdd(Trunc(fJoystickSens * uDT * GetJoystickY));
      end;

      // Prevent the ship from going outside the screen
      if (fShip.Y < 60) then fShip.Y := 60;
    end;

    if (Keys[SDLK_DOWN] <> 0) or (GetJoystickY > 0.05) then begin
      // Move the ship up
      if (Keys[SDLK_DOWN] <> 0) then begin
        fShip.YAdd(Trunc(2 * uDT));
      end else begin
        fShip.YAdd(Trunc(fJoystickSens * uDT * GetJoystickY));
      end;

      // Prevent the ship from going outside the screen
      if (fShip.Y > 410) then fShip.Y := 410;
    end;

    if (Keys[SDLK_LEFT] <> 0) or (GetJoystickX < -0.05) then begin
      // Move the ship up
      if (Keys[SDLK_LEFT] <> 0) then begin
        fShip.XAdd(Trunc(-2 * uDT));
      end else begin
        fShip.XAdd(Trunc(fJoystickSens * uDT * GetJoystickX));
      end;

      // Prevent the ship from going outside the screen
      if (fShip.X < 0) then fShip.X := 0;
    end;

    if (Keys[SDLK_RIGHT] <> 0) or (GetJoystickX > 0.05) then begin
      // Move the ship up
      if (Keys[SDLK_RIGHT] <> 0) then begin
        fShip.XAdd(Trunc(2 * uDT));
      end else begin
        fShip.XAdd(Trunc(fJoystickSens * uDT * GetJoystickX));
      end;

      // Prevent the ship from going outside the screen
      if (fShip.X > 570) then fShip.X := 570;
    end;

    // Loop through all the bullets
    for iLoop := 0 to Pred(High(fBullets)) do begin
      // If they are visible
      if fBullets[iLoop].Visible then begin
        // then move them 3.5*dt units right
        fBullets[iLoop].XAdd(3.5 * uDT);

        // If they are outside the screen area
        if(fBullets[iLoop].X > 640) then begin
          // hide them
          fBullets[iLoop].Visible := False;
        end;
      end;
    end;

    for iLoop := 0 to Pred(High(fExplosionsLife)) do begin
      if (fExplosionsLife[iLoop] <= 0) then Continue;
      fExplosions[iLoop].XAdd(-(fExplosions[iLoop].MoveSpeed + 1) * uDT * 0.25);
      fExplosionsLife[iLoop] := fExplosionsLife[iLoop] - FrameTime;
      if (fExplosionsLife[iLoop] < 0) then begin
        fExplosionsLife[iLoop] := 0;
      end;
    end;

    // Loop through all the enemies
    for iLoop := 0 to Pred(High(fEnemies)) do begin
      // If they are visible
      if fEnemies[iLoop].Visible then begin
        // Move them left (their speed+1)*0.5*dt units
        fEnemies[iLoop].XAdd(-(fEnemies[iLoop].MoveSpeed + 1) * uDT * 0.25);
        // If the sprites are colored blue then
        if (feinfo[iLoop] > 0) then begin
          // Adjust their y coordinate. I wanted smooth motion so
          // we make their y position equal the cosinus of
          // their x coordiante+some random number that prevents
          // everyone from following the same path
          // Since we have to convert the degrees into radians, we
          // multiply the value with 0.0174532925 (pi/180)
          // The cosinus function returs a value from -1 to +1, so we
          // multiply it with 100 to get some reasonable value
          // We also add 230 to the end result to center everything
          // on the screen
          fEnemies[iLoop].Y := ((cos((fEnemies[iLoop].X + feinfo[iLoop])*0.0174532925) *100)+230);
        end;
      
        // If the enemy moves out of the screen
        if (fEnemies[iLoop].X < -50) then begin
          // Get rid of it
          fEnemies[iLoop].Visible := False;
        
          // and then move on to the next enemy
          Continue;
        end;

        if IsState(gsPlaying) then begin
          // Check for bullet collisions
          for iJ := 0 to Pred(High(fBullets)) do begin
            // If we have a bullet at that spot
            if fBullets[iJ].Visible then begin
              // Check if it collides with the enemy
              if Sprite_Collide(fBullets[iJ], fEnemies[iLoop]) then begin
                // If so then decrease the enemies life
                felife[iLoop] := felife[iLoop] - 1;

                // If the enemy is dead
                if (felife[iLoop] < 1) then begin
                  // Then don't draw it anymore
                  fEnemies[iLoop].Visible := False;
                  // Increase the players score by 10
                  fScore := fScore + 10;

                  // play some spooky sound
                  Mix_PlayChannel(1, fExplode1.Effect, 0);
                  // Get rid of the bullet
                  fBullets[iJ].Visible := False;

                  AddEnemyExplosion(fEnemies[iLoop]);

                  // move on to the next enemy
                  Continue;
                end else begin
                  if (felife[iLoop] = 1) then begin
                    fEnemies[iLoop].Frame := 0;
                  end;

                  // If we only injured the enemy then increase
                  // the score by one
                  Inc(fScore);
                  // Get rid of the bullet
                  fBullets[iJ].Visible := False;
                end;
              end;
            end;
          end;

          // Check for player collisions
          // If the ship is vulnearable to attack - is not blinking
          // And the enemy collided with the ship then
          if (fShip.AnimSpeed = 0) and Sprite_Collide(fShip, fEnemies[iLoop])
          then begin
            fShip.Visible := False;

            // Get rid of the enemy
            fEnemies[iLoop].Visible := False;

            // Show explosion of ship
            AddShipExplosion(fShip);

            Break;
          end;
        end;
      end;
    end;

    // If 3 seconds have passed since we died/were resurrected then
    if ((GameTime - fAliveTime) > 3000) then begin
      // Stop the ships blinking animation
      fShip.StopAnim();
      fShip.AnimSpeed := 0;

      // Show the frame of animation with the ship on it
      fShip.Frame := 0;
    end;


    // If 3/4 seconds have passed since the last enemy came out
    if ((GameTime - fLastEnemy) > 750) then begin
      // Add a new enemy
      addEnemy;

      // Mark the time when this enemy came out
      fLastEnemy := Trunc(GameTime);
    end;
  end;

  if not IsState(gsPaused) then begin
    // Scroll the screen by dt units
    fScroll := fScroll + uDT;

    // If we have scrolled more than the background image then
    // decrease the scrolled amount by the width of the background image
    if (fScroll >= 3909) then begin
      fScroll := fScroll - 3909;
    end;
  end;
end;

procedure TGameMain.DrawScene;
var
  iLoop            : Integer;
begin
  // We first fill the entire screen with black
  SDL_FillRect(Screen, nil, 0);

  // Draw the scroll
  DrawScroll();

  // Now we draw some logo on the screen
  DrawIMG(fLogo, 0, 0);

  // If we are alive
  if (fLives > 0) then begin
    // Then draw a small ship for each life that we got
    for iLoop := 0 to Pred(fLives) do begin
      DrawIMG(fSmallShip, 605 - iLoop * 30, 36);
    end;
  end;

  // Draw the score
  fFont.TextOut(Screen, 5, 37, 'Score:');
  fYellowFont.TextOut(fscreen, 50, 37, IntToStr(fScore));

  // and credits
  fFont.TextOut(fScreen, 5, 460, 'Code by              Sprites by                           Music by');
  fYellowFont.TextOut(fScreen, 63, 460, 'Marius                   Dann / Jotaroh                  seablue');
  fYellowFont.TextOut(fScreen, 465, 460, 'http://cone3d.gamedev.net');

  // Loop thorough all the spots for bullets
  for iLoop := 0 to Pred(High(fBullets)) do begin
    if fBullets[iLoop].Visible then fBullets[iLoop].Draw;
  end;

  // Loop thorough all the spots for enemies
  for iLoop := 0 to Pred(High(fEnemies)) do begin
    if fEnemies[iLoop].Visible then fEnemies[iLoop].Draw();
  end;

  // Loop thorough all the spots for enemies
  for iLoop := 0 to Pred(High(fExplosionsLife)) do begin
    if (fExplosionsLife[iloop] > 0) then fExplosions[iLoop].Draw();
  end;

  if fExplosion2.Visible then begin
    // Draw ship explostion
    fExplosion2.Draw;
  end else begin
    // Draw the main ship
    fShip.Draw;
  end;

  // If the game is over, post a little message
  if IsState(gsGameOver) then begin
    DrawIMG(fGameOverImg, 192, 200);
  end;
  
  // Filp the screen..
  SDL_Flip(screen);
end;

procedure TGameMain.AfterDrawScene;
begin
  SDL_Delay(1);
end;

procedure TGameMain.GameShutdown;
begin
  // Fade out all the audio channels in 1 second
  Mix_FadeOutChannel(-1, 1000);

  // and the music in 1 second
  Mix_FadeOutMusic(1000);

  // wait one second
  SDL_Delay(1000);
end;

end.

