program template_sdl;

{$IFDEF FPC}
  {$LINKLIB c}
  {$MODE DELPHI}
  {$H+}
{$ELSE}
  {$APPTYPE Console}
{$ENDIF}


{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Florian Sievert (Phobeus)
  Phobeus@DelphiGL.com
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2005 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  Documetation on SDL_mixer is poor. However you will find it pretty is to use, if you get the first lines running.
  hopefull this small sample will help to find a good start.

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2005-09-01 Added surface flip contributed by i0n0s & Icon (FS)
  2005-05-19 Some minor improvements (FS)
  2005-04-27 Added support for dglheader (FS)
  2004-07-23 Added texture loader sample (FS)
  2004-02-20 Created (FS)


  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 

  FS
    Florian Sievert (Phobeus)
    email : Phobeus@DelphiGL.com
    Visit: http://www.delphigl.com

*******************************************************************************)

uses
  //CRT,
  SysUtils,

  SDL,
  SDL_Image,
  SDLutils,
  dglOpenGL;

const
  WINDOW_CAPTION		=       'http://www.delphigl.com/ .:. SDL-Template'; 
  WINDOW_RESIZE		=       FALSE;  // allow window resize?
  FULLSCREEN			=       FALSE;  // use fullscreen

  SCREEN_WIDTH			= 640;  // width of screen
  SCREEN_HEIGHT		= 480;  // height of screen
  SCREEN_BPP			= 32;   // bit per pixel

  FPS_INTERVAL		= 1000;

var
  surface : PSDL_Surface;

  texture : gluInt;

  videoInfo   : TSDL_VideoInfo;
  videoflags  : Uint32;

  done        : Integer;

  FPSCount    : LongInt;


procedure Quit_App;
begin
  // release ressources
  Writeln('Release Ressources...');

  if (FULLSCREEN) then SDL_WM_ToggleFullScreen(surface);
  SDL_FreeSurface (surface) ;
  SDL_Quit;

  WriteLn('Bye Bye ;)');
  Halt(0);
end;

//----------------------------------------------------------------------------
// draw the scene
//----------------------------------------------------------------------------
procedure glDrawScene;
begin
  // clear color buffer
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );

  glLoadIdentity;
  glTranslatef(0,0,-2);

  glBegin(GL_QUADS);
    glTexCoord2f(0,0); glVertex3f(-0.5,-0.5,0.0);
    glTexCoord2f(1,0); glVertex3f( 0.5,-0.5,0.0);
    glTexCoord2f(1,1); glVertex3f( 0.5, 0.3,0.0);
    glTexCoord2f(0,1); glVertex3f(-0.5, 0.5,0.0);
  glEnd();
end;

//----------------------------------------------------------------------------
// handle resize event
//----------------------------------------------------------------------------
function glResizeWindow(width: integer; height: integer): boolean;
begin
  // catch a division by zero
  if ( height = 0 ) then height := 1;

  // refresh viewport and projection matrix
  glViewport( 0, 0, width, height );

  glMatrixMode( GL_PROJECTION );
    glLoadIdentity;
    gluPerspective( 45.0, width / height, 0.1, 100.0 );
  glMatrixMode( GL_MODELVIEW );

  // reset world matrix
  glLoadIdentity;

  // successfull
  result := true;
end;

//----------------------------------------------------------------------------
// handle timer event
//----------------------------------------------------------------------------
function glTimer(interval: UInt32; param: pointer): UInt32;
begin
  SDL_WM_SetCaption(pchar(WINDOW_CAPTION+' ['+IntToStr(FPSCount * 1000 DIV FPS_INTERVAL)+' FPS]'), nil);
  FPSCount := 0;
  Result := interval;
end;

//----------------------------------------------------------------------------
// handle keyboard inpu
//----------------------------------------------------------------------------
procedure glHandleKeyPress(keysym: TSDL_keysym);
begin
  case keysym.sym of
    SDLK_ESCAPE : done := -1;
  end;
end;

//----------------------------------------------------------------------------
// handle sdl events
//----------------------------------------------------------------------------
procedure glHandleEvents;
var event : TSDL_Event;
begin
  // process events
  while (SDL_PollEvent(@event) = 1) do
  begin
    case event.type_ of
      // quit signal
      SDL_QUITEV :
      begin
        Done := -1;
      end;

      // key was pressed
      SDL_KEYDOWN :
      begin
        glHandleKeyPress( event.key.keysym );
      end;

      // window was resized
      SDL_VIDEORESIZE :
      begin
        surface := SDL_SetVideoMode(event.resize.w,event.resize.h,SCREEN_BPP,videoflags);

        if ( surface = nil ) then
        begin
          Writeln('surface lost');
          Quit_App;
        end;
        glResizeWindow(event.resize.w,event.resize.h);
      end;
    end;
  end;
end;

procedure Init_SDL;
begin
  // Init SDL
  WriteLn('Init SDL...');

  if ( SDL_Init( SDL_INIT_VIDEO ) < 0 ) then
  begin
    WriteLn('Failed to init SDL: '+SDL_GetError);
    Quit_App;
  end;

  // get information about graphic card
  videoInfo := SDL_GetVideoInfo^;
  if (@videoInfo = nil) then
  begin
    WriteLn('Unable to detect graphic card');
    Quit_App;
  end;

  // set sdl flags: we want a opengl surface
  videoFlags := SDL_OPENGL or SDL_DOUBLEBUF or SDL_HWPALETTE;

  if (videoInfo.hw_available <> 0) then
  begin
    WriteLn('Hardware Surface enabled...');
    videoFlags := videoFlags or SDL_HWSURFACE;
  end
  else
  begin
    WriteLn('Software Surface enabled...');
    videoFlags := videoFlags or SDL_SWSURFACE
  end;

  // hardware blit?
  if (videoInfo.blit_hw <> 0) then
  begin
    WriteLn('Hardware blit');
    videoFlags := videoFlags or SDL_HWACCEL;
  end else WriteLn('Software blit');

  // set opengl attributes
  SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

  if (WINDOW_RESIZE) then videoflags := videoFlags or SDL_RESIZABLE;    // Enable window resizing

  // set window propterties
  SDL_WM_SetCaption(WINDOW_CAPTION, nil);
  SDL_WM_SetIcon(IMG_Load('../data/gfx/dgl_icon.png'), 0); 

  // Init a surface
  surface := SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, videoflags);
  if not assigned(surface) then
  begin
    WriteLn('Failed to init surface: '+SDL_GetError);
    Quit_App;
  end;

  // switch to fullscrene?
  if (FULLSCREEN) then SDL_WM_ToggleFullScreen(surface);

  // Init sdl timer
  SDL_Init(SDL_INIT_TIMER);
end;

//----------------------------------------------------------------------------
// Init OpenGL
//----------------------------------------------------------------------------
procedure Init_OpenGL;
begin
  // init opengl
  InitOpenGL;
  ReadExtensions;

  // display some information about the opengl system
  WriteLn('Extensions:  '+glGetString(GL_EXTENSIONS));
  WriteLn('------');
  WriteLn('Renderer: '+glGetString(GL_RENDERER));
  WriteLn('Version: '+glGetString(GL_VERSION));
  WriteLn('Vendor:  '+glGetString(GL_VENDOR));

  // setup some good start values
  glEnable(GL_TEXTURE_2D);									// enable texture mapping
  glShadeModel(GL_SMOOTH);									// enable smooth shading 
  glClearColor(0.0, 0.0, 0.0, 0.5);						// clear color is black
  glClearDepth(1.0);											// clear depth buffer
  glEnable(GL_DEPTH_TEST);									// enable depth testing
  glDepthFunc(GL_LEQUAL);									
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
end;

//----------------------------------------------------------------------------
// help function to flip the texture
//----------------------------------------------------------------------------
procedure surfaceFlip(Surface: PSDL_Surface);
var 
  x,y: Integer;
  a,b: UInt32;
begin
  // we flip the surface
  for y:=0 to ((surface.h-1) div 2) do
    for x:=0 to surface.w-1 do
    begin
      a:=SDL_GetPixel(Surface,x,y);
      b:=SDL_GetPixel(Surface,x,surface.h-1-y);
      SDL_PutPixel(Surface,x,y,b);
      SDL_PutPixel(Surface,x,surface.h-1-y,a);
    end;
end; 

//----------------------------------------------------------------------------
// Init ressources
//----------------------------------------------------------------------------
procedure Init_Resource;
var tex : PSDL_Surface;
begin
  // we load a texture
  tex := IMG_Load('../data/gfx/wiki.jpg');

  if assigned(tex) then
  begin
    surfaceFlip(tex);
    glGenTextures(1, @texture);
    glBindTexture(GL_TEXTURE_2D, texture);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

    glTexImage2D(GL_TEXTURE_2D, 0, 3, tex^.w, tex^.h,0, GL_RGB, GL_UNSIGNED_BYTE, tex^.pixels);

    SDL_FreeSurface(tex);
  end;

  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,texture);
end;



(******************************************************************************)
(******************************************************************************)
//  main program
(******************************************************************************)
(******************************************************************************)
begin
  // Init
  Writeln('Starting...');

  Init_SDL;
  Init_OpenGL;	
  Init_Resource;

  glResizeWindow(SCREEN_WIDTH, SCREEN_HEIGHT);

  // set timer
  SDL_AddTimer(1000,@glTimer,nil);

  Writeln('Start rendering...');

  // enter main loop
  while ( Done <> -1 ) do
  begin
    glHandleEvents;
    glDrawScene;

    // display framebuffer
    SDL_GL_SwapBuffers;
    Inc(FPSCount);
  end;

  // quite application
  Quit_App;
end.
